@@ -26,6 +26,7 @@ gboolean batchConn = FALSE;
 void interrupt(int sign)
 {
 	signal(sign, SIG_IGN);
+	GSM_AbortOperation(gsm);
 	gshutdown = TRUE;
 }
 
@@ -234,7 +235,7 @@ GSM_Error GSM_ReadHTTPFile(const char *url, GSM_File *file)
 	if (dl_handle == NULL) return FALSE;
 
 	curl_easy_setopt(dl_handle, CURLOPT_URL, url);
-	curl_easy_setopt(dl_handle, CURLOPT_USERAGENT, "Gammu/" VERSION);
+	curl_easy_setopt(dl_handle, CURLOPT_USERAGENT, "Gammu/" GAMMU_VERSION);
 	curl_easy_setopt(dl_handle, CURLOPT_WRITEFUNCTION, write_mem);
 	curl_easy_setopt(dl_handle, CURLOPT_WRITEDATA, file);
 	curl_easy_setopt(dl_handle, CURLOPT_FOLLOWLOCATION, 1);
@@ -281,7 +282,7 @@ void GSM_Init(gboolean checkerror)
 		return;
 
 	/* Connect to phone */
-	error = GSM_InitConnection(gsm, 3);
+	error = GSM_InitConnection(gsm, 1);
 	if (checkerror)
 		Print_Error(error);
 
