@@ -4,7 +4,6 @@
 #include <gammu-statemachine.h>
 
 #include "common.h"
-#include "formats.h"
 
 typedef enum {
 	H_Call = 1,
@@ -38,7 +37,7 @@ typedef enum {
 	H_Decode,
 #endif
 	H_Gammu,
-	H_SMSD,
+	H_Obsolete,
 	H_Other
 } HelpCategory;
 
