@@ -28,10 +28,11 @@
 #include "../helper/cmdline.h"
 
 volatile gboolean 			wasincomingsms 		= FALSE;
+volatile int num_replies = 0;
 
 GSM_MultiSMSMessage		IncomingSMSData;
 
-void IncomingSMS(GSM_StateMachine *sm UNUSED, GSM_SMSMessage sms, void *user_data)
+void IncomingSMS(GSM_StateMachine *sm UNUSED, GSM_SMSMessage *sms, void *user_data)
 {
 	printf("%s\n", _("SMS message received"));
 	fflush(stdout);
@@ -42,7 +43,7 @@ void IncomingSMS(GSM_StateMachine *sm UNUSED, GSM_SMSMessage sms, void *user_dat
  		return;
  	}
  	wasincomingsms = TRUE;
- 	memcpy(&IncomingSMSData.SMS[0],&sms,sizeof(GSM_SMSMessage));
+ 	IncomingSMSData.SMS[0] = *sms;
  	IncomingSMSData.Number = 1;
 }
 
@@ -72,19 +73,19 @@ void DisplayIncomingSMS(void)
  	wasincomingsms = FALSE;
 }
 
-void IncomingCB(GSM_StateMachine *sm UNUSED, GSM_CBMessage CB, void *user_data)
+void IncomingCB(GSM_StateMachine *sm UNUSED, GSM_CBMessage *CB, void *user_data)
 {
 	printf("%s\n", _("CB message received"));
-	printf(_("Channel %i, text \"%s\"\n"),CB.Channel,DecodeUnicodeConsole(CB.Text));
+	printf(_("Channel %i, text \"%s\"\n"),CB->Channel,DecodeUnicodeConsole(CB->Text));
 	fflush(stdout);
 }
 
-void IncomingUSSD(GSM_StateMachine *sm UNUSED, GSM_USSDMessage ussd, void *user_data)
+void IncomingUSSD(GSM_StateMachine *sm UNUSED, GSM_USSDMessage *ussd, void *user_data)
 {
 	printf("%s\n", _("USSD received"));
 	printf(LISTFORMAT, _("Status"));
 
-	switch(ussd.Status) {
+	switch(ussd->Status) {
 		case USSD_NoActionNeeded:
 			printf("%s\n", _("No action needed"));
 			break;
@@ -110,20 +111,22 @@ void IncomingUSSD(GSM_StateMachine *sm UNUSED, GSM_USSDMessage ussd, void *user_
 			printf("%s\n", _("Unknown"));
 			break;
 	}
-	printf(LISTFORMAT "\"%s\"\n", _("Service reply"), DecodeUnicodeConsole(ussd.Text));
+	printf(LISTFORMAT "\"%s\"\n", _("Service reply"), DecodeUnicodeConsole(ussd->Text));
 	fflush(stdout);
 }
 
-void IncomingUSSD2(GSM_StateMachine *sm, GSM_USSDMessage ussd, void * user_data)
+void IncomingUSSD2(GSM_StateMachine *sm, GSM_USSDMessage *ussd, void * user_data)
 {
 	IncomingUSSD(sm, ussd, user_data);
-
-	gshutdown = TRUE;
+	num_replies++;
 }
 
 void GetUSSD(int argc UNUSED, char *argv[])
 {
 	GSM_Error error;
+	int last_replies;
+	time_t last_reply;
+
 	GSM_Init(TRUE);
 
 	signal(SIGINT, interrupt);
@@ -142,7 +145,22 @@ void GetUSSD(int argc UNUSED, char *argv[])
 	}
 	Print_Error(error);
 
-	while (!gshutdown) GSM_ReadDevice(gsm, FALSE);
+	num_replies = 0;
+	last_replies = 0;
+	last_reply = time(NULL);
+	while (!gshutdown) {
+		if (num_replies != last_replies) {
+			last_replies = num_replies;
+			last_reply = time(NULL);
+		} else if (num_replies == 0 && difftime(time(NULL), last_reply) > 60) {
+			/* Wait one minute for reply */
+			gshutdown = TRUE;
+		} else if (num_replies > 0 && difftime(time(NULL), last_reply) > 30) {
+			/* Wait for consequent replies for 30 seconds */
+			gshutdown = TRUE;
+		}
+		GSM_ReadDevice(gsm, FALSE);
+	}
 
 	error=GSM_SetIncomingUSSD(gsm, FALSE);
 	Print_Error(error);
@@ -334,8 +352,8 @@ void GetAllSMS(int argc, char *argv[])
 
 #ifdef GSM_ENABLE_BACKUP
 	if (argc == 3 && strcasecmp(argv[2],"-pbk") == 0) {
-		ReadPhonebook(Backup.PhonePhonebook, MEM_ME, NULL, GSM_BACKUP_MAX_PHONEPHONEBOOK);
-		ReadPhonebook(Backup.SIMPhonebook, MEM_SM, NULL, GSM_BACKUP_MAX_SIMPHONEBOOK);
+		ReadPhonebook(Backup.PhonePhonebook, MEM_ME, NULL, GSM_BACKUP_MAX_PHONEPHONEBOOK, NULL, TRUE);
+		ReadPhonebook(Backup.SIMPhonebook, MEM_SM, NULL, GSM_BACKUP_MAX_SIMPHONEBOOK, NULL, TRUE);
 	}
 #endif
 
@@ -403,8 +421,8 @@ void GetEachSMS(int argc, char *argv[])
 
 #ifdef GSM_ENABLE_BACKUP
 	if (argc == 3 && strcasecmp(argv[2],"-pbk") == 0) {
-		ReadPhonebook(Backup.PhonePhonebook, MEM_ME, NULL, GSM_BACKUP_MAX_PHONEPHONEBOOK);
-		ReadPhonebook(Backup.SIMPhonebook, MEM_SM, NULL, GSM_BACKUP_MAX_SIMPHONEBOOK);
+		ReadPhonebook(Backup.PhonePhonebook, MEM_ME, NULL, GSM_BACKUP_MAX_PHONEPHONEBOOK, NULL, TRUE);
+		ReadPhonebook(Backup.SIMPhonebook, MEM_SM, NULL, GSM_BACKUP_MAX_SIMPHONEBOOK, NULL, TRUE);
 	}
 #endif
 
@@ -566,7 +584,8 @@ void SendSaveDisplaySMS(int argc, char *argv[])
 		case SMS_Display:
 			for (i = 0; i < sms.Number; i++) {
 				printf(LISTFORMAT "%i\n", _("Message number"), i);
-				DisplaySMSFrame(&sms.SMS[i], gsm);
+				error = DisplaySMSFrame(&sms.SMS[i], gsm);
+				Print_Error(error);
 			}
 
 			printf(LISTFORMAT "%i\n", _("Number of messages"), sms.Number);
