@@ -100,6 +100,101 @@ void PrintNetworkInfo(GSM_NetworkInfo NetInfo)
 	}
 }
 
+void PrintBatteryCharge(GSM_BatteryCharge *BatteryCharge)
+{
+	if (BatteryCharge->BatteryPercent != -1) {
+		printf(LISTFORMAT, _("Battery level"));
+		printf(_("%i percent"), BatteryCharge->BatteryPercent);
+		printf("\n");
+	}
+	if (BatteryCharge->BatteryCapacity != -1) {
+		printf(LISTFORMAT, _("Battery capacity"));
+		printf(_("%i mAh"), BatteryCharge->BatteryCapacity);
+		printf("\n");
+	}
+	if (BatteryCharge->BatteryTemperature != -1) {
+		printf(LISTFORMAT, _("Battery temperature"));
+		/* l10n: This means degrees Celsius */
+		printf(_("%i C"), BatteryCharge->BatteryTemperature);
+		printf("\n");
+	}
+	if (BatteryCharge->PhoneTemperature != -1) {
+		printf(LISTFORMAT, _("Phone temperature"));
+		printf(_("%i C"), BatteryCharge->PhoneTemperature);
+		printf("\n");
+	}
+	if (BatteryCharge->BatteryVoltage != -1) {
+		printf(LISTFORMAT, _("Battery voltage"));
+		printf(_("%i mV"), BatteryCharge->BatteryVoltage);
+		printf("\n");
+	}
+	if (BatteryCharge->ChargeVoltage != -1) {
+		printf(LISTFORMAT, _("Charge voltage"));
+		printf(_("%i mV"), BatteryCharge->ChargeVoltage);
+		printf("\n");
+	}
+	if (BatteryCharge->ChargeCurrent != -1) {
+		printf(LISTFORMAT, _("Charge current"));
+		printf(_("%i mA"), BatteryCharge->ChargeCurrent);
+		printf("\n");
+	}
+	if (BatteryCharge->PhoneCurrent != -1) {
+		printf(LISTFORMAT, _("Phone current"));
+		printf(_("%i mA"), BatteryCharge->PhoneCurrent);
+		printf("\n");
+	}
+	if (BatteryCharge->ChargeState != 0) {
+		printf(LISTFORMAT, _("Charge state"));
+		switch (BatteryCharge->ChargeState) {
+			case GSM_BatteryPowered:
+				printf("%s", _("powered from battery"));
+				break;
+			case GSM_BatteryConnected:
+				printf("%s", _("battery connected, but not powered from battery"));
+				break;
+			case GSM_BatteryCharging:
+				printf("%s", _("battery connected and is being charged"));
+				break;
+			case GSM_BatteryFull:
+				printf("%s", _("battery connected and is fully charged"));
+				break;
+			case GSM_BatteryNotConnected:
+				printf("%s", _("battery not connected"));
+				break;
+			case GSM_PowerFault:
+				printf("%s", _("detected power failure"));
+				break;
+#ifndef CHECK_CASES
+			default:
+				printf("%s", _("unknown"));
+				break;
+#endif
+		}
+		printf("\n");
+	}
+	if (BatteryCharge->BatteryType != 0) {
+		printf(LISTFORMAT, _("Battery type"));
+		switch (BatteryCharge->BatteryType) {
+			case GSM_BatteryLiIon:
+				printf("%s", _("Lithium Ion"));
+				break;
+			case GSM_BatteryLiPol:
+				printf("%s", _("Lithium Polymer"));
+				break;
+			case GSM_BatteryNiMH:
+				printf("%s", _("NiMH"));
+				break;
+			case GSM_BatteryUnknown:
+#ifndef CHECK_CASES
+			default:
+#endif
+				printf("%s", _("unknown"));
+				break;
+		}
+		printf("\n");
+	}
+}
+
 GSM_Error GSM_PlayRingtone(GSM_Ringtone ringtone)
 {
 	int 		i;
@@ -371,20 +466,96 @@ void NetworkInfo(int argc UNUSED, char *argv[] UNUSED)
 	GSM_Terminate();
 }
 
-void IncomingCall(GSM_StateMachine *sm UNUSED, GSM_Call call, void *user_data)
+void Battery(int argc UNUSED, char *argv[] UNUSED)
+{
+	GSM_BatteryCharge BatteryCharge;
+	GSM_Error error;
+
+	GSM_Init(TRUE);
+
+	error = GSM_GetBatteryCharge(gsm, &BatteryCharge);
+	Print_Error(error);
+
+	PrintBatteryCharge(&BatteryCharge);
+
+	GSM_Terminate();
+}
+
+void Screenshot(int argc UNUSED, char *argv[])
+{
+	GSM_BinaryPicture pic;
+	const char *ext = NULL;
+	char *fname;
+	GSM_Error error;
+	FILE *f;
+	size_t written;
+	int ret;
+
+	GSM_Init(TRUE);
+
+	error = GSM_GetScreenshot(gsm, &pic);
+	Print_Error(error);
+
+	switch (pic.Type) {
+		case PICTURE_BMP:
+			ext = ".bmp";
+			break;
+		case PICTURE_GIF:
+			ext = ".gif";
+			break;
+		case PICTURE_JPG:
+			ext = ".jpg";
+			break;
+		case PICTURE_ICN:
+			ext = ".icn";
+			break;
+		case PICTURE_PNG:
+			ext = ".png";
+			break;
+	}
+	if (ext == NULL) {
+		printf_err("Unknown file type: %d\n", pic.Type);
+		return;
+	}
+
+	fname = (char *)malloc(strlen(argv[2]) + strlen(ext) + 1);
+	strcpy(fname, argv[2]);
+	strcat(fname, ext);
+
+	f = fopen(fname, "w");
+	if (f == NULL) {
+		printf_err("Failed to open file: %s\n", fname);
+		return;
+	}
+	written = fwrite(pic.Buffer, 1, pic.Length, f);
+	ret = fclose(f);
+	if (ret != 0 || written != pic.Length) {
+		printf_err("Failed to write file: %s\n", fname);
+		return;
+	}
+
+	printf_info("File saved as %s\n", fname);
+
+	free(fname);
+	free(pic.Buffer);
+
+	GSM_Terminate();
+}
+
+void IncomingCall(GSM_StateMachine *sm UNUSED, GSM_Call *call, void *user_data)
 {
 	printf(LISTFORMAT, _("Call info"));
 
-	if (call.CallIDAvailable) {
-		printf(_("ID %i, "),call.CallID);
+	if (call->CallIDAvailable) {
+		printf(_("ID %i, "),call->CallID);
 	}
-	switch(call.Status) {
-		case GSM_CALL_IncomingCall  	: printf(_("incoming call from \"%s\"\n"),DecodeUnicodeConsole(call.PhoneNumber));  	break;
-		case GSM_CALL_OutgoingCall  	: printf(_("outgoing call to \"%s\"\n"),DecodeUnicodeConsole(call.PhoneNumber));    	break;
+	switch(call->Status) {
+		case GSM_CALL_IncomingCall  	: printf(_("incoming call from \"%s\"\n"),DecodeUnicodeConsole(call->PhoneNumber));  	break;
+		case GSM_CALL_OutgoingCall  	: printf(_("outgoing call to \"%s\"\n"),DecodeUnicodeConsole(call->PhoneNumber));    	break;
 		case GSM_CALL_CallStart     	: printf("%s\n", _("call started")); 					  	  		break;
 		case GSM_CALL_CallEnd	   	: printf("%s\n", _("end of call (unknown side)")); 					  	break;
 		case GSM_CALL_CallLocalEnd  	: printf("%s\n", _("call end from our side"));						  	break;
-		case GSM_CALL_CallRemoteEnd 	: printf(_("call end from remote side (code %i)\n"),call.StatusCode);			break;
+		case GSM_CALL_CallRemoteEnd 	: printf(_("call end from remote side (code %i)\n"),call->StatusCode);			break;
 		case GSM_CALL_CallEstablished   : printf("%s\n", _("call established. Waiting for answer"));				  	break;
 		case GSM_CALL_CallHeld		: printf("%s\n", _("call held"));								break;
 		case GSM_CALL_CallResumed	: printf("%s\n", _("call resumed"));								break;
@@ -474,97 +645,7 @@ void Monitor(int argc, char *argv[])
 		}
 		CHECK_EXIT;
 		if ( (error = GSM_GetBatteryCharge(gsm,&BatteryCharge)) == ERR_NONE) {
-            		if (BatteryCharge.BatteryPercent != -1) {
-				printf(LISTFORMAT, _("Battery level"));
-				printf(_("%i percent"), BatteryCharge.BatteryPercent);
-				printf("\n");
-			}
-            		if (BatteryCharge.BatteryCapacity != -1) {
-				printf(LISTFORMAT, _("Battery capacity"));
-				printf(_("%i mAh"), BatteryCharge.BatteryCapacity);
-				printf("\n");
-			}
-            		if (BatteryCharge.BatteryTemperature != -1) {
-				printf(LISTFORMAT, _("Battery temperature"));
-				/* l10n: This means degrees Celsius */
-				printf(_("%i C"), BatteryCharge.BatteryTemperature);
-				printf("\n");
-			}
-            		if (BatteryCharge.PhoneTemperature != -1) {
-				printf(LISTFORMAT, _("Phone temperature"));
-				printf(_("%i C"), BatteryCharge.PhoneTemperature);
-				printf("\n");
-			}
-            		if (BatteryCharge.BatteryVoltage != -1) {
-				printf(LISTFORMAT, _("Battery voltage"));
-				printf(_("%i mV"), BatteryCharge.BatteryVoltage);
-				printf("\n");
-			}
-            		if (BatteryCharge.ChargeVoltage != -1) {
-				printf(LISTFORMAT, _("Charge voltage"));
-				printf(_("%i mV"), BatteryCharge.ChargeVoltage);
-				printf("\n");
-			}
-            		if (BatteryCharge.ChargeCurrent != -1) {
-				printf(LISTFORMAT, _("Charge current"));
-				printf(_("%i mA"), BatteryCharge.ChargeCurrent);
-				printf("\n");
-			}
-            		if (BatteryCharge.PhoneCurrent != -1) {
-				printf(LISTFORMAT, _("Phone current"));
-				printf(_("%i mA"), BatteryCharge.PhoneCurrent);
-				printf("\n");
-			}
-            		if (BatteryCharge.ChargeState != 0) {
-                		printf(LISTFORMAT, _("Charge state"));
-                		switch (BatteryCharge.ChargeState) {
-                    			case GSM_BatteryPowered:
-						printf("%s", _("powered from battery"));
-						break;
-                    			case GSM_BatteryConnected:
-						printf("%s", _("battery connected, but not powered from battery"));
-                        			break;
-                    			case GSM_BatteryCharging:
-						printf("%s", _("battery connected and is being charged"));
-                        			break;
-                    			case GSM_BatteryFull:
-						printf("%s", _("battery connected and is fully charged"));
-                        			break;
-                    			case GSM_BatteryNotConnected:
-                        			printf("%s", _("battery not connected"));
-                        			break;
-                    			case GSM_PowerFault:
-                        			printf("%s", _("detected power failure"));
-                        			break;
-#ifndef CHECK_CASES
-                    			default:
-                        			printf("%s", _("unknown"));
-                       				break;
-#endif
-                		}
-                		printf("\n");
-            		}
-            		if (BatteryCharge.BatteryType != 0) {
-                		printf(LISTFORMAT, _("Battery type"));
-                		switch (BatteryCharge.BatteryType) {
-                    			case GSM_BatteryLiIon:
-						printf("%s", _("Lithium Ion"));
-						break;
-                    			case GSM_BatteryLiPol:
-						printf("%s", _("Lithium Polymer"));
-						break;
-                    			case GSM_BatteryNiMH:
-						printf("%s", _("NiMH"));
-						break;
-					case GSM_BatteryUnknown:
-#ifndef CHECK_CASES
-                    			default:
-#endif
-                        			printf("%s", _("unknown"));
-                       				break;
-                		}
-                		printf("\n");
-            		}
+			PrintBatteryCharge(&BatteryCharge);
         	}
 		CHECK_EXIT;
 		if ( (error = GSM_GetSignalQuality(gsm,&SignalQuality)) == ERR_NONE) {
@@ -663,21 +744,21 @@ void GetRingtone(int argc, char *argv[])
 
 void GetRingtonesList(int argc UNUSED, char *argv[] UNUSED)
 {
- 	GSM_AllRingtonesInfo 	Info = {0, NULL};
+	GSM_AllRingtonesInfo 	Info = {0, NULL};
 	GSM_Error error;
 	int			i;
 
 	GSM_Init(TRUE);
 
 	error=GSM_GetRingtonesInfo(gsm,&Info);
- 	if (error != ERR_NONE && Info.Ringtone) free(Info.Ringtone);
+	if (error != ERR_NONE && Info.Ringtone) free(Info.Ringtone);
 	Print_Error(error);
 
 	GSM_Terminate();
 
 	for (i=0;i<Info.Number;i++) printf("%i. \"%s\"\n",i+1,DecodeUnicodeConsole(Info.Ringtone[i].Name));
 
- 	if (Info.Ringtone) free(Info.Ringtone);
+	if (Info.Ringtone) free(Info.Ringtone);
 }
 
 void DialVoice(int argc, char *argv[])
@@ -704,9 +785,9 @@ void DialVoice(int argc, char *argv[])
 
 int TerminateID = -1;
 
-void IncomingCall0(GSM_StateMachine *sm UNUSED, GSM_Call call, void * user_data)
+void IncomingCall0(GSM_StateMachine *sm UNUSED, GSM_Call *call, void * user_data)
 {
-	if (call.CallIDAvailable) TerminateID = call.CallID;
+	if (call->CallIDAvailable) TerminateID = call->CallID;
 }
 
 void MakeTerminatedCall(int argc, char *argv[])
@@ -946,8 +1027,9 @@ void GetBitmap(int argc, char *argv[])
 	GSM_File		File;
 	GSM_MultiBitmap 	MultiBitmap;
 	int			location=0, Handle, Size;
- 	GSM_AllRingtonesInfo 	Info = {0, NULL};
+	GSM_AllRingtonesInfo 	Info = {0, NULL};
 	unsigned char		buffer[10];
+	const unsigned char *ringtonename;
 	GSM_Error error;
 
 	if (strcasecmp(argv[2],"STARTUP") == 0) {
@@ -1012,14 +1094,13 @@ void GetBitmap(int argc, char *argv[])
 			error = ERR_NONE;
 
 			printf(LISTFORMAT, _("Ringtone"));
-			if (UnicodeLength(GSM_GetRingtoneName(&Info,MultiBitmap.Bitmap[0].RingtoneID))!=0) {
+			ringtonename = GSM_GetRingtoneName(&Info, MultiBitmap.Bitmap[0].RingtoneID);
+			if (ringtonename != NULL) {
 				/* l10n: Ringtone name and ID format */
-				printf(_("\"%s\" (ID %i)\n"),
-					DecodeUnicodeConsole(GSM_GetRingtoneName(&Info,MultiBitmap.Bitmap[0].RingtoneID)),
-					MultiBitmap.Bitmap[0].RingtoneID);
+				printf(_("\"%s\" (ID %i)\n"), DecodeUnicodeConsole(ringtonename), MultiBitmap.Bitmap[0].RingtoneID);
 			} else {
 				/* l10n: Ringtone ID format */
-				printf(_("ID %i\n"),MultiBitmap.Bitmap[0].RingtoneID);
+				printf(_("ID %i\n"), MultiBitmap.Bitmap[0].RingtoneID);
 			}
 
 			if (Info.Ringtone) free(Info.Ringtone);
@@ -1330,17 +1411,17 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 	DoClear = FALSE;
 	error = GSM_GetNextCalendar(gsm,&Calendar,TRUE);
 	if (error == ERR_NONE) {
- 		if (answer_yes("%s", _("Delete phone calendar notes?"))) DoClear = TRUE;
+		if (answer_yes("%s", _("Delete phone calendar notes?"))) DoClear = TRUE;
 	}
 	if (DoClear) {
 		fprintf(stderr, LISTFORMAT, _("Deleting"));
 		error=GSM_DeleteAllCalendar(gsm);
 		if (error == ERR_NOTSUPPORTED || error == ERR_NOTIMPLEMENTED) {
- 			while (1) {
+			while (1) {
 				error = GSM_GetNextCalendar(gsm,&Calendar,TRUE);
 				if (error != ERR_NONE) break;
 				error = GSM_DeleteCalendar(gsm,&Calendar);
- 				Print_Error(error);
+				Print_Error(error);
 				fprintf(stderr, "*");
 			}
 			fprintf(stderr, "\n");
@@ -1360,11 +1441,11 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 		fprintf(stderr, LISTFORMAT, _("Deleting"));
 		error=GSM_DeleteAllToDo(gsm);
 		if (error == ERR_NOTSUPPORTED || error == ERR_NOTIMPLEMENTED) {
- 			while (1) {
+			while (1) {
 				error = GSM_GetNextToDo(gsm,&ToDo,TRUE);
 				if (error != ERR_NONE) break;
 				error = GSM_DeleteToDo(gsm,&ToDo);
- 				Print_Error(error);
+				Print_Error(error);
 				fprintf(stderr, "*");
 			}
 			fprintf(stderr, "\n");
@@ -1424,10 +1505,10 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 	error=GSM_GetFMStation(gsm,&Station);
 	if (error == ERR_NONE || error == ERR_EMPTY) {
 	 	if (answer_yes("%s", _("Delete all phone FM radio stations?"))) {
- 			error=GSM_ClearFMStations(gsm);
- 			Print_Error(error);
+			error=GSM_ClearFMStations(gsm);
+			Print_Error(error);
 		}
- 	}
+	}
 
 	GSM_Terminate();
 }
@@ -1917,9 +1998,34 @@ void EnterSecurityCode(int argc UNUSED, char *argv[])
 		strcpy(Code.Code,argv[3]);
 	}
 
+	if (argc >= 5) {
+		if (strcmp(argv[4], "-") == 0) {
+			sprintf(message, _("Enter new PIN code: "));
+#ifdef HAVE_GETPASS
+			pass = getpass(message);
+			strcpy(Code.Code, pass);
+#else
+			/* Read code from stdin */
+#ifdef HAVE_UNISTD_H
+			if (isatty(fileno(stdin))) {
+				printf("%s", message);
+			}
+#endif
+			if (fscanf(stdin, "%15s", Code.Code) != 1) {
+				printf_err("%s\n", _("No PIN code entered!"));
+				Terminate(2);
+			}
+#endif
+		} else {
+			strcpy(Code.NewPIN, argv[4]);
+		}
+	} else {
+		Code.NewPIN[0] = 0;
+	}
+
 	GSM_Init(TRUE);
 
-	error=GSM_EnterSecurityCode(gsm,Code);
+	error=GSM_EnterSecurityCode(gsm, &Code);
 	Print_Error(error);
 
 	GSM_Terminate();
@@ -1931,7 +2037,8 @@ void GetProfile(int argc, char *argv[])
 	int			start,stop,i,j,k;
 	GSM_Bitmap		caller[5];
 	gboolean			callerinit[5],special;
- 	GSM_AllRingtonesInfo 	Info = {0, NULL};
+	GSM_AllRingtonesInfo 	Info = {0, NULL};
+	const unsigned char *ringtonename;
 	GSM_Error error;
 
 	GetStartStop(&start, &stop, 2, argc, argv);
@@ -1966,10 +2073,13 @@ void GetProfile(int argc, char *argv[])
 				} else {
 					printf(LISTFORMAT, _("Message alert tone ID"));
 				}
-				if (UnicodeLength(GSM_GetRingtoneName(&Info,Profile.FeatureValue[j]))!=0) {
-					printf("\"%s\"\n",DecodeUnicodeConsole(GSM_GetRingtoneName(&Info,Profile.FeatureValue[j])));
+				ringtonename = GSM_GetRingtoneName(&Info, Profile.FeatureValue[j]);
+				if (ringtonename != NULL) {
+					/* l10n: Ringtone name and ID format */
+					printf(_("\"%s\" (ID %i)\n"), DecodeUnicodeConsole(ringtonename), Profile.FeatureValue[j]);
 				} else {
-					printf("%i\n",Profile.FeatureValue[j]);
+					/* l10n: Ringtone ID format */
+					printf(_("ID %i\n"), Profile.FeatureValue[j]);
 				}
 				break;
 			case Profile_CallerGroups:
@@ -2088,7 +2198,8 @@ void GetSpeedDial(int argc, char *argv[])
 
 			Phonebook.Location	= SpeedDial.MemoryLocation;
 			Phonebook.MemoryType 	= SpeedDial.MemoryType;
-			error=GSM_GetMemory(gsm,&Phonebook);
+			error = GSM_GetMemory(gsm,&Phonebook);
+			Print_Error(error);
 
 			GSM_PhonebookFindDefaultNameNumberGroup(&Phonebook, &Name, &Number, &Group);
 
@@ -2121,7 +2232,7 @@ void ResetPhoneSettings(int argc UNUSED, char *argv[])
 	error=GSM_ResetPhoneSettings(gsm,Type);
 	Print_Error(error);
 
- 	GSM_Terminate();
+	GSM_Terminate();
 }
 
 void SendDTMF(int argc UNUSED, char *argv[])
@@ -2132,7 +2243,7 @@ void SendDTMF(int argc UNUSED, char *argv[])
 	error=GSM_SendDTMF(gsm,argv[2]);
 	Print_Error(error);
 
- 	GSM_Terminate();
+	GSM_Terminate();
 }
 
 void GetDisplayStatus(int argc UNUSED, char *argv[] UNUSED)
@@ -2148,19 +2259,33 @@ void GetDisplayStatus(int argc UNUSED, char *argv[] UNUSED)
 
 	printf(LISTFORMAT "\n", _("Currently shown on the display"));
 
-	for (i=0;i<Features.Number;i++) {
+	for (i = 0; i < Features.Number; i++) {
 		switch(Features.Feature[i]) {
-		case GSM_CallActive	: printf("%s\n", _("Call active"));	break;
-		case GSM_UnreadSMS	: printf("%s\n", _("Unread SMS"));	break;
-		case GSM_VoiceCall	: printf("%s\n", _("Voice call"));	break;
-		case GSM_FaxCall	: printf("%s\n", _("Fax call"));	break;
-		case GSM_DataCall	: printf("%s\n", _("Data call"));	break;
-		case GSM_KeypadLocked	: printf("%s\n", _("Keypad locked"));	break;
-		case GSM_SMSMemoryFull	: printf("%s\n", _("SMS memory full"));
+			case GSM_CallActive:
+				printf("%s\n", _("Call active"));
+				break;
+			case GSM_UnreadSMS:
+				printf("%s\n", _("Unread SMS"));
+				break;
+			case GSM_VoiceCall:
+				printf("%s\n", _("Voice call"));
+				break;
+			case GSM_FaxCall:
+				printf("%s\n", _("Fax call"));
+				break;
+			case GSM_DataCall:
+				printf("%s\n", _("Data call"));
+				break;
+			case GSM_KeypadLocked:
+				printf("%s\n", _("Keypad locked"));
+				break;
+			case GSM_SMSMemoryFull:
+				printf("%s\n", _("SMS memory full"));
+				break;
 		}
 	}
 
- 	GSM_Terminate();
+	GSM_Terminate();
 }
 
 void SetAutoNetworkLogin(int argc UNUSED, char *argv[] UNUSED)
@@ -2171,7 +2296,7 @@ void SetAutoNetworkLogin(int argc UNUSED, char *argv[] UNUSED)
 	error=GSM_SetAutoNetworkLogin(gsm);
 	Print_Error(error);
 
- 	GSM_Terminate();
+	GSM_Terminate();
 }
 
 void GetFMStation(int argc, char *argv[])
@@ -2191,7 +2316,7 @@ void GetFMStation(int argc, char *argv[])
 		switch (error) {
 		case ERR_EMPTY:
 			printf("%s\n", _("Entry is empty"));
-		    	break;
+			break;
 		case ERR_NONE:
 			printf(LISTFORMAT "\"%s\"\n", _("Station name"), DecodeUnicodeConsole(Station.StationName));
 			printf(LISTFORMAT "%.1f MHz\n", _("Frequency"), Station.Frequency);
@@ -2207,91 +2332,138 @@ void CallDivert(int argc, char *argv[])
 {
 	GSM_Error error;
 	GSM_MultiCallDivert cd;
+	gboolean get = FALSE;
 	int i;
 
-	     if (strcasecmp("get",	argv[2]) == 0) {}
-	else if (strcasecmp("set",	argv[2]) == 0) {}
-	else {
-		printf(_("Unknown divert action (\"%s\")\n"),argv[2]);
+	if (strcasecmp("get", argv[2]) == 0) {
+		get = TRUE;
+	} else if (strcasecmp("set", argv[2]) == 0) {
+		get = FALSE;
+	} else {
+		printf(_("Unknown divert action (\"%s\")\n"), argv[2]);
 		Terminate(2);
 	}
 
-	     if (strcasecmp("all"       , argv[3]) == 0) {cd.Request.DivertType = GSM_DIVERT_AllTypes  ;}
-	else if (strcasecmp("busy"      , argv[3]) == 0) {cd.Request.DivertType = GSM_DIVERT_Busy      ;}
-	else if (strcasecmp("noans"     , argv[3]) == 0) {cd.Request.DivertType = GSM_DIVERT_NoAnswer  ;}
-	else if (strcasecmp("outofreach", argv[3]) == 0) {cd.Request.DivertType = GSM_DIVERT_OutOfReach;}
-	else {
-		printf(_("Unknown divert type (\"%s\")\n"),argv[3]);
+	if (strcasecmp("all", argv[3]) == 0) {
+		cd.Request.DivertType = GSM_DIVERT_AllTypes;
+	} else if (strcasecmp("busy", argv[3]) == 0) {
+		cd.Request.DivertType = GSM_DIVERT_Busy;
+	} else if (strcasecmp("noans", argv[3]) == 0) {
+		cd.Request.DivertType = GSM_DIVERT_NoAnswer;
+	} else if (strcasecmp("outofreach", argv[3]) == 0) {
+		cd.Request.DivertType = GSM_DIVERT_OutOfReach;
+	} else {
+		printf(_("Unknown divert type (\"%s\")\n"), argv[3]);
 		Terminate(2);
 	}
 
-	     if (strcasecmp("all"  , argv[4]) == 0) {cd.Request.CallType = GSM_DIVERT_AllCalls  ;}
-	else if (strcasecmp("voice", argv[4]) == 0) {cd.Request.CallType = GSM_DIVERT_VoiceCalls;}
-	else if (strcasecmp("fax"  , argv[4]) == 0) {cd.Request.CallType = GSM_DIVERT_FaxCalls  ;}
-	else if (strcasecmp("data" , argv[4]) == 0) {cd.Request.CallType = GSM_DIVERT_DataCalls ;}
-	else {
-		printf(_("Unknown call type (\"%s\")\n"),argv[4]);
+	if (strcasecmp("all", argv[4]) == 0) {
+		cd.Request.CallType = GSM_DIVERT_AllCalls;
+	} else if (strcasecmp("voice", argv[4]) == 0) {
+		cd.Request.CallType = GSM_DIVERT_VoiceCalls;
+	} else if (strcasecmp("fax", argv[4]) == 0) {
+		cd.Request.CallType = GSM_DIVERT_FaxCalls;
+	} else if (strcasecmp("data", argv[4]) == 0) {
+		cd.Request.CallType = GSM_DIVERT_DataCalls;
+	} else {
+		printf(_("Unknown call type (\"%s\")\n"), argv[4]);
 		Terminate(2);
 	}
 
 	GSM_Init(TRUE);
 
-	if (strcasecmp("get", argv[2]) == 0) {
+	if (get) {
 		error = GSM_GetCallDivert(gsm,&cd);
-	    	Print_Error(error);
+		Print_Error(error);
 		printf("%s", _("Query:"));
 		printf("\n");
 		printf("  " LISTFORMAT, _("Divert type"));
 	} else {
 		cd.Request.Number[0] = 0;
 		cd.Request.Number[1] = 0;
-		if (argc > 5) EncodeUnicode(cd.Request.Number,argv[5],strlen(argv[5]));
+		if (argc > 5) {
+			EncodeUnicode(cd.Request.Number,argv[5],strlen(argv[5]));
+		}
 
 		cd.Request.Timeout = 0;
-		if (argc > 6) cd.Request.Timeout = GetInt(argv[6]);
+		if (argc > 6) {
+			cd.Request.Timeout = GetInt(argv[6]);
+		}
 
 		error = GSM_SetCallDivert(gsm,&cd);
-	    	Print_Error(error);
+		Print_Error(error);
 		printf("%s", _("Changed:"));
 		printf("\n");
 		printf("  " LISTFORMAT, _("Divert type"));
 	}
 
-      	switch (cd.Request.DivertType) {
-   		case GSM_DIVERT_Busy      : printf("%s", _("when busy"));				break;
-	        case GSM_DIVERT_NoAnswer  : printf("%s", _("when not answered"));			break;
-      	        case GSM_DIVERT_OutOfReach: printf("%s", _("when phone off or no coverage"));	break;
-                case GSM_DIVERT_AllTypes  : printf("%s", _("all types of diverts"));			break;
-                default		          : printf(_("unknown %i"),cd.Request.DivertType);			break;
-        }
+	switch (cd.Request.DivertType) {
+		case GSM_DIVERT_Busy:
+			printf("%s", _("when busy"));
+			break;
+		case GSM_DIVERT_NoAnswer:
+			printf("%s", _("when not answered"));
+			break;
+		case GSM_DIVERT_OutOfReach:
+			printf("%s", _("when phone off or no coverage"));
+			break;
+		case GSM_DIVERT_AllTypes:
+			printf("%s", _("all types of diverts"));
+			break;
+		default:
+			printf(_("unknown %i"),cd.Request.DivertType);
+			break;
+	}
 	printf("\n");
+
 	printf("  " LISTFORMAT, _("Call type"));
 	switch (cd.Request.CallType) {
-	 	case GSM_DIVERT_VoiceCalls: printf("%s", _("voice"));				break;
-                case GSM_DIVERT_FaxCalls  : printf("%s", _("fax"));				break;
-                case GSM_DIVERT_DataCalls : printf("%s", _("data"));		 		break;
-		case GSM_DIVERT_AllCalls  : printf("%s", _("data & fax & voice"));		break;
-                default                   : printf(_("unknown %i"),cd.Request.CallType);   		break;
-        }
+		case GSM_DIVERT_VoiceCalls:
+			printf("%s", _("voice"));
+			break;
+		case GSM_DIVERT_FaxCalls:
+			printf("%s", _("fax"));
+			break;
+		case GSM_DIVERT_DataCalls:
+			printf("%s", _("data"));
+			break;
+		case GSM_DIVERT_AllCalls:
+			printf("%s", _("data & fax & voice"));
+			break;
+		default:
+			printf(_("unknown %i"),cd.Request.CallType);
+			break;
+	}
 	printf("\n");
+
 	printf("%s", _("Response:"));
 	printf("\n");
 
-	for (i=0;i<cd.Response.EntriesNum;i++) {
+	for (i = 0; i < cd.Response.EntriesNum; i++) {
 		printf("\n");
 		printf("  " LISTFORMAT, _("Call type"));
-        	switch (cd.Response.Entries[i].CallType) {
-                	case GSM_DIVERT_VoiceCalls: printf("%s", _("voice"));		 	break;
-                	case GSM_DIVERT_FaxCalls  : printf("%s", _("fax"));		 	break;
-                	case GSM_DIVERT_DataCalls : printf("%s", _("data"));		 	break;
-                	default                   : printf(_("unknown %i"),cd.Response.Entries[i].CallType);break;
-              	}
+		switch (cd.Response.Entries[i].CallType) {
+			case GSM_DIVERT_VoiceCalls:
+				printf("%s", _("voice"));
+				break;
+			case GSM_DIVERT_FaxCalls:
+				printf("%s", _("fax"));
+				break;
+			case GSM_DIVERT_DataCalls:
+				printf("%s", _("data"));
+				break;
+			default:
+				printf(_("unknown %i"),cd.Response.Entries[i].CallType);
+				break;
+		}
 		printf("\n");
+
 		printf("  " LISTFORMAT, _("Timeout"));
 		PRINTSECONDS(cd.Response.Entries[i].Timeout);
 		printf("\n");
-                printf("  "LISTFORMAT "%s\n", _("Number"),DecodeUnicodeString(cd.Response.Entries[i].Number));
-        }
+
+		printf("  " LISTFORMAT "%s\n", _("Number"), DecodeUnicodeString(cd.Response.Entries[i].Number));
+	}
 	printf("\n");
 
 	GSM_Terminate();
@@ -2303,7 +2475,23 @@ void CancelAllDiverts(int argc UNUSED, char *argv[] UNUSED)
 	GSM_Init(TRUE);
 
 	error = GSM_CancelAllDiverts(gsm);
-    	Print_Error(error);
+	Print_Error(error);
+
+	GSM_Terminate();
+}
+
+void Install(int argc UNUSED, char *argv[] UNUSED)
+{
+	GSM_Error error;
+
+	GSM_Init(FALSE);
+
+	/* TODO: make search path configurable */
+	error = GSM_Install(gsm, INI_GetValue(cfg, "gammu", "datapath", FALSE));
+	Print_Error(error);
+
+	printf("%s\n", _("The application has been successfully sent to the phone."));
+	printf("%s\n", _("Please find received files in Inbox and install them."));
 
 	GSM_Terminate();
 }
