@@ -60,9 +60,7 @@ const GSM_MemoryEntry *SearchPhoneNumber(const unsigned char *number, const GSM_
 			switch (List[i]->Entries[*pos].EntryType) {
 				case PBK_Number_General:
 				case PBK_Number_Mobile:
-				case PBK_Number_Work:
 				case PBK_Number_Fax:
-				case PBK_Number_Home:
 				case PBK_Number_Pager:
 				case PBK_Number_Other:
 					if (mywstrncmp(List[i]->Entries[*pos].Text,number,-1)) {
@@ -113,15 +111,9 @@ void PrintPhoneNumber(const unsigned char *number, const void *Info)
 		case PBK_Number_Mobile:
 			printf(" (%s)", _("mobile"));
 			break;
-		case PBK_Number_Work:
-			printf(" (%s)", _("work"));
-			break;
 		case PBK_Number_Fax:
 			printf(" (%s)", _("fax"));
 			break;
-		case PBK_Number_Home:
-			printf(" (%s)", _("home"));
-			break;
 		case PBK_Number_Pager:
 			printf(" (%s)", _("pager"));
 			break;
@@ -131,6 +123,16 @@ void PrintPhoneNumber(const unsigned char *number, const void *Info)
 		default:
 			break;
 	}
+	switch (pbk->Entries[pos].Location) {
+		case PBK_Location_Home:
+			printf(" [%s]", _("home"));
+			break;
+		case PBK_Location_Work:
+			printf(" [%s]", _("work"));
+			break;
+		case PBK_Location_Unknown:
+			break;
+	}
 #endif
 }
 
@@ -390,7 +392,7 @@ void DisplayMultiSMSInfo (GSM_MultiSMSMessage *sms, gboolean eachsms, gboolean e
 				Pos = 0;
 				error = GSM_DecodeVCARD(GSM_GetGlobalDebug(), SMSInfo.Entries[i].File->Buffer, &Pos, &pbk, Nokia_VCard21);
 				if (error == ERR_NONE) {
-					error = PrintMemoryEntry(&pbk, sm);
+					PrintMemoryEntry(&pbk, sm);
 				}
 			} else {
 				printf("\n");
@@ -504,7 +506,7 @@ void DisplayMultiSMSInfo (GSM_MultiSMSMessage *sms, gboolean eachsms, gboolean e
 }
 
 
-void DisplaySMSFrame(GSM_SMSMessage *SMS, GSM_StateMachine *sm)
+GSM_Error DisplaySMSFrame(GSM_SMSMessage *SMS, GSM_StateMachine *sm)
 {
 	GSM_Error 		error;
 	int			i, length, current = 0;
@@ -513,15 +515,14 @@ void DisplaySMSFrame(GSM_SMSMessage *SMS, GSM_StateMachine *sm)
 
 	error=PHONE_EncodeSMSFrame(sm,SMS,buffer,PHONE_SMSSubmit,&length,TRUE);
 	if (error != ERR_NONE) {
-		printf("%s\n", _("Error"));
-		exit(-1);
+		return error;
 	}
         length = length - PHONE_SMSSubmit.Text;
 
         for(i=SMS->UDH.Length;i<length;i++) {
 		req[i-SMS->UDH.Length]=buffer[PHONE_SMSSubmit.Text+i];
 	}
-        EncodeHexBin(hexmsg, req, length-SMS->UDH.Length);
+        EncodeHexBin(hexmsg, req, MAX(0, length-SMS->UDH.Length));
 
         for(i=0;i<SMS->UDH.Length;i++) {
 		req[i]=buffer[PHONE_SMSSubmit.Text+i];
@@ -550,6 +551,7 @@ void DisplaySMSFrame(GSM_SMSMessage *SMS, GSM_StateMachine *sm)
         printf(LISTFORMAT "%s\n", _("Whole PDU"), hexreq);
 	printf("\n");
 	fflush(stdout);
+	return ERR_NONE;
 }
 
 /* How should editor hadle tabs in this file? Add editor commands here.
