@@ -30,6 +30,30 @@ const unsigned char *GSM_GetNetworkName(const char *NetworkCode);
  */
 const unsigned char *GSM_GetCountryName(const char *CountryCode);
 
+/**
+ * Structure for defining code-name mappings.
+ *
+ * \ingroup Info
+ */
+typedef struct {
+	const char Code[8];
+	const char Name[64];
+} GSM_CodeName;
+
+/**
+ * List of network codes, terminated by empty name/code.
+ *
+ * \ingroup Info
+ */
+extern const GSM_CodeName GSM_Networks[];
+
+/**
+ * List of country codes, terminated by empty name/code.
+ *
+ * \ingroup Info
+ */
+extern const GSM_CodeName GSM_Countries[];
+
 /**
  * Status of network logging
  *
@@ -638,6 +662,42 @@ typedef enum {
 	 * tell so.
 	 */
 	F_SIEMENS_PBK,
+	/**
+	 * Disable AT+SYNCML probing.
+	 */
+	F_NO_ATSYNCML,
+	/**
+	 * Phone supports m-obex (usually Samsung phones).
+	 */
+	F_MOBEX,
+	/**
+	 * Phone supports m-obex (usually Samsung phones) using AT$TSSPCSW=1.
+	 */
+	F_TSSPCSW,
+	/**
+	 * Disable GetNext* operations on the dummy phone.
+	 */
+	F_DISABLE_GETNEXT,
+	/**
+	 * Disable GetNextSMS operations on the dummy phone.
+	 */
+	F_DISABLE_GETNEXTSMS,
+	/**
+	 * CMGL hangs, so should not be used.
+	 */
+	F_DISABLE_CMGL,
+	/**
+	 * Phone does not support UTF8 even if it reports it.
+	 */
+	F_NO_UTF8,
+	/**
+	 * Samsung B2100 in UCS-2 mode provides a garbled UTF-8 instead.
+	 */
+	F_SAMSUNG_UTF8,
+	/**
+	 * SMS text is always UTF-8 encoded.
+	 */
+	F_SMS_UTF8_ENCODED,
 
 	/**
 	 * Just marker of highest feature code, should not be used.
@@ -721,6 +781,11 @@ gboolean GSM_AddPhoneFeature(GSM_PhoneModel * model, GSM_Feature feature);
 /**
  * Reads manufacturer from phone.
  *
+ * \param s State machine pointer.
+ * \param value Pointer where to store manufacturer name
+ *
+ * \return Error code.
+ *
  * \ingroup Info
  */
 GSM_Error GSM_GetManufacturer(GSM_StateMachine * s, char *value);
@@ -728,6 +793,11 @@ GSM_Error GSM_GetManufacturer(GSM_StateMachine * s, char *value);
 /**
  * Reads model from phone.
  *
+ * \param s State machine pointer.
+ * \param value Pointer where to store model name
+ *
+ * \return Error code.
+ *
  * \ingroup Info
  */
 GSM_Error GSM_GetModel(GSM_StateMachine * s, char *value);
@@ -735,6 +805,10 @@ GSM_Error GSM_GetModel(GSM_StateMachine * s, char *value);
 /**
  * Reads model info from state machine.
  *
+ * \param s State machine pointer.
+ *
+ * \return Pointer to phone information structure.
+ *
  * \ingroup Info
  */
 GSM_PhoneModel *GSM_GetModelInfo(GSM_StateMachine * s);
@@ -742,6 +816,13 @@ GSM_PhoneModel *GSM_GetModelInfo(GSM_StateMachine * s);
 /**
  * Reads firmware information from phone.
  *
+ * \param s State machine pointer.
+ * \param value Pointer where to store revision text
+ * \param date Pointer where to store revision date
+ * \param num Pointer where to store revision number
+ *
+ * \return Error code.
+ *
  * \ingroup Info
  */
 GSM_Error GSM_GetFirmware(GSM_StateMachine * s, char *value, char *date,
