@@ -115,6 +115,15 @@ GSM_FeatureName AllFeatureNames[] = {
 	{"SMS_NO_ME", F_SMS_NO_ME},
 	{"SMS_NO_SM", F_SMS_NO_SM},
 	{"SIEMENS_PBK", F_SIEMENS_PBK},
+	{"NO_ATSYNCML", F_NO_ATSYNCML},
+	{"MOBEX", F_MOBEX},
+	{"TSSPCSW", F_TSSPCSW},
+	{"DISABLE_GETNEXT", F_DISABLE_GETNEXT},
+	{"DISABLE_GETNEXTSMS", F_DISABLE_GETNEXTSMS},
+	{"DISABLE_CMGL", F_DISABLE_CMGL},
+	{"NO_UTF8", F_NO_UTF8},
+	{"SAMSUNG_UTF8", F_SAMSUNG_UTF8},
+	{"SMS_UTF8_ENCODED", F_SMS_UTF8_ENCODED},
 	{"", 0},
 };
 
@@ -261,7 +270,8 @@ GSM_PhoneModel allmodels[] = {
 	{"1650", "RM-305","Nokia 1650",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,F_PBKNOPOSTAL,0}},/* guess */
 	{"1680", "RM-394","Nokia 1680",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,F_PBKNOPOSTAL,0}},/* guess */
 	{"1681c","RH-118","Nokia 1681c",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,F_PBKNOPOSTAL,0}},/* guess */
-	{"2600", "RH-340","Nokia 2600",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
+	{"2323c","RH-543","Nokia 2323c",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
+	{"2330c","RH-512","Nokia 2330c",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"2600", "RM-340","Nokia 2600",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,F_PBKNOPOSTAL,0}},/* guess */
 	{"2610", "RH-86", "Nokia 2610",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"2610", "RH-87" ,"Nokia 2610",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
@@ -269,8 +279,12 @@ GSM_PhoneModel allmodels[] = {
 	{"2630b","RM-299","Nokia 2630b",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,F_PBKFAVORITEMESSAGE,F_6230iCALLER,0}},/* guess */
 	{"2660", "RM-292","Nokia 2660", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"2680", "RM-392","Nokia 2680", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
-	{"2700", "RM-561","Nokia 2700",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
+	{"2690", "RM-635","Nokia 2690", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
+	{"2700", "RM-561","Nokia 2700",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,F_6230iCALLER,0}},/* guess */
+	{"2720a", "RM-520","Nokia 2720a",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
+	{"2730c", "RM-578","Nokia 2730c",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"2760", "RM-258","Nokia 2760",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
+	{"2760h", "RM-391","Nokia 2760h",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"3109c","RM-274","Nokia 3109c",{F_SERIES40_30,F_FILES2,F_TODO66,F_PBKUSER,F_PBKTONEGAL,F_NOTES,F_SMS_FILES,F_CHAT,F_SYNCML,F_6230iCALLER,0}},/* guess */
 	{"3110c","RM-237","Nokia 3110c",{F_SERIES40_30,F_PBKTONEGAL,F_TODO66,F_PBKSMSLIST,F_PBKUSER,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,F_SMS_FILES,0}},/* guess */
 	{"3120c","RM-364","Nokia 3120c",{F_SERIES40_30,F_PBKTONEGAL,F_TODO66,F_PBKSMSLIST,F_PBKUSER,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,F_SMS_FILES,0}},/* guess */
@@ -278,8 +292,11 @@ GSM_PhoneModel allmodels[] = {
 	{"3220" ,"RH-49" ,"",           {F_PBKTONEGAL,F_TODO66,F_PBKSMSLIST,F_PBKUSER,F_WAPMMSPROXY,F_NOTES,F_SYNCML,F_FILES2,F_3220_MMS,0}},
 	{"3500" ,"RM-272","Nokia 3500", {F_SERIES40_30,F_PBKTONEGAL,F_TODO66,F_PBKSMSLIST,F_PBKUSER,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,0}},/* guess */
 	{"3500" ,"RM-273","Nokia 3500", {F_SERIES40_30,F_PBKTONEGAL,F_TODO66,F_PBKSMSLIST,F_PBKUSER,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,0}},/* guess */
+	{"3600s" ,"RM-352","Nokia 3600s", {F_SERIES40_30,F_PBKTONEGAL,F_TODO66,F_PBKSMSLIST,F_PBKUSER,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,0}},/* guess */
+	{"3610f" ,"RM-429","Nokia 3610 Fold", {F_SERIES40_30,F_PBKTONEGAL,F_PBKSMSLIST,F_PBKUSER,F_TODO66,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,F_SMS_FILES,0}},
 	{"3720c","RM-518","Nokia 3720c",{F_SERIES40_30,F_PBKTONEGAL,F_TODO66,F_PBKSMSLIST,F_PBKUSER,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,0}},/* guess */
 	{"5000", "RM-362","Nokia 5000", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
+	{"5000d-2b", "RM-363","Nokia 5000d-2b", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"5100" ,"NPM-6" ,"Nokia 5100", {F_PBKTONEGAL,F_TODO66,F_RADIO,0}},
 	{"5130" ,"RM-495","Nokia 5130", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"5100" ,"NPM-6U","Nokia 5100", {F_PBKTONEGAL,F_TODO66,F_RADIO,0}},
@@ -291,6 +308,7 @@ GSM_PhoneModel allmodels[] = {
 	{"5220" ,"RM-411","Nokia 5220", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},
 	{"5300" ,"RM-146","Nokia 5300", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},
 	{"5310" ,"RM-303","Nokia 5310", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
+	{"5310b","RM-304","Nokia 5310b", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"5610" ,"RM-242","Nokia 5610", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"6020" ,"RM-30" ,"Nokia 6020", {F_VOICETAGS, F_FILES2, F_CHAT, F_WAPMMSPROXY, F_TODO66, F_PBKTONEGAL, F_NOTES,0}},/* guess */
 	{"6020b","RM-31" ,"Nokia 6020b", {F_VOICETAGS, F_FILES2, F_CHAT, F_WAPMMSPROXY, F_TODO66, F_PBKTONEGAL, F_NOTES,0}},/* guess */
@@ -330,7 +348,8 @@ GSM_PhoneModel allmodels[] = {
 	{"6275i","RM-154","Nokia 6275i", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* fixme */
 	{"6280" ,"RM-78","Nokia 6280", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,F_6230iCALLER,0}},/* fixme */
 	{"6300" ,"RM-217" ,"Nokia 6300", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},
-	{"6303c","RM-443" ,"Nokia 6303c", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},
+	{"6303c","RM-443" ,"Nokia 6303c", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,F_6230iCALLER,0}},
+	{"6303i","RM-638" ,"Nokia 6303i", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},
 	{"6310" ,"NPE-4" ,"Nokia 6310", {F_TODO63,F_CAL65,F_NOMIDI,F_NOMMS,F_VOICETAGS,0}},
 	{"6310i","NPL-1" ,"Nokia 6310i",{F_TODO63,F_CAL65,F_NOMIDI,F_BLUETOOTH,F_NOMMS,F_VOICETAGS,F_PROFILES,0}},
 	{"6340i","RH-13" ,"Nokia 6340i",{F_TODO63,F_CAL65,F_NOMIDI,F_BLUETOOTH,F_NOMMS,F_VOICETAGS,F_PROFILES,0}},
@@ -343,7 +362,9 @@ GSM_PhoneModel allmodels[] = {
 	{"6610i","RM-37" ,"Nokia 6610i",{F_PBKTONEGAL,F_TODO66,F_RADIO,F_SYNCML,0}},
 	{"6600", "RM-325","Nokia 6600", {F_SERIES40_30,F_FILES2,F_TODO66,F_NOTES,F_SMS_FILES,0}},
 	{"6600s","RM-570","Nokia 6600s",{F_SERIES40_30,F_FILES2,F_TODO66,F_NOTES,F_SMS_FILES,0}},
+	{"6600s","RM-414","Nokia 6600s",{F_SERIES40_30,F_FILES2,F_TODO66,F_NOTES,F_SMS_FILES,0}},
 	{"6750-1b","RM-381","Nokia 3750-1b", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* fixme */
+	{"6700c","RM-470","Nokia 6700c",{F_SERIES40_30,F_FILES2,F_TODO66,F_NOTES,F_SMS_FILES,0}},
 	{"6800" ,"NSB-9" ,"Nokia 6800", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKSMSLIST,0}},
 	{"6800" ,"NHL-6" ,"Nokia 6800", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKSMSLIST,0}},
 	{"6810" ,"RM-2"  ,"Nokia 6810", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKSMSLIST,F_NOTES,F_CHAT,F_SYNCML,0}},/* guess */
@@ -353,11 +374,13 @@ GSM_PhoneModel allmodels[] = {
 	{"7200" ,"RH-23" ,"Nokia 7200", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKSMSLIST,F_PBKUSER,0}},/* guess */
 	{"7210" ,"NHL-4" ,"Nokia 7210", {F_PBKTONEGAL,F_TODO66,F_RADIO,0}},
 	{"7210s","RM-436","Nokia 7210s",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* fixme */
+	{"7230","RM-604" ,"Nokia 7230", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,F_6230iCALLER,0}},
 	{"7250" ,"NHL-4J","Nokia 7250", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKIMG,F_SYNCML,0}},
 	{"7250i","NHL-4JX","Nokia 7250i",{F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKIMG,F_SYNCML,0}},
 	{"7260" ,"RM-17" ,"Nokia 7260", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKSMSLIST,F_PBKUSER,F_WAPMMSPROXY,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,0}},
 	{"7270" ,"RM-8"  ,"Nokia 7270", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKSMSLIST,F_PBKUSER,F_WAPMMSPROXY,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,0}},
 	{"7280" ,"RM-14" ,"Nokia 7280", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKSMSLIST,F_PBKUSER,F_WAPMMSPROXY,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,F_6230iWAP,F_6230iCALLER,0}}, /* guess */
+	{"7310c", "RM-379","Nokia 7310c",{F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* guess */
 	{"7360" ,"RM-127","Nokia 7360", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKSMSLIST,F_PBKUSER,F_WAPMMSPROXY,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,0}},
 	{"7370" ,"RM-70" ,"Nokia 7370", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* fixme */
 	{"7373" ,"RM-209","Nokia 7373", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* fixme */
@@ -369,6 +392,9 @@ GSM_PhoneModel allmodels[] = {
 	{"8800" ,"RM-13" ,"Nokia 8800", {F_PBKTONEGAL,F_TODO66,F_RADIO,F_PBKSMSLIST,F_PBKUSER,F_NOTES,F_CHAT,F_SYNCML,F_FILES2,0}},
 	{"8910" ,"NHM-4" ,"Nokia 8910", {F_CAL62,F_NOMIDI,F_NOFILESYSTEM,F_NOMMS,0}},
 	{"8910i","NHM-4" ,"Nokia 8910i",{F_CAL62,F_NOMIDI,F_NOFILESYSTEM,F_NOMMS,0}},
+	{"C2-01","RM-721","Nokia C2-01", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* fixme */
+	{"C3"	,"RM-614","Nokia C3", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* fixme */
+	{"X3"	,"RM-540","Nokia X3", {F_SERIES40_30,F_FILES2,F_TODO66,F_RADIO,F_NOTES,F_SMS_FILES,0}},/* fixme */
 #endif
 #ifdef GSM_ENABLE_NOKIA3320
 	{"3320" ,"NPC-1" ,"Nokia 3320", {F_CAL62,F_DAYMONTH,0}},/* fixme */
@@ -381,6 +407,9 @@ GSM_PhoneModel allmodels[] = {
 	{"9210" ,"RAE-3" ,"",           {0}},
 	{"9210i","RAE-5" ,"",           {0}},
 #endif
+#if defined(GSM_ENABLE_S90) || defined(GSM_ENABLE_ATGEN)
+	{"E52", "RM-469", "Nokia E52", {0}},
+#endif
 #ifdef GSM_ENABLE_ATGEN
 	/* Generic AT */
 	{"at"   ,	  "at",		  "",				   {0}},
@@ -389,7 +418,7 @@ GSM_PhoneModel allmodels[] = {
 	{"7650", "NHL-2", "Nokia 7650", {0}},
 	{"9300", "Nokia 9300", "Nokia 9300", {0}},
 	{"N70", "Nokia E70", "Nokia E70", {0}},
-	{"E61", "Nokia E61", "Nokia E61", {F_ENCODED_USSD, 0}},
+	{"E61", "Nokia E61", "Nokia E61", {0}},
 
 	/* Siemens */
 	{"M20"  ,	  "M20",	  "",				   {F_M20SMS,F_SLOWWRITE,0}},
@@ -411,7 +440,8 @@ GSM_PhoneModel allmodels[] = {
 	{"S45"	,	  "6618" ,	  "",				   {0}},
 	{"ME45" ,	  "3618" ,	  "",				   {0}},
 	{"S55" ,	  "S55" ,	  "",				   {0}},
-	{"S65" ,	  "S65" ,	  "SIEMENS S65",		   {0}},
+	{"S65" ,	  "S65" ,	  "SIEMENS S65",		   {F_SIEMENS_PBK, 0}},
+	{"S68",		  "S68",	  "",				   {F_OBEX,F_SQWE, 0}},
 	{"MC75" ,	  "MC75",	  "",				   {F_NO_ATOBEX, 0}},
 	{"AX75",	  "AX75",	  "SIEMENS AX75",		   {F_SIEMENS_PBK, 0}},
 
@@ -429,8 +459,16 @@ GSM_PhoneModel allmodels[] = {
 	{"D357",	  "SGH-D357" ,	  "",				   {F_EXTRA_PBK_FIELD, 0}},
 	{"J750",	  "SGH-J750" ,	  "",				   {F_NO_ATOBEX, 0}},
 	{"J700",	  "SGH-J700" ,	  "",				   {F_NO_ATOBEX, 0}},
-	{"J700",	  "SAMSUNG SGH-J700" , "",			   {F_NO_ATOBEX, 0}},
+	{"J700",	  "SAMSUNG SGH-J700" , "",			   {F_NO_ATOBEX, F_DISABLE_CMGL, 0}},
 	{"D980",	  "SAMSUNG SGH-D980" , "",			   {F_NO_ATOBEX, 0}},
+	{"B2100",         "SAMSUNG B2100", "", {F_DISABLE_CMGL,
+						F_SMS_SM, F_SMS_ME,
+						F_SMS_UTF8_ENCODED,
+						F_NO_UTF8, F_SAMSUNG_UTF8,
+						F_TSSPCSW, F_OBEX, F_IRMC_LEVEL_2, 0}},
+
+	/* Philips */
+	{"Xenium 9@9d",	  "Xenium 9@9d", "",				   {F_OBEX, 0}},
 
 	/* Ericsson/Sony Ericsson */
 	{"T28s",	"1101101-BVT28s","",				   {F_OBEX, 0}},
@@ -636,6 +674,10 @@ GSM_PhoneModel allmodels[] = {
 	{"W205a",	"AAB-1880011-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"S312",	"AAB-1880013-BV" ,"S312",			   {F_OBEX, F_IRMC_LEVEL_2, F_SUBMIT_SIM_ONLY, 0}},
 	{"W395/W395c",	"AAB-1880014-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"W100i (Spiro)",	"AAB-1880025-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"W100a (Spiro)",	"AAB-1880026-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"W150i (Yendo)",	"AAB-1880027-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"W150a (Yendo)",	"AAB-1880028-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"Z520i/Z520c",	"AAC-1052011-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"Z520a",	"AAC-1052012-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"Z525i",	"AAC-1052013-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
@@ -648,6 +690,7 @@ GSM_PhoneModel allmodels[] = {
 	{"W580i/W580c",	"AAC-1052042-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"W908c",	"AAC-1052051-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"K858c",	"AAC-1052061-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"W302/W302c",	"AAC-1052091-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"W760c",	"AAC-1052101-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"S302",	"AAC-1052121-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"C702a/C702c",	"AAC-1052141-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
@@ -690,11 +733,13 @@ GSM_PhoneModel allmodels[] = {
 	{"TM506",	"AAD-3132011-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"CS8",	"AAD-3152021-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"K850i",	"AAD-3252011-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"K850a",	"AAD-3252012-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"Z750i",	"AAD-3252022-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"Z750a",	"AAD-3252023-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"W760i",	"AAD-3252041-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"Z780i",	"AAD-3252051-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"C905a",	"AAD-3252071-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"G705",	"AAD-3352031-AV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"G705a",	"AAD-3252081-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"C905",	"AAD-3352021-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"G705/G705u",	"AAD-3352031-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
@@ -716,7 +761,7 @@ GSM_PhoneModel allmodels[] = {
 	{"W995",	"AAD-3880023-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"W508a",	"AAD-3880024-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"W715",	"AAD-3880025-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
-	{"C903",	"AAD-3380026-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"C903",	"AAD-3880026-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"C903a",	"AAD-3880027-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"T707",	"AAD-3880029-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"TM717",	"AAD-3880030-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
@@ -728,9 +773,36 @@ GSM_PhoneModel allmodels[] = {
 	{"T715",	"AAD-3880039-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"T715a",	"AAD-3880040-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"F100i (Jalou)",	"AAD-3880041-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"X2i",	"AAD-3880043-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"X2a",	"AAD-3880044-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"U100i (Yari)",	"AAD-3880046-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"U100a (Yari)",	"AAD-3880047-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"X5",	"AAD-3880049-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"J105i (Naite)",	"AAD-3880052-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"J105a (Naite)",	"AAD-3880053-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"X10i",	"AAD-3880056-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"U5i (Vivaz)",	"AAD-3880058-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"U5a (Vivaz)",	"AAD-3880059-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"M1i (Aspen)",	"AAD-3880061-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"M1a (Aspen)",	"AAD-3880062-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"J20i (Hazel)",	"AAD-3880063-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"J10i2 (Elm)",	"AAD-3880065-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"J10",	"AAD-3880067-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"E10a (X10 mini)",	"AAD-3880068-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"E10i (X10 mini)",	"AAD-3880069-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"U5at",	"AAD-3880070-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"U20a (X10 mini pro)",	"AAD-3880071-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"U20i (X10 mini pro)",	"AAD-3880072-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"X10a/SO-01B",	"AAD-3880074-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"U8a (Vivaz Pro)",	"AAD-3880076-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"U8i (Vivaz Pro)",	"AAD-3880077-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"J20",	"AAD-3880080-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"W20i",	"AAD-3880083-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"W20",	"AAD-3880084-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"E15i (X8)",	"AAD-3880091-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"E15a (X8)",	"AAD-3880092-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"J108i (Cedar)",	"AAD-3880095-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"J108a (Cedar)",	"AAD-3880096-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"K790i/K790c",	"AAF-1022011-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"Z500a",	"AAF-1031011-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"S710a",	"AAF-1031021-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
@@ -747,6 +819,17 @@ GSM_PhoneModel allmodels[] = {
 	{"W64S (PTX-930)",	"AAH-5880001-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"SO001 (PTX-932)",	"AAH-5880002-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"SOX01 (PTX-934)",	"AAH-5880003-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SO002 (PTX-934)",	"AAH-5880005-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SO003 (PTX-942)",	"AAH-5880006-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SO004 (PTX-943)",	"AAH-5880007-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SOY04 (PTX-94?)",	"AAH-5880008-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SO005 (PTX-945)",	"AAH-5880009-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SO006 (PTX-947)",	"AAH-5880010-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SOX02 (PTX-952)",	"AAH-5880011-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SOY01 (PTX-931)",	"AAJ-6880001-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SOY02 (PTX-938)",	"AAJ-6880002-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"SOY03 (PTX-941)",	"AAJ-6880003-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
+	{"A8i",	"AAK-7880001-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"GR64",	"BAC-1051012-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"GS64",	"BAC-1051022-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
 	{"GA64",	"BAC-1051023-BV" ,"",				   {F_OBEX, F_SUBMIT_SIM_ONLY, 0}},
@@ -829,7 +912,8 @@ GSM_PhoneModel allmodels[] = {
 
 	/* Other */
 	{"iPAQ" ,	  "iPAQ"  ,	  "" ,				   {0}},
-	{"A2D"  ,	  "A2D"  ,	  "" ,				   {0}},
+	{"A2D"  ,	  "A2D"  ,	  "" ,				   {F_CPIN_NO_OK, 0}},
+	{"MULTIBAND  900E  1800", "MULTIBAND  900E  1800", "",		   {F_CPIN_NO_OK, 0}},
 	{"9210" ,	  "RAE-3",	  "Nokia Communicator GSM900/1800",{0}},
 	{"MTK1" ,	  "MTK1",	  "",				   {0}},
 	{"MTK2" ,	  "MTK2",	  "",				   {0}},
@@ -845,6 +929,7 @@ GSM_PhoneModel allmodels[] = {
 	{"E17X",	"E17X",		"",		{F_NO_UCS2, 0}},
 	{"E220",	"E220",		"",		{0}},
 	{"E160",	"E160",		"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
+	{"E160X",	"E160X",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
 	{"E161",	"E161",		"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
 	{"E162",	"E162",		"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
 	{"E160G",	"E160G",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
@@ -852,6 +937,9 @@ GSM_PhoneModel allmodels[] = {
 	{"E169",	"E169",		"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
 	{"E169G",	"E169G",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
 	{"E1556",	"E1556",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
+	{"E1550",	"E1550",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
+	{"E1750",	"E1750",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
+	{"EM770W",	"EM770W",	"",		{F_SMS_LOCATION_0, F_ENCODED_USSD, F_FOUR_DIGIT_YEAR, 0}},
 
 	/* Ubinetics */
 	{"GDC201",	"GDC201",	"",		{F_SMS_NO_ME, 0}},
@@ -894,14 +982,14 @@ GSM_PhoneModel *GetModelData(GSM_StateMachine *s, const char *model, const char
 
 gboolean GSM_IsPhoneFeatureAvailable(GSM_PhoneModel *model, GSM_Feature feature)
 {
-	int i = 0;
+	int i;
 
-	while (model->features[i] != 0) {
+	for (i = 0; model->features[i] != 0; i++) {
 		if (model->features[i] == feature) {
 			return TRUE;
 		}
-		i++;
 	}
+
 	return FALSE;
 }
 
@@ -909,15 +997,15 @@ gboolean GSM_AddPhoneFeature(GSM_PhoneModel *model, GSM_Feature feature)
 {
 	int	i	= 0;
 
-	while (model->features[i] != 0) {
+	for (i = 0; model->features[i] != 0; i++) {
 		if (model->features[i] == feature) {
 			return TRUE;
 		}
-		i++;
 	}
+
 	if (i == GSM_MAX_PHONE_FEATURES) return FALSE;
 	model->features[i++] = feature;
-	model->features[i++] = 0;
+	model->features[i] = 0;
 	return TRUE;
 }
 
