@@ -40,13 +40,17 @@
 #define FALLBACK_GAMMURC "/etc/gammurc"
 #define GAMMURC_NAME "/.gammurc"
 #endif
+#define XDG_GAMMURC_NAME "/gammu/config"
 
 /* Win32 compatibility */
 #ifndef PATH_MAX
 #define PATH_MAX (MAX_PATH)
 #endif
 
-
+/**
+ * Returns current debuging descriptor. It honors use_global
+ * flag.
+ */
 GSM_Debug_Info *GSM_GetDI(GSM_StateMachine *s)
 {
 	GSM_Debug_Info *curdi;
@@ -93,13 +97,20 @@ static const GSM_ConnectionInfo GSM_Connections[] = {
 
 	/* cables */
 	{"mbus", GCT_MBUS2, FALSE},
+
 	{"fbus", GCT_FBUS2, FALSE},
+
 	{"fbususb", GCT_FBUS2USB, FALSE},
+
 	{"fbuspl2303", GCT_FBUS2PL2303, FALSE},
+
 	{"dlr3", GCT_FBUS2DLR3, FALSE},
 	{"fbusdlr3", GCT_FBUS2DLR3, FALSE},
+
 	{"dku5", GCT_DKU5FBUS2, FALSE},
+	{"fbusdku5", GCT_DKU5FBUS2, FALSE},
 	{"dku5fbus", GCT_DKU5FBUS2, FALSE},
+
 	{"ark3116fbus", GCT_DKU5FBUS2, TRUE},
 #ifdef WIN32
 	{"dku2", GCT_DKU2PHONET, FALSE},
@@ -124,6 +135,8 @@ static const GSM_ConnectionInfo GSM_Connections[] = {
 	{"bluerfphonet", GCT_BLUEPHONET, FALSE},
 	{"bluerfat", GCT_BLUEAT, FALSE},
 	{"bluerfgnapbus", GCT_BLUEGNAPBUS, FALSE},
+	{"blues60", GCT_BLUES60, FALSE},
+	{"bluerfs60", GCT_BLUES60, FALSE},
 
 	/* old "serial" irda */
 	{"infrared", GCT_FBUS2IRDA, FALSE},
@@ -274,6 +287,9 @@ static GSM_Error GSM_RegisterAllConnections(GSM_StateMachine *s, const char *con
 #ifdef GSM_ENABLE_IRDAOBEX
 	GSM_RegisterConnection(s, GCT_IRDAOBEX,   &IrdaDevice,    &OBEXProtocol);
 #endif
+#ifdef GSM_ENABLE_BLUEGNAPBUS
+	GSM_RegisterConnection(s, GCT_BLUES60,	  &BlueToothDevice,&S60Protocol);
+#endif
 #ifdef GSM_ENABLE_BLUEGNAPBUS
 	GSM_RegisterConnection(s, GCT_BLUEGNAPBUS,&BlueToothDevice,&GNAPBUSProtocol);
 #endif
@@ -376,6 +392,13 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 			return ERR_NONE;
 		}
 #endif
+#ifdef GSM_ENABLE_S60
+		if (s->ConnectionType == GCT_BLUES60) {
+			smprintf(s,"[Module           - \"%s\"]\n",S60Phone.models);
+			s->Phone.Functions = &S60Phone;
+			return ERR_NONE;
+		}
+#endif
 
 #ifdef GSM_ENABLE_NOKIA6510
 		if ( s->ConnectionType ==  GCT_MBUS2 ||
@@ -447,6 +470,9 @@ GSM_Error GSM_RegisterAllPhoneModules(GSM_StateMachine *s)
 #ifdef GSM_ENABLE_GNAPGEN
 	GSM_RegisterModule(s,&GNAPGENPhone);
 #endif
+#ifdef GSM_ENABLE_S60
+	GSM_RegisterModule(s,&S60Phone);
+#endif
 #ifdef GSM_ENABLE_NOKIA3320
 	GSM_RegisterModule(s,&N3320Phone);
 #endif
@@ -583,6 +609,11 @@ GSM_Error GSM_TryGetModel(GSM_StateMachine *s)
 				s->Phone.Functions = &GNAPGENPhone;
 				break;
 #endif
+#ifdef GSM_ENABLE_S60
+			case GCT_BLUES60:
+				s->Phone.Functions = &S60Phone;
+				break;
+#endif
 #if defined(GSM_ENABLE_NOKIA_DCT3) || defined(GSM_ENABLE_NOKIA_DCT4)
 			case GCT_MBUS2:
 			case GCT_FBUS2:
@@ -679,7 +710,7 @@ GSM_Error GSM_InitConnection_Log(GSM_StateMachine *s, int ReplyNum, GSM_Log_Func
 		}
 
 		smprintf_level(s, D_ERROR, "[Gammu            - %s built %s %s using %s]\n",
-				VERSION,
+				GAMMU_VERSION,
 				__TIME__,
 				__DATE__,
 				GetCompiler()
@@ -695,13 +726,13 @@ GSM_Error GSM_InitConnection_Log(GSM_StateMachine *s, int ReplyNum, GSM_Log_Func
 		smprintf_level(s, D_ERROR, "[Device           - \"%s\"]\n",
 				s->CurrentConfig->Device);
 		if (strlen(GetOS()) != 0) {
-			smprintf_level(s, D_ERROR, "[Runing on        - %s]\n",
+			smprintf_level(s, D_ERROR, "[Running on       - %s]\n",
 					GetOS());
 		}
 
 		if (GSM_GetDI(s)->dl == DL_BINARY) {
-			smprintf(s,"%c",((unsigned char)strlen(VERSION)));
-			smprintf(s,"%s",VERSION);
+			smprintf(s,"%c",((unsigned char)strlen(GAMMU_VERSION)));
+			smprintf(s,"%s",GAMMU_VERSION);
 		}
 
 		error = GSM_RegisterAllConnections(s, s->CurrentConfig->Connection);
@@ -712,8 +743,16 @@ GSM_Error GSM_InitConnection_Log(GSM_StateMachine *s, int ReplyNum, GSM_Log_Func
 
 autodetect:
 		/* Model auto */
-		if (s->CurrentConfig->Model[0] == 0) {
+		/* Try to guess correct driver based on model */
+		if (s->CurrentConfig->Model[0] == 0 &&
+				s->ConnectionType != GCT_NONE &&
+				s->ConnectionType != GCT_IRDAOBEX &&
+				s->ConnectionType != GCT_BLUEOBEX &&
+				s->ConnectionType != GCT_BLUEGNAPBUS &&
+				s->ConnectionType != GCT_IRDAGNAPBUS &&
+				s->ConnectionType != GCT_BLUES60) {
 			error = GSM_TryGetModel(s);
+			/* Fall back to other configuraitons if the device is not existing (or similar error) */
 			if ((i != s->ConfigNum - 1) && (
 				(error == ERR_DEVICEOPENERROR) ||
 				(error == ERR_DEVICELOCKED) ||
@@ -827,12 +866,12 @@ int GSM_ReadDevice (GSM_StateMachine *s, gboolean waitforreply)
 	int		res=0,count=0,i=0;
 
 	if (!GSM_IsConnected(s)) {
-		return ERR_NOTCONNECTED;
+		return -1;
 	}
 
 	GSM_GetCurrentDateTime (&Date);
 	i=Date.Second;
-	while (i==Date.Second) {
+	while (i==Date.Second && !s->Abort) {
 		res = s->Device.Functions->ReadDevice(s, buff, sizeof(buff));
 
 		if (!waitforreply) {
@@ -854,7 +893,7 @@ GSM_Error GSM_TerminateConnection(GSM_StateMachine *s)
 {
 	GSM_Error error;
 
-	if (!s->opened) return ERR_UNKNOWN;
+	if (!s->opened) return ERR_NONE;
 
 	smprintf(s,"[Terminating]\n");
 
@@ -881,8 +920,14 @@ gboolean GSM_IsConnected(GSM_StateMachine *s) {
 	return (s != NULL) && s->Phone.Functions != NULL && s->opened;
 }
 
+GSM_Error GSM_AbortOperation(GSM_StateMachine * s)
+{
+	s->Abort = TRUE;
+	return ERR_NONE;
+}
+
 GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
-			  int length, unsigned char type, int timeout)
+			  int length, int type, int timeout)
 {
 	GSM_Phone_Data *Phone = &s->Phone.Data;
 	GSM_Protocol_Message sentmsg;
@@ -901,6 +946,9 @@ GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
 		if (GSM_ReadDevice(s, TRUE) > 0) {
 			i = 0;
 		} else {
+			if (s->Abort) {
+				return ERR_ABORTED;
+			}
 			usleep(10000);
 		}
 
@@ -921,7 +969,7 @@ GSM_Error GSM_WaitForOnce(GSM_StateMachine *s, unsigned const char *buffer,
 }
 
 GSM_Error GSM_WaitFor (GSM_StateMachine *s, unsigned const char *buffer,
-		       int length, unsigned char type, int timeout,
+		       int length, int type, int timeout,
 		       GSM_Phone_RequestID request)
 {
 	GSM_Phone_Data		*Phone = &s->Phone.Data;
@@ -945,6 +993,11 @@ GSM_Error GSM_WaitFor (GSM_StateMachine *s, unsigned const char *buffer,
 		error = s->Protocol.Functions->WriteMessage(s, buffer, length, type);
 		if (error!=ERR_NONE) return error;
 
+		/* Special case when no reply is expected */
+		if (request == ID_None) {
+			return ERR_NONE;
+		}
+
 		error = GSM_WaitForOnce(s, buffer, length, type, timeout);
 		if (error != ERR_TIMEOUT) return error;
         }
@@ -962,8 +1015,13 @@ static GSM_Error CheckReplyFunctions(GSM_StateMachine *s, GSM_Reply_Function *Re
 
 	while (Reply[i].requestID!=ID_None) {
 		execute=FALSE;
+		/* Long ID frames like S60 */
+		if (Reply[i].msgtype[0] == 0 && Reply[i].subtypechar == 0) {
+			if (Reply[i].subtype == msg->Type) {
+				execute = TRUE;
+			}
 		/* Binary frames like in Nokia */
-		if (strlen(Reply[i].msgtype) < 2) {
+		} else if (strlen(Reply[i].msgtype) < 2) {
 			if (Reply[i].msgtype[0]==msg->Type) {
 				if (Reply[i].subtypechar!=0) {
 					if (Reply[i].subtypechar<=msg->Length) {
@@ -1020,7 +1078,7 @@ GSM_Error GSM_DispatchMessage(GSM_StateMachine *s)
 	}
 
 	if (error==ERR_NONE) {
-		error=Reply[reply].Function(*msg, s);
+		error=Reply[reply].Function(msg, s);
 		if (Reply[reply].requestID==Phone->RequestID) {
 			if (error == ERR_NEEDANOTHERANSWER) {
 				error = ERR_NONE;
@@ -1098,13 +1156,30 @@ GSM_Error GSM_FindGammuRC (INI_Section **result, const char *force_config)
 	}
 #endif
 
-	/* Reset as we're using strcat */
-	configfile[0] = 0;
+	/* XDG paths */
+	envpath = getenv("XDG_CONFIG_HOME");
+	if (envpath) {
+		strcpy(configfile, envpath);
+		strcat(configfile, XDG_GAMMURC_NAME);
 
-	/* Try user home */
+		error = GSM_TryReadGammuRC(configfile, result);
+		if (error == ERR_NONE) return ERR_NONE;
+	} else {
+		envpath  = getenv("HOME");
+		if (envpath) {
+			strcpy(configfile, envpath);
+			strcat(configfile, "/.config");
+			strcat(configfile, XDG_GAMMURC_NAME);
+
+			error = GSM_TryReadGammuRC(configfile, result);
+			if (error == ERR_NONE) return ERR_NONE;
+		}
+	}
+
+	/* Try home from environment */
 	envpath  = getenv("HOME");
 	if (envpath) {
-		strcat(configfile, envpath);
+		strcpy(configfile, envpath);
 		strcat(configfile, GAMMURC_NAME);
 
 		error = GSM_TryReadGammuRC(configfile, result);
@@ -1112,9 +1187,10 @@ GSM_Error GSM_FindGammuRC (INI_Section **result, const char *force_config)
 	}
 
 #if defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
+	/* Tru home from passwd */
 	pwent = getpwuid(getuid());
 	if (pwent != NULL) {
-		strcat(configfile, pwent->pw_dir);
+		strcpy(configfile, pwent->pw_dir);
 		strcat(configfile, GAMMURC_NAME);
 
 		error = GSM_TryReadGammuRC(configfile, result);
@@ -1206,10 +1282,10 @@ GSM_Error GSM_ReadConfig(INI_Section *cfg_info, GSM_Config *cfg, int num)
 #if defined(WIN32) || defined(DJGPP)
         static const char *DefaultPort		= "com2:";
 #else
-        static const char *DefaultPort		= "/dev/ttyS1";
+        static const char *DefaultPort		= "/dev/ttyACM0";
 #endif
         static const char *DefaultModel		= "";
-        static const char *DefaultConnection		= "fbus";
+        static const char *DefaultConnection		= "at";
 	static gboolean DefaultSynchronizeTime	= FALSE;
 	static const char *DefaultDebugFile		= "";
 	static const char *DefaultDebugLevel		= "";
@@ -1626,6 +1702,7 @@ GSM_StateMachine *GSM_AllocStateMachine(void)
 	GSM_StateMachine *ret;
 	ret = (GSM_StateMachine *)calloc(1, sizeof(GSM_StateMachine));
 	ret->CurrentConfig = &(ret->Config[0]);
+	ret->Abort = FALSE;
 	return ret;
 }
 
