@@ -27,6 +27,7 @@
 #include <gammu-misc.h>
 #include <gammu-category.h>
 #include <gammu-backup.h>
+#include <string.h>
 
 typedef struct _GSM_User	 	GSM_User;
 
@@ -66,6 +67,9 @@ typedef struct _GSM_User	 	GSM_User;
 #ifdef GSM_ENABLE_GNAPGEN
 #  include "phone/symbian/gnapgen.h"
 #endif
+#ifdef GSM_ENABLE_S60
+#  include "phone/s60/s60phone.h"
+#endif
 #  include "phone/dummy/dummy.h"
 
 #ifndef GSM_USED_MBUS2
@@ -151,6 +155,9 @@ typedef struct _GSM_User	 	GSM_User;
 #if defined(GSM_ENABLE_BLUEGNAPBUS) || defined(GSM_ENABLE_IRDAGNAPBUS)
 #  include "protocol/symbian/gnapbus.h"
 #endif
+#if defined(GSM_ENABLE_S60)
+#  include "protocol/s60/s60.h"
+#endif
 
 #define GSM_ENABLE_SERIALDEVICE
 #ifndef GSM_USED_SERIALDEVICE
@@ -166,7 +173,9 @@ typedef struct _GSM_User	 	GSM_User;
 #ifndef GSM_USED_IRDADEVICE
 #  undef GSM_ENABLE_IRDADEVICE
 #endif
-#define GSM_ENABLE_BLUETOOTHDEVICE
+#if defined(BLUEZ_FOUND) || defined(BSD_BLUE_FOUND) || defined(OSX_BLUE_FOUND) || defined(WIN32)
+#  define GSM_ENABLE_BLUETOOTHDEVICE
+#endif
 #ifndef GSM_USED_BLUETOOTHDEVICE
 #  undef GSM_ENABLE_BLUETOOTHDEVICE
 #endif
@@ -321,7 +330,7 @@ typedef struct {
 	 * Writes message to device.
 	 */
 	GSM_Error (*WriteMessage) (GSM_StateMachine *s, unsigned const char *buffer,
-				   int length, unsigned char type);
+				   int length, int type);
 	/**
 	 * This one is called when character is received from device.
 	 */
@@ -357,6 +366,9 @@ typedef struct {
 #if defined(GSM_ENABLE_BLUEGNAPBUS) || defined(GSM_ENABLE_IRDAGNAPBUS)
 	extern GSM_Protocol_Functions GNAPBUSProtocol;
 #endif
+#if defined(GSM_ENABLE_S60)
+	extern GSM_Protocol_Functions S60Protocol;
+#endif
 
 /**
  * Structure containing protocol specific data and pointer to protocol
@@ -386,6 +398,9 @@ typedef struct {
 #endif
 #if defined(GSM_ENABLE_BLUEGNAPBUS) || defined(GSM_ENABLE_IRDAGNAPBUS)
 		GSM_Protocol_GNAPBUSData	GNAPBUS;
+#endif
+#if defined(GSM_ENABLE_S60)
+		GSM_Protocol_S60Data		S60;
 #endif
 	} Data;
 	/**
@@ -521,6 +536,10 @@ typedef struct {
 	 * Calendar status.
 	 */
 	GSM_CalendarStatus	*CalStatus;
+	/**
+	 * Todo status.
+	 */
+	GSM_ToDoStatus		*ToDoStatus;
 	/**
 	 * Used internally by phone drivers.
 	 */
@@ -553,6 +572,10 @@ typedef struct {
 	 * Pointer to note structure used internally by phone drivers.
 	 */
 	GSM_NoteEntry		*Note;
+	/**
+	 * Pointer to picture structure used internally by phone drivers.
+	 */
+	GSM_BinaryPicture	*Picture;
 	/**
 	 * Used internally by phone drivers.
 	 */
@@ -686,6 +709,9 @@ typedef struct {
 #endif
 #ifdef GSM_ENABLE_GNAPGEN
 		GSM_Phone_GNAPGENData 	 GNAPGEN;
+#endif
+#ifdef GSM_ENABLE_S60
+		GSM_Phone_S60Data	 S60;
 #endif
 		GSM_Phone_DUMMYData	 DUMMY;
 	} Priv;
@@ -705,6 +731,10 @@ typedef struct {
 	 * @ref GSM_Reply_Function for details about it.
 	 */
 	GSM_Reply_Function	      	*ReplyFunctions;
+	/**
+	 * Installs required applets to the phone.
+	 */
+	GSM_Error (*Install)	 	(GSM_StateMachine *s, const char *ExtraPath);
 	/**
 	 * Initializes phone.
 	 */
@@ -802,7 +832,7 @@ typedef struct {
 	/**
 	 * Enters security code (PIN, PUK,...) .
 	 */
-	GSM_Error (*EnterSecurityCode)  (GSM_StateMachine *s, GSM_SecurityCode Code);
+	GSM_Error (*EnterSecurityCode)  (GSM_StateMachine *s, GSM_SecurityCode *Code);
 	/**
 	 * Queries whether some security code needs to be entered./
 	 */
@@ -1254,6 +1284,10 @@ typedef struct {
 	 * Sets GPRS access point.
 	 */
 	GSM_Error (*SetGPRSAccessPoint) (GSM_StateMachine *s, GSM_GPRSAccessPoint *point);
+	/**
+	 * Gets phone screenshot
+	 */
+	GSM_Error (*GetScreenshot)	(GSM_StateMachine *s, GSM_BinaryPicture *picture);
 } GSM_Phone_Functions;
 
 	extern GSM_Phone_Functions NAUTOPhone;
@@ -1292,6 +1326,9 @@ typedef struct {
 #endif
 #ifdef GSM_ENABLE_GNAPGEN
 	extern GSM_Phone_Functions GNAPGENPhone;
+#endif
+#ifdef GSM_ENABLE_S60
+	extern GSM_Phone_Functions S60Phone;
 #endif
 	extern GSM_Phone_Functions DUMMYPhone;
 
@@ -1353,6 +1390,11 @@ struct _GSM_StateMachine {
 	int			ReplyNum;				/**< How many times make sth. 			*/
 	int			Speed;					/**< For some protocols used speed		*/
 
+	/**
+	 * Flag for interrupting communication.
+	 */
+	volatile gboolean Abort;
+
 	GSM_Device		Device; /**< Device driver data and functions */
 	GSM_Protocol		Protocol; /**< Protocol driver data and functions */
 	GSM_Phone		Phone; /**< Phone driver data and functions */
@@ -1371,7 +1413,7 @@ struct _GSM_StateMachine {
 GSM_Error GSM_RegisterAllPhoneModules	(GSM_StateMachine *s);
 
 GSM_Error GSM_WaitForOnce		(GSM_StateMachine *s, unsigned const char *buffer,
-			  		 int length, unsigned char type, int timeout);
+			  		 int length, int type, int timeout);
 
 /**
  * Wait for reply from the phone.
@@ -1386,7 +1428,7 @@ GSM_Error GSM_WaitForOnce		(GSM_StateMachine *s, unsigned const char *buffer,
  * \return Error code, ERR_NONE on sucecss.
  */
 GSM_Error GSM_WaitFor			(GSM_StateMachine *s, unsigned const char *buffer,
-		       			 int length, unsigned char type, int timeout,
+		       			 int length, int type, int timeout,
 					 GSM_Phone_RequestID request) WARNUNUSED;
 
 /**
