@@ -38,96 +38,93 @@
 #include "coding.h"
 
 /* function changes #10 #13 chars to \n \r */
-char *EncodeUnicodeSpecialChars(const unsigned char *buffer)
+unsigned char *EncodeUnicodeSpecialChars(unsigned char *dest, const unsigned char *buffer)
 {
 	int 			Pos=0, Pos2=0;
-	static unsigned char	Buf[20000];
 
 	while (buffer[Pos*2]!=0x00 || buffer[Pos*2+1]!=0x00) {
 		if (buffer[Pos*2] == 0x00 && buffer[Pos*2+1] == 10) {
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = '\\';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = '\\';
 			Pos2++;
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = 'n';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = 'n';
 			Pos2++;
 		} else if (buffer[Pos*2] == 0x00 && buffer[Pos*2+1] == 13) {
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = '\\';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = '\\';
 			Pos2++;
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = 'r';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = 'r';
 			Pos2++;
 		} else if (buffer[Pos*2] == 0x00 && buffer[Pos*2+1] == '\\') {
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = '\\';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = '\\';
 			Pos2++;
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = '\\';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = '\\';
 			Pos2++;
 		} else if (buffer[Pos*2] == 0x00 && buffer[Pos*2+1] == ';') {
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = '\\';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = '\\';
 			Pos2++;
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = ';';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = ';';
 			Pos2++;
 		} else if (buffer[Pos*2] == 0x00 && buffer[Pos*2+1] == ',') {
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = '\\';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = '\\';
 			Pos2++;
-			Buf[Pos2*2]   = 0x00;
-			Buf[Pos2*2+1] = ',';
+			dest[Pos2*2]   = 0x00;
+			dest[Pos2*2+1] = ',';
 			Pos2++;
 		} else {
-			Buf[Pos2*2]   = buffer[Pos*2];
-			Buf[Pos2*2+1] = buffer[Pos*2+1];
+			dest[Pos2*2]   = buffer[Pos*2];
+			dest[Pos2*2+1] = buffer[Pos*2+1];
 			Pos2++;
 		}
 		Pos++;
 	}
-	Buf[Pos2*2]   = 0;
-	Buf[Pos2*2+1] = 0;
-	return Buf;
+	dest[Pos2*2]   = 0;
+	dest[Pos2*2+1] = 0;
+	return dest;
 }
 
 /* function changes #10 #13 chars to \n \r */
-char *EncodeSpecialChars(const char *buffer)
+char *EncodeSpecialChars(char *dest, const char *buffer)
 {
 	int 			Pos=0, Pos2=0;
-	static unsigned char	Buf[10000];
 
 	while (buffer[Pos]!=0x00) {
 		switch (buffer[Pos]) {
 		case 10:
-			Buf[Pos2++] = '\\';
-			Buf[Pos2++] = 'n';
+			dest[Pos2++] = '\\';
+			dest[Pos2++] = 'n';
 			break;
 		case 13:
-			Buf[Pos2++] = '\\';
-			Buf[Pos2++] = 'r';
+			dest[Pos2++] = '\\';
+			dest[Pos2++] = 'r';
 			break;
 		case '\\':
-			Buf[Pos2++] = '\\';
-			Buf[Pos2++] = '\\';
+			dest[Pos2++] = '\\';
+			dest[Pos2++] = '\\';
 			break;
 		default:
-			Buf[Pos2++] = buffer[Pos];
+			dest[Pos2++] = buffer[Pos];
 		}
 		Pos++;
 	}
-	Buf[Pos2] = 0;
-	return Buf;
+	dest[Pos2] = 0;
+	return dest;
 }
 
-char *DecodeUnicodeSpecialChars(const unsigned char *buffer)
+unsigned char *DecodeUnicodeSpecialChars(unsigned char *dest, const unsigned char *buffer)
 {
 	int 			Pos=0, Pos2=0, level=0;
-	static unsigned char	Buf[10000];
 
 	while (buffer[Pos*2]!=0x00 || buffer[Pos*2+1]!=0x00) {
-		Buf[Pos2*2]   = buffer[Pos*2];
-		Buf[Pos2*2+1] = buffer[Pos*2+1];
+		dest[Pos2*2]   = buffer[Pos*2];
+		dest[Pos2*2+1] = buffer[Pos*2+1];
 		switch (level) {
 		case 0:
 			if (buffer[Pos*2] == 0x00 && buffer[Pos*2+1] == '\\') {
@@ -138,34 +135,33 @@ char *DecodeUnicodeSpecialChars(const unsigned char *buffer)
 			break;
 		case 1:
 			if (buffer[Pos*2] == 0x00 && buffer[Pos*2+1] == 'n') {
-				Buf[Pos2*2]   = 0;
-				Buf[Pos2*2+1] = 10;
+				dest[Pos2*2]   = 0;
+				dest[Pos2*2+1] = 10;
 			} else
 			if (buffer[Pos*2] == 0x00 && buffer[Pos*2+1] == 'r') {
-				Buf[Pos2*2]   = 0;
-				Buf[Pos2*2+1] = 13;
+				dest[Pos2*2]   = 0;
+				dest[Pos2*2+1] = 13;
 			} else
 			if (buffer[Pos*2] == 0x00 && buffer[Pos*2+1] == '\\') {
-				Buf[Pos2*2]   = 0;
-				Buf[Pos2*2+1] = '\\';
+				dest[Pos2*2]   = 0;
+				dest[Pos2*2+1] = '\\';
 			}
 			Pos2++;
 			level = 0;
 		}
 		Pos++;
 	}
-	Buf[Pos2*2]   = 0;
-	Buf[Pos2*2+1] = 0;
-	return Buf;
+	dest[Pos2*2]   = 0;
+	dest[Pos2*2+1] = 0;
+	return dest;
 }
 
-char *DecodeSpecialChars(const char *buffer)
+char *DecodeSpecialChars(char *dest, const char *buffer)
 {
 	int 			Pos=0, Pos2=0, level=0;
-	static unsigned char	Buf[10000];
 
 	while (buffer[Pos]!=0x00) {
-		Buf[Pos2] = buffer[Pos];
+		dest[Pos2] = buffer[Pos];
 		switch (level) {
 		case 0:
 			if (buffer[Pos] == '\\') {
@@ -175,16 +171,16 @@ char *DecodeSpecialChars(const char *buffer)
 			}
 			break;
 		case 1:
-			if (buffer[Pos] == 'n') Buf[Pos2] = 10;
-			if (buffer[Pos] == 'r') Buf[Pos2] = 13;
-			if (buffer[Pos] == '\\') Buf[Pos2] = '\\';
+			if (buffer[Pos] == 'n') dest[Pos2] = 10;
+			if (buffer[Pos] == 'r') dest[Pos2] = 13;
+			if (buffer[Pos] == '\\') dest[Pos2] = '\\';
 			Pos2++;
 			level = 0;
 		}
 		Pos++;
 	}
-	Buf[Pos2] = 0;
-	return Buf;
+	dest[Pos2] = 0;
+	return dest;
 }
 
 size_t UnicodeLength(const unsigned char *str)
@@ -331,7 +327,7 @@ void DecodeBCD (unsigned char *dest, const unsigned char *src, int len)
 	        digit=src[i] >> 4;
                 if (digit<10) dest[current++]=digit + '0';
 	}
-	dest[current++]=0;
+	dest[current]=0;
 }
 
 void EncodeBCD (unsigned char *dest, const unsigned char *src, int len, gboolean fill)
@@ -367,7 +363,7 @@ int DecodeWithHexBinAlphabet (unsigned char mychar)
 	return -1;
 }
 
-unsigned char EncodeWithHexBinAlphabet (int digit)
+char EncodeWithHexBinAlphabet (int digit)
 {
 	if (digit >= 0 && digit <= 9) return '0'+(digit);
 	if (digit >=10 && digit <=15) return 'A'+(digit-10);
@@ -387,20 +383,12 @@ void DecodeHexUnicode (unsigned char *dest, const char *src, size_t len)
 			DecodeWithHexBinAlphabet(src[i + 3]);
 	}
 	dest[current++] = 0;
-	dest[current++] = 0;
+	dest[current] = 0;
 }
 
 void EncodeHexUnicode (char *dest, const unsigned char *src, size_t len)
 {
-	size_t i, current = 0;
-
-	for (i = 0; i < len; i++) {
-		dest[current++] = EncodeWithHexBinAlphabet(src[2*i] >> 4);
-		dest[current++] = EncodeWithHexBinAlphabet(src[2*i] & 0x0f);
-		dest[current++] = EncodeWithHexBinAlphabet(src[2*i+1] >> 4);
-		dest[current++] = EncodeWithHexBinAlphabet(src[2*i+1] & 0x0f);
-	}
-	dest[current++] = 0;
+	EncodeHexBin(dest, src, len * 2);
 }
 
 gboolean DecodeHexBin (unsigned char *dest, const unsigned char *src, int len)
@@ -413,19 +401,19 @@ gboolean DecodeHexBin (unsigned char *dest, const unsigned char *src, int len)
 		if (low < 0 || high < 0) return FALSE;
 		dest[current++] = (high << 4) | low;
 	}
-	dest[current++] = 0;
+	dest[current] = 0;
 	return TRUE;
 }
 
-void EncodeHexBin (unsigned char *dest, const unsigned char *src, int len)
+void EncodeHexBin (char *dest, const unsigned char *src, size_t len)
 {
-	int i,current=0;
+	size_t i, outpos = 0;
 
 	for (i = 0; i < len; i++) {
-		dest[current++] = EncodeWithHexBinAlphabet(src[i] >> 0x04);
-		dest[current++] = EncodeWithHexBinAlphabet(src[i] & 0x0f);
+		dest[outpos++] = EncodeWithHexBinAlphabet(src[i] >> 4);
+		dest[outpos++] = EncodeWithHexBinAlphabet(src[i] & 0xF);
 	}
-	dest[current++] = 0;
+	dest[outpos] = 0;
 }
 
 /* ETSI GSM 03.38, section 6.2.1: Default alphabet for SMS messages */
@@ -471,67 +459,65 @@ static unsigned char GSM_DefaultAlphabetUnicode[128+1][2] =
  * 0x1b, 0x65) are visible as one single additional char (for example,
  * 0x1b, 0x65 gives Euro char saved in Unicode as 0x20, 0xAC)
  * This table contains:
- * 1. two first chars means sequence of chars from GSM default alphabet
+ * 1. two first char means second char from the sequence of chars from GSM default alphabet (first being 0x1b)
  * 2. two second is target (encoded) char saved in Unicode
  */
-static unsigned char GSM_DefaultAlphabetCharsExtension[][4] =
+static unsigned char GSM_DefaultAlphabetCharsExtension[][3] =
 {
-	{0x1b,0x14,0x00,0x5e},	/* ^	*/
-	{0x1b,0x28,0x00,0x7b},	/* {	*/
-	{0x1b,0x29,0x00,0x7d},	/* }	*/
-	{0x1b,0x2f,0x00,0x5c},	/* \	*/
-	{0x1b,0x3c,0x00,0x5b},	/* [	*/
-	{0x1b,0x3d,0x00,0x7E},	/* ~	*/
-	{0x1b,0x3e,0x00,0x5d},	/* ]	*/
-	{0x1b,0x40,0x00,0x7C},	/* |	*/
-	{0x1b,0x65,0x20,0xAC},	/* Euro */
-	{0x00,0x00,0x00,0x00}
+	{0x0a,0x00,0x0c},	/* \r	*/
+	{0x14,0x00,0x5e},	/* ^	*/
+	{0x28,0x00,0x7b},	/* {	*/
+	{0x29,0x00,0x7d},	/* }	*/
+	{0x2f,0x00,0x5c},	/* \	*/
+	{0x3c,0x00,0x5b},	/* [	*/
+	{0x3d,0x00,0x7E},	/* ~	*/
+	{0x3e,0x00,0x5d},	/* ]	*/
+	{0x40,0x00,0x7C},	/* |	*/
+	{0x65,0x20,0xAC},	/* Euro */
+	{0x00,0x00,0x00}
 };
 
 void DecodeDefault (unsigned char *dest, const unsigned char *src, size_t len, gboolean UseExtensions, unsigned char *ExtraAlphabet)
 {
-	size_t 	i,current=0,j;
-	gboolean	FoundSpecial = FALSE;
+	size_t 	pos, current = 0, i;
 
 #ifdef DEBUG
 	DumpMessageText(&GSM_global_debug, src, len);
 #endif
 
-	for (i = 0; i < len; i++) {
-		FoundSpecial = FALSE;
-		if ((i < (len-1)) && UseExtensions) {
-			j=0;
-			while (GSM_DefaultAlphabetCharsExtension[j][0]!=0x00) {
-				if (GSM_DefaultAlphabetCharsExtension[j][0]==src[i] &&
-				    GSM_DefaultAlphabetCharsExtension[j][1]==src[i+1]) {
-					FoundSpecial = TRUE;
-					dest[current++] = GSM_DefaultAlphabetCharsExtension[j][2];
-					dest[current++] = GSM_DefaultAlphabetCharsExtension[j][3];
-					i++;
+	for (pos = 0; pos < len; pos++) {
+		if ((pos < (len - 1)) && UseExtensions && src[pos] == 0x1b) {
+			for (i = 0; GSM_DefaultAlphabetCharsExtension[i][0] != 0x00; i++) {
+				if (GSM_DefaultAlphabetCharsExtension[i][0] == src[pos + 1]) {
+					dest[current++] = GSM_DefaultAlphabetCharsExtension[i][1];
+					dest[current++] = GSM_DefaultAlphabetCharsExtension[i][2];
+					pos++;
 					break;
 				}
-				j++;
+			}
+			/* Skip rest if we've found something */
+			if (GSM_DefaultAlphabetCharsExtension[i][0] != 0x00) {
+				continue;
 			}
 		}
-       		if (ExtraAlphabet!=NULL && !FoundSpecial) {
-			j = 0;
-			while (ExtraAlphabet[j] != 0x00 || ExtraAlphabet[j+1] != 0x00 || ExtraAlphabet[j+2] != 0x00) {
-				if (ExtraAlphabet[j] == src[i]) {
-					dest[current++] = ExtraAlphabet[j+1];
-					dest[current++] = ExtraAlphabet[j+2];
-					FoundSpecial 	= TRUE;
-                            		break;
-                        	}
-                        	j=j+3;
-                    	}
-                }
-		if (!FoundSpecial) {
-			dest[current++] = GSM_DefaultAlphabetUnicode[src[i]][0];
-			dest[current++] = GSM_DefaultAlphabetUnicode[src[i]][1];
+		if (ExtraAlphabet != NULL) {
+			for (i = 0; ExtraAlphabet[i] != 0x00; i += 3) {
+				if (ExtraAlphabet[i] == src[pos]) {
+					dest[current++] = ExtraAlphabet[i + 1];
+					dest[current++] = ExtraAlphabet[i + 2];
+					break;
+				}
+			}
+			/* Skip rest if we've found something */
+			if (ExtraAlphabet[i] != 0x00) {
+				continue;
+			}
 		}
+		dest[current++] = GSM_DefaultAlphabetUnicode[src[pos]][0];
+		dest[current++] = GSM_DefaultAlphabetUnicode[src[pos]][1];
 	}
 	dest[current++]=0;
-	dest[current++]=0;
+	dest[current]=0;
 #ifdef DEBUG
 	DumpMessageText(&GSM_global_debug, dest, UnicodeLength(dest)*2);
 #endif
@@ -566,10 +552,10 @@ void EncodeDefault(unsigned char *dest, const unsigned char *src, size_t *len, g
 		FoundSpecial = FALSE;
 		j = 0;
 		while (GSM_DefaultAlphabetCharsExtension[j][0]!=0x00 && UseExtensions) {
-			if (src[i*2] 	== GSM_DefaultAlphabetCharsExtension[j][2] &&
-			    src[i*2+1] 	== GSM_DefaultAlphabetCharsExtension[j][3]) {
+			if (src[i*2] 	== GSM_DefaultAlphabetCharsExtension[j][1] &&
+			    src[i*2+1] 	== GSM_DefaultAlphabetCharsExtension[j][2]) {
+				dest[current++] = 0x1b;
 				dest[current++] = GSM_DefaultAlphabetCharsExtension[j][0];
-				dest[current++] = GSM_DefaultAlphabetCharsExtension[j][1];
 				FoundSpecial 	= TRUE;
 				break;
 			}
@@ -644,8 +630,8 @@ void FindDefaultAlphabetLen(const unsigned char *src, size_t *srclen, size_t *sm
 		FoundSpecial = FALSE;
 		j = 0;
 		while (GSM_DefaultAlphabetCharsExtension[j][0]!=0x00) {
-			if (src[i*2] 	== GSM_DefaultAlphabetCharsExtension[j][2] &&
-			    src[i*2+1] 	== GSM_DefaultAlphabetCharsExtension[j][3]) {
+			if (src[i*2] 	== GSM_DefaultAlphabetCharsExtension[j][1] &&
+			    src[i*2+1] 	== GSM_DefaultAlphabetCharsExtension[j][2]) {
 				FoundSpecial = TRUE;
 				if (current+2 > maxlen) {
 					*srclen = i;
@@ -748,16 +734,22 @@ int GSM_PackSevenBitsToEight(int offset, const unsigned char *input, unsigned ch
         return (output_pos - output);
 }
 
-int GSM_UnpackSemiOctetNumber(GSM_Debug_Info *di, unsigned char *retval, const unsigned char *Number, gboolean semioctet)
+GSM_Error GSM_UnpackSemiOctetNumber(GSM_Debug_Info *di, unsigned char *retval, const unsigned char *Number, size_t *pos, size_t bufferlength, gboolean semioctet)
 {
 	unsigned char	Buffer[GSM_MAX_NUMBER_LENGTH + 1];
-	int		length		= Number[0];
+	size_t		length		= Number[*pos];
+	GSM_Error ret = ERR_NONE;
 
-	smfprintf(di, "L=%d\n", length);
+	smfprintf(di, "Number Length=%ld\n", (long)length);
 
 	/* Default ouput on error */
 	strcpy(Buffer, "<NOT DECODED>");
 
+	if (length > bufferlength) {
+		smfprintf(di, "Number too long!\n");
+		return ERR_UNKNOWN;
+	}
+
 	if (semioctet) {
 		/* Convert number of semioctets to number of chars */
 		if (length % 2) length++;
@@ -766,48 +758,50 @@ int GSM_UnpackSemiOctetNumber(GSM_Debug_Info *di, unsigned char *retval, const u
 
 	/* Check length */
 	if (length > GSM_MAX_NUMBER_LENGTH) {
-		smfprintf(di, "Number too big, not decoding! (Length=%d, MAX=%d)\n", length, GSM_MAX_NUMBER_LENGTH);
+		smfprintf(di, "Number too big, not decoding! (Length=%ld, MAX=%d)\n", (long)length, GSM_MAX_NUMBER_LENGTH);
+		ret = ERR_UNKNOWN;
 		goto out;
 	}
 
 	/*without leading byte with format of number*/
 	length--;
 
-	switch ((Number[1] & 0x70)) {
+	switch ((Number[*pos + 1] & 0x70)) {
 	case (NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN & 0x70):
 		if (length > 6) length++;
-		smfprintf(di, "Alphanumeric number, length %i\n",length);
-		GSM_UnpackEightBitsToSeven(0, length, length, Number+2, Buffer);
+		smfprintf(di, "Alphanumeric number, length %ld\n", (long)length);
+		GSM_UnpackEightBitsToSeven(0, length, length, Number+*pos+2, Buffer);
 		Buffer[length]=0;
 		break;
 	case (NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN & 0x70):
 		smfprintf(di, "International number\n");
 		Buffer[0]='+';
-		DecodeBCD(Buffer+1,Number+2, length);
+		DecodeBCD(Buffer+1,Number+*pos+2, length);
 		break;
 	default:
-		smfprintf(di, "Default number %02x (%d %d %d %d|%d %d %d %d)\n",Number[1],
-				Number[1] & 0x80 ? 1 : 0,
-				Number[1] & 0x40 ? 1 : 0,
-				Number[1] & 0x20 ? 1 : 0,
-				Number[1] & 0x10 ? 1 : 0,
-				Number[1] & 0x08 ? 1 : 0,
-				Number[1] & 0x04 ? 1 : 0,
-				Number[1] & 0x02 ? 1 : 0,
-				Number[1] & 0x01 ? 1 : 0
+		smfprintf(di, "Default number %02x (%d %d %d %d|%d %d %d %d)\n",Number[*pos],
+				Number[*pos] & 0x80 ? 1 : 0,
+				Number[*pos] & 0x40 ? 1 : 0,
+				Number[*pos] & 0x20 ? 1 : 0,
+				Number[*pos] & 0x10 ? 1 : 0,
+				Number[*pos] & 0x08 ? 1 : 0,
+				Number[*pos] & 0x04 ? 1 : 0,
+				Number[*pos] & 0x02 ? 1 : 0,
+				Number[*pos] & 0x01 ? 1 : 0
 				);
-		DecodeBCD (Buffer, Number+2, length);
+		DecodeBCD (Buffer, Number+*pos+2, length);
 		break;
 	}
 
-	smfprintf(di, "Len %i\n",length);
+	smfprintf(di, "Len %ld\n", (long)length);
 out:
 	EncodeUnicode(retval,Buffer,strlen(Buffer));
 	if (semioctet) {
-		return 2 + ((Number[0] + 1) / 2);
+		*pos += 2 + ((Number[*pos] + 1) / 2);
 	} else {
-		return 1 + Number[0];
+		*pos += 1 + Number[*pos];
 	}
+	return ret;
 }
 
 /**
@@ -828,7 +822,7 @@ out:
 int GSM_PackSemiOctetNumber(const unsigned char *Number, unsigned char *Output, gboolean semioctet)
 {
 	unsigned char	format;
-	int		length, i;
+	int		length, i, skip = 0;
 	unsigned char    *buffer;
 
 	length = UnicodeLength(Number);
@@ -838,12 +832,19 @@ int GSM_PackSemiOctetNumber(const unsigned char *Number, unsigned char *Output,
 		return 0;
 	}
 
-	memset(buffer, 0, length + 2);
-	memcpy(buffer, DecodeUnicodeString(Number), length + 1);
+	DecodeUnicode(Number, buffer);
 
 	/* Checking for format number */
-	if (buffer[0] == '+' || (buffer[0] == '0' && buffer[1] == '0')) {
+	if (buffer[0] == '+') {
 		format = NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;
+		skip = 1;
+	} else if (buffer[0] == '0' && buffer[1] == '0') {
+		format = NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;
+		skip = 2;
+	} else if (buffer[0] == '+' && buffer[1] == '0' && buffer[2] == '0') {
+		/* This is obviously wrong, but try to cope with that */
+		format = NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;
+		skip = 3;
 	} else {
 		format = NUMBER_UNKNOWN_NUMBERING_PLAN_ISDN;
 	}
@@ -868,8 +869,8 @@ int GSM_PackSemiOctetNumber(const unsigned char *Number, unsigned char *Output,
 		if (strlen(buffer)==7) length--;
 		break;
 	case NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN:
-		length--;
-		EncodeBCD (Output+1, buffer+1, length, TRUE);
+		length -= skip;
+		EncodeBCD (Output+1, buffer + skip, length, TRUE);
 		break;
 	default:
 		EncodeBCD (Output+1, buffer, length, TRUE);
@@ -938,7 +939,7 @@ void ReadUnicodeFile(unsigned char *Dest, const unsigned char *Source)
 		j=j+2;
 	}
 	Dest[current++] = 0;
-	Dest[current++]	= 0;
+	Dest[current]	= 0;
 }
 
 INLINE int GetBit(unsigned char *Buffer, size_t BitNum)
@@ -1096,7 +1097,7 @@ void EncodeUnicodeSpecialNOKIAChars(unsigned char *dest, const unsigned char *sr
 		dest[current++]	= 0x01;
 	}
 	dest[current++] = 0x00;
-	dest[current++] = 0x00;
+	dest[current] = 0x00;
 }
 
 void DecodeUnicodeSpecialNOKIAChars(unsigned char *dest, const unsigned char *src, int len)
@@ -1128,7 +1129,7 @@ void DecodeUnicodeSpecialNOKIAChars(unsigned char *dest, const unsigned char *sr
 		}
 	}
 	dest[current++] = 0x00;
-	dest[current++] = 0x00;
+	dest[current] = 0x00;
 }
 
 
@@ -1289,7 +1290,7 @@ GSM_Error MyGetLine(char *Buffer, size_t *Pos, char *OutBuffer, size_t MaxLen, s
 	OutBuffer[0] = 0;
 	pos = 0;
 	if (Buffer == NULL) return ERR_NONE;
-	while ((*Pos) < MaxLen) {
+	for (; (*Pos) < MaxLen; (*Pos)++) {
 		switch (Buffer[*Pos]) {
 		case 0x00:
 			return ERR_NONE;
@@ -1303,8 +1304,7 @@ GSM_Error MyGetLine(char *Buffer, size_t *Pos, char *OutBuffer, size_t MaxLen, s
 					if (was_lf && skip) return ERR_NONE;
 					was_lf = TRUE;
 				}
-			}
-			if (pos != 0 && !skip) {
+			} else {
 				if (MergeLines) {
 					/* (Quote printable new line) Does string end with = ? */
 					if (OutBuffer[pos - 1] == '=' && quoted_printable) {
@@ -1324,6 +1324,17 @@ GSM_Error MyGetLine(char *Buffer, size_t *Pos, char *OutBuffer, size_t MaxLen, s
 						*Pos = tmp;
 						break;
 					}
+					/* We ignore empty lines in this mode */
+					if (pos == 0) {
+						continue;
+					}
+				}
+				if (Buffer[*Pos] == 0x0d && (*Pos)+1 < MaxLen && Buffer[*Pos + 1] == 0x0a) {
+					/* Skip \r\n */
+					(*Pos) += 2;
+				} else {
+					/* Skip single \r or \n */
+					(*Pos)++;
 				}
 				return ERR_NONE;
 			}
@@ -1340,7 +1351,6 @@ GSM_Error MyGetLine(char *Buffer, size_t *Pos, char *OutBuffer, size_t MaxLen, s
 			OutBuffer[pos] = 0;
 			if (pos + 1 >= MaxOutLen) return ERR_MOREMEMORY;
 		}
-		(*Pos)++;
 	}
 	return ERR_NONE;
 }
@@ -1491,7 +1501,7 @@ gboolean EncodeUTF8QuotedPrintable(char *dest, const unsigned char *src)
 			}
 	    	}
 	}
-	dest[j++]=0;
+	dest[j]=0;
 	return retval;
 }
 
@@ -1511,7 +1521,7 @@ gboolean EncodeUTF8(char *dest, const unsigned char *src)
 			j+= DecodeWithUnicodeAlphabet(((wchar_t)(src[i*2]*256+src[i*2+1])), dest + j);
 	    	}
 	}
-	dest[j++]=0;
+	dest[j]=0;
 	return retval;
 }
 
@@ -1557,7 +1567,7 @@ void DecodeISO88591QuotedPrintable(unsigned char *dest, const unsigned char *src
 		i++;
 	}
 	dest[j++] = 0;
-	dest[j++] = 0;
+	dest[j] = 0;
 }
 
 /* Make Unicode string from UTF8 string */
@@ -1593,7 +1603,7 @@ void DecodeUTF8QuotedPrintable(unsigned char *dest, const char *src, int len)
 		dest[j++] = ret & 0xff;
 	}
 	dest[j++] = 0;
-	dest[j++] = 0;
+	dest[j] = 0;
 }
 
 void DecodeUTF8(unsigned char *dest, const char *src, int len)
@@ -1612,7 +1622,7 @@ void DecodeUTF8(unsigned char *dest, const char *src, int len)
 		dest[j++] = ret & 0xff;
 	}
 	dest[j++] = 0;
-	dest[j++] = 0;
+	dest[j] = 0;
 }
 
 void DecodeXMLUTF8(unsigned char *dest, const char *src, int len)
@@ -1631,10 +1641,14 @@ void DecodeXMLUTF8(unsigned char *dest, const char *src, int len)
 		DecodeUTF8(dest, src, len);
 		return;
 	}
+	if (src == NULL) {
+		*dest = 0;
+		return;
+	}
 
 	/* Find ampersand and decode the */
 	lastpos = src;
-	while ((lastpos != 0) && ((pos = strchr(lastpos, '&')) != NULL)) {
+	while ((*lastpos != 0) && ((pos = strchr(lastpos, '&')) != NULL)) {
 		/* Store current string */
 		strncat(tmp, lastpos, pos - lastpos);
 		lastpos = pos;
@@ -1721,7 +1735,7 @@ void DecodeUTF7(unsigned char *dest, const unsigned char *src, int len)
 		}
 	}
 	dest[j++] = 0;
-	dest[j++] = 0;
+	dest[j] = 0;
 }
 
 /*
@@ -1783,7 +1797,7 @@ void EncodeBASE64(const unsigned char *Input, char *Output, const size_t Length)
 		}
         }
 
-	Output[outpos++] = 0;
+	Output[outpos] = 0;
 }
 
 static void DecodeBASE64Block(const char in[4], unsigned char out[3])
