@@ -55,7 +55,7 @@ GSM_Error ATGEN_SetSMSC(GSM_StateMachine *s, GSM_SMSC *smsc)
 	return error;
 }
 
-GSM_Error ATGEN_ReplyGetSMSMemories(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplyGetSMSMemories(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	char *pos_start = NULL, *pos_end = NULL, *pos_tmp = NULL;
 	const char *Line;
@@ -81,7 +81,11 @@ GSM_Error ATGEN_ReplyGetSMSMemories(GSM_Protocol_Message msg, GSM_StateMachine *
 		Priv->PhoneSaveSMS = AT_NOTAVAILABLE;
 		Priv->SIMSaveSMS = AT_NOTAVAILABLE;
 
-		Line = GetLineString(msg.Buffer, &Priv->Lines, 2);
+		Line = GetLineString(msg->Buffer, &Priv->Lines, 2);
+		/* Skip empty line in response */
+		if (strcmp(Line, "") == 0) {
+			Line = GetLineString(msg->Buffer, &Priv->Lines, 3);
+		}
 		if (strcmp(Line, "+CPMS: ") == 0 && Priv->Manufacturer == AT_Samsung) {
 			smprintf(s, "Assuming broken Samsung response, both memories available!\n");
 			Priv->PhoneSMSMemory = AT_AVAILABLE;
@@ -91,18 +95,18 @@ GSM_Error ATGEN_ReplyGetSMSMemories(GSM_Protocol_Message msg, GSM_StateMachine *
 			goto completed;
 		}
 
-		if (strchr(msg.Buffer, '(') == NULL) {
+		if (strchr(msg->Buffer, '(') == NULL) {
 			smprintf(s, "Assuming broken iWOW style response, no lists!\n");
-			pos_start = strstr(msg.Buffer, "\", \"");
+			pos_start = strstr(msg->Buffer, "\", \"");
 
 			if (pos_start == NULL) {
-				pos_start = strstr(msg.Buffer, "\",\"");
+				pos_start = strstr(msg->Buffer, "\",\"");
 			}
 		} else {
-			pos_start = strstr(msg.Buffer, "), (");
+			pos_start = strstr(msg->Buffer, "), (");
 
 			if (pos_start == NULL) {
-				pos_start = strstr(msg.Buffer, "),(");
+				pos_start = strstr(msg->Buffer, "),(");
 			}
 		}
 		if (pos_start != NULL) {
@@ -119,18 +123,18 @@ GSM_Error ATGEN_ReplyGetSMSMemories(GSM_Protocol_Message msg, GSM_StateMachine *
 				Priv->PhoneSaveSMS = AT_AVAILABLE;
 			}
 		}
-		if (strstr(msg.Buffer, "\"SM\"") != NULL) {
+		if (strstr(msg->Buffer, "\"SM\"") != NULL) {
 			Priv->SIMSMSMemory = AT_AVAILABLE;
 		} else {
 			Priv->SIMSMSMemory = AT_NOTAVAILABLE;
 		}
-		if (strstr(msg.Buffer, "\"ME\"") != NULL) {
+		if (strstr(msg->Buffer, "\"ME\"") != NULL) {
 			Priv->PhoneSMSMemory = AT_AVAILABLE;
 		} else {
 			Priv->PhoneSMSMemory = AT_NOTAVAILABLE;
 
 			/* Check for Motorola style folders */
-			if (strstr(msg.Buffer, "\"MT\"") != NULL && strstr(msg.Buffer, "\"OM\"") != NULL) {
+			if (strstr(msg->Buffer, "\"MT\"") != NULL && strstr(msg->Buffer, "\"OM\"") != NULL) {
 				Priv->PhoneSMSMemory = AT_AVAILABLE;
 				Priv->PhoneSaveSMS = AT_AVAILABLE;
 				Priv->MotorolaSMS = TRUE;
@@ -148,6 +152,7 @@ completed:
 
 		return ERR_NONE;
 	case AT_Reply_Error:
+		return ERR_NOTSUPPORTED;
 	case AT_Reply_CMSError:
 		return ATGEN_HandleCMSError(s);
 	case AT_Reply_CMEError:
@@ -306,7 +311,9 @@ GSM_Error ATGEN_GetSMSMode(GSM_StateMachine *s)
 	}
 
 	/* Prefer PDU mode for most phones */
-	if (!GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_USE_SMSTEXTMODE)) {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_USE_SMSTEXTMODE)) {
+		smprintf(s, "Forcibily enabled SMS text mode\n");
+	} else {
 		smprintf(s, "Trying SMS PDU mode\n");
 		ATGEN_WaitForAutoLen(s, "AT+CMGF=0\r", 0x00, 9, ID_GetSMSMode);
 
@@ -445,6 +452,14 @@ GSM_Error ATGEN_DecodePDUMessage(GSM_StateMachine *s, const char *PDU, const int
 		return ERR_MOREMEMORY;
 	}
 
+	/*
+	 * Strip possible ,0 at the end of reply.
+	 * It actually should not be there, but it simply happens with some phones.
+	 */
+	while (length >= 2 && PDU[length - 1] == '0' && PDU[length - 2] == ',') {
+		length -= 2;
+	}
+
 	/* Decode hex encoded binary data */
 	if (!DecodeHexBin(buffer, PDU, length)) {
 		smprintf(s, "Failed to decode hex string!\n");
@@ -520,7 +535,7 @@ GSM_Error ATGEN_DecodePDUMessage(GSM_StateMachine *s, const char *PDU, const int
 	return ERR_NONE;
 }
 
-GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Error error = ERR_UNKNOWN;
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
@@ -539,23 +554,23 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 
 		switch (Priv->SMSMode) {
 		case SMS_AT_PDU:
-			CopyLineString(buffer, msg.Buffer, &Priv->Lines, 2);
+			CopyLineString(buffer, msg->Buffer, &Priv->Lines, 2);
 
 			/* Siemens MC35 (only ?) */
-			if (strstr(msg.Buffer,"+CMGR: 0,,0")!=NULL) {
+			if (strstr(msg->Buffer,"+CMGR: 0,,0")!=NULL) {
 				return ERR_EMPTY;
 			}
 			/* @bug Hardcoded position is wrong! */
 			return ATGEN_DecodePDUMessage(s,
-					GetLineString(msg.Buffer,&Priv->Lines,3),
+					GetLineString(msg->Buffer,&Priv->Lines,3),
 					atoi(buffer + 7));
 		case SMS_AT_TXT:
 			GSM_SetDefaultReceivedSMSData(sms);
 			error = ATGEN_ParseReply(s,
-					GetLineString(msg.Buffer, &Priv->Lines, 2),
-					"+CMGR: @r, @p @d",
+					GetLineString(msg->Buffer, &Priv->Lines, 2),
+					"+CMGR: @r, @p, @d",
 					buffer, sizeof(buffer), sms->Number, sizeof(sms->Number),
-					&sms->DateTime, sizeof(sms->DateTime));
+					&sms->DateTime);
 
 			if (error != ERR_NONE) {
 				return error;
@@ -615,7 +630,7 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 				sms->ReplyViaSameSMSC = FALSE;
 
 				/* Go trough all lines till OK */
-				for (i = 3; strcmp(line = GetLineString(msg.Buffer, &Priv->Lines, i), "OK") != 0; i++) {
+				for (i = 3; strcmp(line = GetLineString(msg->Buffer, &Priv->Lines, i), "OK") != 0; i++) {
 					if (i > 3) {
 						/* Include new line */
 						sms->Text[(2 * sms->Length) + 0] = 0;
@@ -624,7 +639,7 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 						sms->Text[(2 * sms->Length) + 3] = 0;
 						sms->Length++;
 					}
-					length = GetLineLength(msg.Buffer, &Priv->Lines, i);
+					length = GetLineLength(msg->Buffer, &Priv->Lines, i);
 					error = ATGEN_DecodeText(s, line, length,
 						sms->Text + (2 * sms->Length),
 						sizeof(sms->Text) - (2 * sms->Length),
@@ -642,19 +657,19 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 			 */
 			current = 0;
 
-			while (msg.Buffer[current]!='"') {
+			while (msg->Buffer[current]!='"') {
 				current++;
 			}
-			current += ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
-			current += ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+			current += ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
+			current += ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
 			/* It's delivery report according to Nokia AT standards */
 			if (sms->Folder == 1 && buffer[0]!=0 && buffer[0]!='"') {
 				/* ??? */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
 				/* format of sender number */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
 				/* Sender number */
 				/* FIXME: support for all formats */
@@ -662,14 +677,14 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 				smprintf(s, "Sender \"%s\"\n",DecodeUnicodeString(sms->Number));
 
 				/* ??? */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
 				/* Sending datetime */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				i = strlen(buffer);
 				buffer[i] = ',';
 				i++;
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer+i);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer+i);
 				smprintf(s, "\"%s\"\n",buffer);
 				error = ATGEN_DecodeDateTime(s, &sms->DateTime, buffer);
 
@@ -677,11 +692,11 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 					return error;
 				}
 				/* Date of SMSC response */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				i = strlen(buffer);
 				buffer[i] = ',';
 				i++;
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer+i);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer+i);
 				smprintf(s, "\"%s\"\n",buffer);
 				error = ATGEN_DecodeDateTime(s, &sms->SMSCTime, buffer);
 
@@ -689,7 +704,7 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 					return error;
 				}
 				/* TPStatus */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				TPStatus = atoi(buffer);
 				buffer[PHONE_SMSStatusReport.TPStatus] = TPStatus;
 				error = GSM_DecodeSMSFrameStatusReportData(&(s->di), sms, buffer, PHONE_SMSStatusReport);
@@ -708,7 +723,7 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 				EncodeUnicode(sms->Number,buffer+1,strlen(buffer)-2);
 
 				/* Sender number in alphanumeric format ? */
-				current += ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current += ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
 				if (strlen(buffer)!=0) {
 					EncodeUnicode(sms->Number,buffer+1,strlen(buffer)-2);
@@ -717,7 +732,7 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 
 				/* Sending datetime */
 				if (sms->Folder == 1 || sms->Folder == 3) {
-					current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+					current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 					/* FIXME: ATGEN_ExtractOneParameter() is broken as it doesn't respect
 					 * quoting of parameters and thus +FOO: "ab","cd,ef" will consider
 					 * as three arguments: "ab" >> "cd >> ef"
@@ -726,7 +741,7 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 						i = strlen(buffer);
 						buffer[i] = ',';
 						i++;
-						current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer+i);
+						current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer+i);
 					}
 					smprintf(s, "\"%s\"\n",buffer);
 
@@ -744,10 +759,10 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 					}
 				}
 				/* Sender number format */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
 				/* First byte */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				firstbyte = atoi(buffer);
 				sms->ReplyViaSameSMSC = FALSE;
 
@@ -756,7 +771,7 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 					sms->ReplyViaSameSMSC = TRUE;
 				}
 				/* TP PID */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				sms->ReplaceMessage = 0;
 
 				if (atoi(buffer) > 0x40 && atoi(buffer) < 0x48) {
@@ -765,23 +780,23 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 				smprintf(s, "TPPID: %02x %i\n",atoi(buffer),atoi(buffer));
 
 				/* TP DCS */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				TPDCS = atoi(buffer);
 
 				if (sms->Folder == 2 || sms->Folder == 4) {
 					/*TP VP */
-					current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+					current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				}
 				/* SMSC number */
 				/* FIXME: support for all formats */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				EncodeUnicode(sms->SMSC.Number,buffer+1,strlen(buffer)-2);
 
 				/* Format of SMSC number */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
 				/* TPUDL */
-				current+=ATGEN_ExtractOneParameter(msg.Buffer+current, buffer);
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				TPUDL = atoi(buffer);
 				current++;
 				sms->Coding = GSM_GetMessageCoding(&(s->di), TPDCS);
@@ -812,12 +827,16 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 					if ((firstbyte & 0x40)!=0x40) {
 						sms->UDH.Type	= UDH_NoUDH;
 						sms->Length	= TPUDL;
-						EncodeUnicode(sms->Text,msg.Buffer+Priv->Lines.numbers[2*2],TPUDL);
+						EncodeUnicode(sms->Text,msg->Buffer+Priv->Lines.numbers[2*2],TPUDL);
 						break;
 					}
 				case SMS_Coding_Unicode_No_Compression:
 				case SMS_Coding_8bit:
-					DecodeHexBin(buffer+PHONE_SMSDeliver.Text, msg.Buffer+current, TPUDL*2);
+					if (sms->Coding == SMS_Coding_Unicode_No_Compression && GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SMS_UTF8_ENCODED)) {
+						DecodeUTF8(buffer+PHONE_SMSDeliver.Text, msg->Buffer+current, TPUDL);
+					} else {
+						DecodeHexBin(buffer+PHONE_SMSDeliver.Text, msg->Buffer+current, TPUDL*2);
+					}
 					buffer[PHONE_SMSDeliver.firstbyte] 	= firstbyte;
 					buffer[PHONE_SMSDeliver.TPDCS] 		= TPDCS;
 					buffer[PHONE_SMSDeliver.TPUDL] 		= TPUDL;
@@ -828,6 +847,7 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 			}
 			return ERR_NONE;
 		default:
+			smprintf(s, "Internal error - SMS mode not set!\n");
 			return ERR_BUG;
 		}
 		break;
@@ -901,7 +921,7 @@ GSM_Error ATGEN_GetSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sms)
 	return error;
 }
 
-GSM_Error ATGEN_ReplyGetMessageList(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplyGetMessageList(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Error error;
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
@@ -916,7 +936,11 @@ GSM_Error ATGEN_ReplyGetMessageList(GSM_Protocol_Message msg, GSM_StateMachine *
 	case AT_Reply_Error:
 		return ERR_NOTSUPPORTED;
  	case AT_Reply_CMSError:
-		return ATGEN_HandleCMSError(s);
+		if (Priv->ErrorCode == 320 || Priv->ErrorCode == 500) {
+			return ERR_EMPTY;
+		} else {
+			return ATGEN_HandleCMSError(s);
+		}
 	case AT_Reply_CMEError:
 		return ATGEN_HandleCMEError(s);
 	default:
@@ -928,7 +952,7 @@ GSM_Error ATGEN_ReplyGetMessageList(GSM_Protocol_Message msg, GSM_StateMachine *
 
 	/* Walk through lines with +CMGL: */
 	/* First line is our command so we can skip it */
-	for (line = 2; strcmp("OK", str = GetLineString(msg.Buffer, &Priv->Lines, line)) != 0; line++) {
+	for (line = 2; strcmp("OK", str = GetLineString(msg->Buffer, &Priv->Lines, line)) != 0; line++) {
 		/*
 		 * Find +CMGL, it should be on beginning, but it does not have to (see
 		 * corruption mentioned at the end of loop.
@@ -989,7 +1013,7 @@ GSM_Error ATGEN_ReplyGetMessageList(GSM_Protocol_Message msg, GSM_StateMachine *
 				Priv->SMSCache[Priv->SMSCount - 1].State = -1;
 			}
 			/* Get next line (PDU data) */
-			str = GetLineString(msg.Buffer, &Priv->Lines, line);
+			str = GetLineString(msg->Buffer, &Priv->Lines, line);
 
 			if (strlen(str) >= GSM_AT_MAXPDULEN) {
 				smprintf(s, "PDU (%s) too long for cache, skipping!\n", str);
@@ -1021,6 +1045,10 @@ GSM_Error ATGEN_GetSMSList(GSM_StateMachine *s, gboolean first)
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
 	int used = 0;
 
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_DISABLE_CMGL)) {
+		return ERR_NOTSUPPORTED;
+	}
+
 	/* Set mode of SMS */
 	error = ATGEN_GetSMSMode(s);
 
@@ -1080,10 +1108,10 @@ GSM_Error ATGEN_GetSMSList(GSM_StateMachine *s, gboolean first)
 	if (Priv->SMSMode == SMS_AT_TXT) {
 		ATGEN_WaitForAutoLen(s, "AT+CMGL=\"ALL\"\r", 0x00, 500, ID_GetSMSMessage);
 	} else {
-		ATGEN_WaitForAutoLen(s, "AT+CMGL=4\r", 0x00, 20, ID_GetSMSMessage);
+		ATGEN_WaitForAutoLen(s, "AT+CMGL=4\r", 0x00, 500, ID_GetSMSMessage);
 	}
 	if (error == ERR_NOTSUPPORTED) {
-		ATGEN_WaitForAutoLen(s, "AT+CMGL\r", 0x00, 5, ID_GetSMSMessage);
+		ATGEN_WaitForAutoLen(s, "AT+CMGL\r", 0x00, 500, ID_GetSMSMessage);
 	}
 	/*
 	 * We did not read anything, but it is correct, indicate that
@@ -1128,10 +1156,12 @@ GSM_Error ATGEN_GetNextSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sms, gboole
 
 		/* Get list of messages */
 		error = ATGEN_GetSMSList(s, TRUE);
+	} else {
+		error = ERR_NONE;
 	}
 
 	/* Use listed locations if we have them */
-	if (Priv->SMSCache != NULL) {
+	if (error == ERR_NONE && Priv->SMSCache != NULL) {
 		if (start) {
 			found = 0;
 		} else {
@@ -1220,6 +1250,13 @@ GSM_Error ATGEN_GetNextSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sms, gboole
 		}
 	}
 
+	/* Ensure LastSMSStatus is up to date */
+	error = ATGEN_GetSMSStatus(s, &Priv->LastSMSStatus);
+
+	if (error != ERR_NONE) {
+		return error;
+	}
+
 	/* Use brute force if listing does not work */
 	while (TRUE) {
 		sms->SMS[0].Location++;
@@ -1257,7 +1294,7 @@ GSM_Error ATGEN_GetNextSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sms, gboole
 	return error;
 }
 
-GSM_Error ATGEN_ReplyGetSMSStatus(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplyGetSMSStatus(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Error error;
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
@@ -1271,7 +1308,7 @@ GSM_Error ATGEN_ReplyGetSMSStatus(GSM_Protocol_Message msg, GSM_StateMachine *s)
 
 		/* Check for +CPMS: 0,30,0,30,8,330, this is according to ETSI */
 		error = ATGEN_ParseReply(s,
-				GetLineString(msg.Buffer, &Priv->Lines, 2),
+				GetLineString(msg->Buffer, &Priv->Lines, 2),
 				"+CPMS: @i, @i, @0",
 				&used, &size);
 		if (error != ERR_NONE) {
@@ -1282,22 +1319,22 @@ GSM_Error ATGEN_ReplyGetSMSStatus(GSM_Protocol_Message msg, GSM_StateMachine *s)
 			 * 3 "OK"
 			 */
 			error = ATGEN_ParseReply(s,
-					GetLineString(msg.Buffer, &Priv->Lines, 2),
+					GetLineString(msg->Buffer, &Priv->Lines, 2),
 					"+CPMS: @s, @i, @i, @0",
 					&buffer, sizeof(buffer), &used, &size);
 		}
 		if (error != ERR_NONE) {
 			/* For phones with single memory */
 			error = ATGEN_ParseReply(s,
-					GetLineString(msg.Buffer, &Priv->Lines, 2),
+					GetLineString(msg->Buffer, &Priv->Lines, 2),
 					"+CPMS: @i, @i",
 					&used, &size);
 		}
 		if (error == ERR_NONE) {
 			smprintf(s, "Used : %i\n", used);
 			smprintf(s, "Size : %i\n", size);
-			if ((strstr(msg.Buffer, "CPMS=\"ME") != NULL) ||
-				(Priv->MotorolaSMS && strstr(msg.Buffer, "CPMS=\"MT") != NULL)) {
+			if ((strstr(msg->Buffer, "CPMS=\"ME") != NULL) ||
+				(Priv->MotorolaSMS && strstr(msg->Buffer, "CPMS=\"MT") != NULL)) {
 				SMSStatus->PhoneUsed = used;
 				SMSStatus->PhoneSize = size;
 			} else {
@@ -1307,7 +1344,7 @@ GSM_Error ATGEN_ReplyGetSMSStatus(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		}
 		return error;
 	case AT_Reply_Error:
-		if (strstr(msg.Buffer,"SM")!=NULL) {
+		if (strstr(msg->Buffer,"SM")!=NULL) {
 			smprintf(s, "Can't access SIM card\n");
 			return ERR_SECURITYERROR;
 		}
@@ -1394,21 +1431,21 @@ GSM_Error ATGEN_GetSMSStatus(GSM_StateMachine *s, GSM_SMSMemoryStatus *status)
 	return ERR_NONE;
 }
 
-GSM_Error ATGEN_ReplyAddSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplyAddSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Error error;
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
 	size_t i = 0;
 	int folder = 0;
 
-	if (s->Protocol.Data.AT.EditMode) {
-		if (Priv->ReplyState != AT_Reply_SMSEdit) {
-			return ATGEN_HandleCMSError(s);
-		}
-		s->Protocol.Data.AT.EditMode = FALSE;
-		return ERR_NONE;
-	}
 	switch (Priv->ReplyState) {
+	case AT_Reply_SMSEdit:
+		if (s->Protocol.Data.AT.EditMode) {
+			s->Protocol.Data.AT.EditMode = FALSE;
+			return ERR_NONE;
+		}
+		smprintf(s, "Received unexpected SMS edit prompt!\n");
+		return ERR_UNKNOWN;
 	case AT_Reply_OK:
 		smprintf(s, "SMS saved OK\n");
 
@@ -1419,7 +1456,7 @@ GSM_Error ATGEN_ReplyAddSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine *s
 			i++;
 		}
 		error = ATGEN_ParseReply(s,
-				GetLineString(msg.Buffer, &Priv->Lines, i - 1),
+				GetLineString(msg->Buffer, &Priv->Lines, i - 1),
 				"+CMGW: @i",
 				&s->Phone.Data.SaveSMSMessage->Location);
 
@@ -1699,7 +1736,9 @@ GSM_Error ATGEN_AddSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 		} else {
 			len = sprintf(buffer, "AT+CMGW=\"%s\",,\"%s\"\r",DecodeUnicodeString(sms->Number),statetxt);
 		}
+		break;
 	default:
+		smprintf(s, "Internal error - SMS mode not set!\n");
 		return ERR_BUG;
 	}
 	Phone->SaveSMSMessage = sms;
@@ -1753,21 +1792,20 @@ GSM_Error ATGEN_AddSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 	return Phone->DispatchError;
 }
 
-GSM_Error ATGEN_ReplySendSMS(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplySendSMS(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Error error;
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
 	int i = 0,reference = 0;
 
-	if (s->Protocol.Data.AT.EditMode) {
-		if (Priv->ReplyState != AT_Reply_SMSEdit) {
-			return ERR_UNKNOWN;
-		}
-		s->Protocol.Data.AT.EditMode = FALSE;
-		return ERR_NONE;
-	}
-
 	switch (Priv->ReplyState) {
+	case AT_Reply_SMSEdit:
+		if (s->Protocol.Data.AT.EditMode) {
+			s->Protocol.Data.AT.EditMode = FALSE;
+			return ERR_NONE;
+		}
+		smprintf(s, "Received unexpected SMS edit prompt!\n");
+		return ERR_UNKNOWN;
 	case AT_Reply_OK:
  		smprintf(s, "SMS sent OK\n");
 
@@ -1777,7 +1815,7 @@ GSM_Error ATGEN_ReplySendSMS(GSM_Protocol_Message msg, GSM_StateMachine *s)
 			i++;
 		}
 		error = ATGEN_ParseReply(s,
-				GetLineString(msg.Buffer, &Priv->Lines, i - 1),
+				GetLineString(msg->Buffer, &Priv->Lines, i - 1),
 				"+CMGS: @i",
 				&reference);
 
@@ -1829,6 +1867,12 @@ GSM_Error ATGEN_SendSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 	if (error != ERR_NONE) {
 		return error;
 	}
+
+	if (sms->SMSC.Number[0] == 0x00 && sms->SMSC.Number[1] == 0x00) {
+		smprintf(s,"No SMSC in SMS to send\n");
+		return ERR_EMPTYSMSC;
+	}
+
 	switch (Phone->Priv.ATGEN.SMSMode) {
 	case SMS_AT_PDU:
 		len = sprintf(buffer, "AT+CMGS=%i\r",current);
@@ -1837,6 +1881,7 @@ GSM_Error ATGEN_SendSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 		len = sprintf(buffer, "AT+CMGS=\"%s\"\r",DecodeUnicodeString(sms->Number));
 		break;
 	default:
+		smprintf(s, "Internal error - SMS mode not set!\n");
 		return ERR_BUG;
 	}
 
@@ -1923,7 +1968,7 @@ GSM_Error ATGEN_SendSavedSMS(GSM_StateMachine *s, int Folder, int Location)
 	return error;
 }
 
-GSM_Error ATGEN_ReplyGetSMSC(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplyGetSMSC(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Error error;
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
@@ -1936,19 +1981,19 @@ GSM_Error ATGEN_ReplyGetSMSC(GSM_Protocol_Message msg, GSM_StateMachine *s)
 
 		/* Parse reply */
 		error = ATGEN_ParseReply(s,
-					GetLineString(msg.Buffer, &Priv->Lines, 2),
+					GetLineString(msg->Buffer, &Priv->Lines, 2),
 					"+CSCA: @p, @i",
 					SMSC->Number, sizeof(SMSC->Number),
 					&number_type);
 		if (error != ERR_NONE) {
 			error = ATGEN_ParseReply(s,
-						GetLineString(msg.Buffer, &Priv->Lines, 2),
+						GetLineString(msg->Buffer, &Priv->Lines, 2),
 						"+CSCA: @p, @0",
 						SMSC->Number, sizeof(SMSC->Number));
 		}
 		if (error != ERR_NONE) {
 			error = ATGEN_ParseReply(s,
-						GetLineString(msg.Buffer, &Priv->Lines, 2),
+						GetLineString(msg->Buffer, &Priv->Lines, 2),
 						"+CSCA: @p",
 						SMSC->Number, sizeof(SMSC->Number));
 			number_type = NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;
@@ -2005,7 +2050,7 @@ GSM_Error ATGEN_GetSMSC(GSM_StateMachine *s, GSM_SMSC *smsc)
 	return error;
 }
 
-GSM_Error ATGEN_ReplyDeleteSMSMessage(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplyDeleteSMSMessage(GSM_Protocol_Message *msg UNUSED, GSM_StateMachine *s)
 {
 	switch (s->Phone.Data.Priv.ATGEN.ReplyState) {
 	case AT_Reply_OK:
@@ -2098,7 +2143,7 @@ GSM_Error ATGEN_GetSMSFolders(GSM_StateMachine *s, GSM_SMSFolders *folders)
 	return ERR_NONE;
 }
 
-GSM_Error ATGEN_IncomingSMSCInfo(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s UNUSED)
+GSM_Error ATGEN_IncomingSMSCInfo(GSM_Protocol_Message *msg UNUSED, GSM_StateMachine *s UNUSED)
 {
 	return ERR_NONE;
 }
@@ -2117,7 +2162,7 @@ GSM_Error ATGEN_SetFastSMSSending(GSM_StateMachine *s, gboolean enable)
 	return error;
 }
 
-GSM_Error ATGEN_IncomingSMSInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_IncomingSMSInfo(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
 	GSM_Phone_Data *Data = &s->Phone.Data;
@@ -2134,7 +2179,7 @@ GSM_Error ATGEN_IncomingSMSInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		sms.InboxFolder  = TRUE;
 		sms.PDU 	 = 0;
 
-		buffer = strchr(msg.Buffer, ':');
+		buffer = strchr(msg->Buffer, ':');
 
 		if (buffer == NULL) {
 			return ERR_UNKNOWNRESPONSE;
@@ -2150,12 +2195,18 @@ GSM_Error ATGEN_IncomingSMSInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 			} else {
 				sms.Folder = 1;
 			}
+		} else if (strncmp(buffer, "MT", 2) == 0 || strncmp(buffer, "\"MT\"", 4) == 0) {
+			if (Priv->SIMSMSMemory == AT_AVAILABLE) {
+				sms.Folder = 3;
+			} else {
+				sms.Folder = 1;
+			}
 		} else if (strncmp(buffer, "SM", 2) == 0 || strncmp(buffer, "\"SM\"", 4) == 0) {
 			sms.Folder = 1;
 		} else {
 			return ERR_UNKNOWNRESPONSE;
 		}
-		buffer = strchr(msg.Buffer, ',');
+		buffer = strchr(msg->Buffer, ',');
 
 		if (buffer == NULL) {
 			return ERR_UNKNOWNRESPONSE;
@@ -2166,12 +2217,12 @@ GSM_Error ATGEN_IncomingSMSInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
 			buffer++;
 		}
 		sms.Location = atoi(buffer);
-		s->User.IncomingSMS(s, sms, s->User.IncomingSMSUserData);
+		s->User.IncomingSMS(s, &sms, s->User.IncomingSMSUserData);
 	}
 	return ERR_NONE;
 }
 
-GSM_Error ATGEN_IncomingSMSDeliver(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_IncomingSMSDeliver(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Phone_Data *Data = &s->Phone.Data;
 	GSM_SMSMessage sms;
@@ -2193,8 +2244,8 @@ GSM_Error ATGEN_IncomingSMSDeliver(GSM_Protocol_Message msg, GSM_StateMachine *s
 			i++;
 		}
 		DecodeHexBin (buffer,
-			GetLineString(msg.Buffer,&Data->Priv.ATGEN.Lines,i),
-			GetLineLength(msg.Buffer,&Data->Priv.ATGEN.Lines,i));
+			GetLineString(msg->Buffer,&Data->Priv.ATGEN.Lines,i),
+			GetLineLength(msg->Buffer,&Data->Priv.ATGEN.Lines,i));
 
 		/* We use locations from SMS layouts like in ../phone2.c(h) */
 		for(i = 0;i < buffer[0]+1;i++) {
@@ -2218,13 +2269,13 @@ GSM_Error ATGEN_IncomingSMSDeliver(GSM_Protocol_Message msg, GSM_StateMachine *s
 			smsframe[i+PHONE_SMSDeliver.Text]=buffer[current++];
 		}
 		GSM_DecodeSMSFrame(&(s->di), &sms,smsframe,PHONE_SMSDeliver);
-		s->User.IncomingSMS(s,sms, s->User.IncomingSMSUserData);
+		s->User.IncomingSMS(s, &sms, s->User.IncomingSMSUserData);
 	}
 	return ERR_NONE;
 }
 
 /* I don't have phone able to do it and can't fill it */
-GSM_Error ATGEN_IncomingSMSReport(GSM_Protocol_Message msg UNUSED, GSM_StateMachine *s)
+GSM_Error ATGEN_IncomingSMSReport(GSM_Protocol_Message *msg UNUSED, GSM_StateMachine *s)
 {
 	smprintf(s, "Incoming SMS received (Report)\n");
 	return ERR_NONE;
@@ -2308,6 +2359,9 @@ int *GetRange(GSM_StateMachine *s, const char *buffer)
 			return NULL;
 		}
 	}
+	if (result == NULL) {
+		return NULL;
+	}
 	smprintf(s, "Returning range: ");
 
 	for (i = 0; result[i] != -1; i++) {
@@ -2317,7 +2371,7 @@ int *GetRange(GSM_StateMachine *s, const char *buffer)
 	return result;
 }
 
-GSM_Error ATGEN_ReplyGetCNMIMode(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplyGetCNMIMode(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
 	const char *buffer;
@@ -2353,7 +2407,7 @@ GSM_Error ATGEN_ReplyGetCNMIMode(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	Priv->CNMIBroadcastProcedure	= 0;
 #endif
 
-	buffer = GetLineString(msg.Buffer, &Priv->Lines, 2);
+	buffer = GetLineString(msg->Buffer, &Priv->Lines, 2);
 
 	if (buffer == NULL) {
 		return  ERR_UNKNOWNRESPONSE;
@@ -2446,11 +2500,11 @@ GSM_Error ATGEN_ReplyGetCNMIMode(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	if (range == NULL) {
 		return  ERR_UNKNOWNRESPONSE;
 	}
-	if (InRange(range, 1)) {
-		Priv->CNMIDeliverProcedure = 1; /* 1 = route message to TE */
+	if (InRange(range, 2)) {
+		Priv->CNMIDeliverProcedure = 2; /* 2 = store message and send where it is stored */
 	}
-	else if (InRange(range, 2)) {
-		Priv->CNMIDeliverProcedure = 2; /* 1 = store message and send where it is stored */
+	else if (InRange(range, 1)) {
+		Priv->CNMIDeliverProcedure = 1; /* 1 = route message to TE */
 	}
 	/* we don't want: 0 = no routing */
 	free(range);
@@ -2592,7 +2646,7 @@ GSM_Error ATGEN_SetIncomingSMS(GSM_StateMachine *s, gboolean enable)
 
 #ifdef GSM_ENABLE_CELLBROADCAST
 
-GSM_Error ATGEN_ReplyIncomingCB(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATGEN_ReplyIncomingCB(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 #if 0
 	GSM_CBMessage 	CB;
@@ -2600,20 +2654,20 @@ GSM_Error ATGEN_ReplyIncomingCB(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	int i = 0;
 	size_t j = 0;
 
-	DecodeHexBin (Buffer,msg.Buffer+6,msg.Length-6);
-	DumpMessage(&di ,Buffer,msg.Length-6);
+	DecodeHexBin (Buffer,msg->Buffer+6,msg.Length-6);
+	DumpMessage(&di ,Buffer,msg->Length-6);
 
 	CB.Channel = Buffer[4];
 
-	for (j = 0;j < msg.Length;j++) {
+	for (j = 0;j < msg->Length;j++) {
 		smprintf(s, "j=" SIZE_T_FORMAT "\n",j);
-		i = GSM_UnpackEightBitsToSeven(0, msg.Buffer[6], msg.Buffer[6], msg.Buffer+j, Buffer2);
-		i = msg.Buffer[6] - 1;
+		i = GSM_UnpackEightBitsToSeven(0, msg->Buffer[6], msg.Buffer[6], msg.Buffer+j, Buffer2);
+		i = msg->Buffer[6] - 1;
 
 		while (i != 0) {
 			if (Buffer[i] == 13) i = i - 1; else break;
 		}
-		DecodeDefault(CB.Text, Buffer2, msg.Buffer[6], FALSE, NULL);
+		DecodeDefault(CB.Text, Buffer2, msg->Buffer[6], TRUE, NULL);
 		smprintf(s, "Channel %i, text \"%s\"\n",CB.Channel,DecodeUnicodeString(CB.Text));
 	}
 	if (s->Phone.Data.EnableIncomingCB && s->User.IncomingCB != NULL) {
