@@ -178,14 +178,14 @@ static GSM_Error SetSamsungFrame(GSM_StateMachine *s, unsigned char *buff, int s
  * SDNDCRC = 0xa : RECEIVECRC = 0xcbf53a1c : BINSIZE = 5
  * CRCERR
  */
-static GSM_Error ReplySetSamsungFrame(GSM_Protocol_Message msg, GSM_StateMachine *s)
+static GSM_Error ReplySetSamsungFrame(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	unsigned long 		txcrc, rxcrc;
 	int 			binsize;
 	char 			*pos;
 
 	/* Parse SDNDCRC */
-	pos = strchr(msg.Buffer, '=');
+	pos = strchr(msg->Buffer, '=');
 	if (!pos) return ERR_UNKNOWN;
 	pos++;
 	txcrc = strtoul(pos, NULL, 0);
@@ -212,7 +212,7 @@ static GSM_Error ReplySetSamsungFrame(GSM_Protocol_Message msg, GSM_StateMachine
  * Bitmaps
  */
 
-GSM_Error SAMSUNG_ReplyGetBitmap(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_ReplyGetBitmap(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
  	GSM_Phone_ATGENData 	*Priv = &s->Phone.Data.Priv.ATGEN;
 	char 		buffer[32];
@@ -225,7 +225,7 @@ GSM_Error SAMSUNG_ReplyGetBitmap(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		/* Parse +IMGR:location,name,0,0,0,0 */
 
 		/* Parse location */
-		pos = strchr(msg.Buffer, ':');
+		pos = strchr(msg->Buffer, ':');
 		if (!pos) return ERR_UNKNOWN;
 		pos++;
 		location = atoi(pos);
@@ -258,7 +258,7 @@ GSM_Error SAMSUNG_ReplyGetBitmap(GSM_Protocol_Message msg, GSM_StateMachine *s)
 	}
 }
 
-GSM_Error SAMSUNG_ReplySetBitmap(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_ReplySetBitmap(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	smprintf(s, "Bitmap sent\n");
 	return ReplySetSamsungFrame(msg, s);
@@ -327,8 +327,8 @@ GSM_Error SAMSUNG_SetBitmap(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
 	if ((dot = strrchr(name, '.')) != NULL)
 		*dot = 0;
 
-	len = sprintf(req, "AT+IMGW=0,\"%s\",2,0,0,0,0,100,%d,%u\r", name,
-		Bitmap->BinaryPic.Length, (unsigned int)crc);
+	len = sprintf(req, "AT+IMGW=0,\"%s\",2,0,0,0,0,100,%ld,%u\r", name,
+		(long int)Bitmap->BinaryPic.Length, (unsigned int)crc);
 
 	error = s->Protocol.Functions->WriteMessage(s, req, len, 0x00);
 	if (error!=ERR_NONE) return error;
@@ -341,7 +341,7 @@ GSM_Error SAMSUNG_SetBitmap(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
  * Ringtones
  */
 
-GSM_Error SAMSUNG_ReplyGetRingtone(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_ReplyGetRingtone(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
  	GSM_Phone_ATGENData 	*Priv = &s->Phone.Data.Priv.ATGEN;
 	unsigned char 		buffer[32];
@@ -354,7 +354,7 @@ GSM_Error SAMSUNG_ReplyGetRingtone(GSM_Protocol_Message msg, GSM_StateMachine *s
 		/* Parse +MELR:location,name,size */
 
 		/* Parse location */
-		pos = strchr(msg.Buffer, ':');
+		pos = strchr(msg->Buffer, ':');
 		if (!pos) return ERR_UNKNOWN;
 		pos++;
 		location = atoi(pos);
@@ -409,7 +409,7 @@ GSM_Error SAMSUNG_GetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone, gbool
 	return GSM_WaitFor (s, req, len, 0x00, 4, ID_GetRingtone);
 }
 
-GSM_Error SAMSUNG_ReplySetRingtone(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_ReplySetRingtone(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	smprintf(s, "Ringtone sent\n");
 	return ReplySetSamsungFrame(msg, s);
@@ -460,24 +460,24 @@ GSM_Error SAMSUNG_CheckCalendar(GSM_StateMachine *s)
 
 	smprintf(s, "Checking for supported calendar commands\n");
 
-	ATGEN_WaitForAutoLen(s, "AT+ORGI?\r", 0x00, 10, ID_GetProtocol);
-	if (error == ERR_NONE) {
-		Priv->SamsungCalendar = SAMSUNG_ORG;
-		return ERR_NONE;
-	}
-
 	ATGEN_WaitForAutoLen(s, "AT+SSHT?\r", 0x00, 10, ID_GetProtocol);
 	if (error == ERR_NONE) {
 		Priv->SamsungCalendar = SAMSUNG_SSH;
 		return ERR_NONE;
 	}
 
+	ATGEN_WaitForAutoLen(s, "AT+ORGI?\r", 0x00, 10, ID_GetProtocol);
+	if (error == ERR_NONE) {
+		Priv->SamsungCalendar = SAMSUNG_ORG;
+		return ERR_NONE;
+	}
+
 	Priv->SamsungCalendar = SAMSUNG_NONE;
 	return ERR_NONE;
 
 }
 
-GSM_Error SAMSUNG_ReplyGetMemoryInfo(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_ReplyGetMemoryInfo(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
  	GSM_Phone_ATGENData 	*Priv = &s->Phone.Data.Priv.ATGEN;
 
@@ -500,7 +500,7 @@ GSM_Error SAMSUNG_ReplyGetMemoryInfo(GSM_Protocol_Message msg, GSM_StateMachine
 	}
 }
 
-GSM_Error SAMSUNG_ReplyGetMemory(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_ReplyGetMemory(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
  	GSM_Phone_ATGENData 	*Priv = &s->Phone.Data.Priv.ATGEN;
  	GSM_MemoryEntry		*Memory = s->Phone.Data.Memory;
@@ -513,48 +513,80 @@ GSM_Error SAMSUNG_ReplyGetMemory(GSM_Protocol_Message msg, GSM_StateMachine *s)
  		smprintf(s, "Phonebook entry received\n");
 		Memory->EntriesNum = 9;
 		Memory->Entries[0].EntryType = PBK_Number_Mobile;
+		Memory->Entries[0].Location = PBK_Location_Unknown;
 		Memory->Entries[0].AddError = ERR_NONE;
 		Memory->Entries[0].VoiceTag = 0;
 		Memory->Entries[0].SMSList[0] = 0;
-		Memory->Entries[1].EntryType = PBK_Number_Home;
+		Memory->Entries[1].EntryType = PBK_Number_General;
+		Memory->Entries[1].Location = PBK_Location_Home;
 		Memory->Entries[1].AddError = ERR_NONE;
 		Memory->Entries[1].VoiceTag = 0;
 		Memory->Entries[1].SMSList[0] = 0;
-		Memory->Entries[2].EntryType = PBK_Number_Work;
+		Memory->Entries[2].EntryType = PBK_Number_General;
+		Memory->Entries[2].Location = PBK_Location_Work;
 		Memory->Entries[2].AddError = ERR_NONE;
 		Memory->Entries[2].VoiceTag = 0;
 		Memory->Entries[2].SMSList[0] = 0;
 		Memory->Entries[3].EntryType = PBK_Number_Fax;
+		Memory->Entries[3].Location = PBK_Location_Unknown;
 		Memory->Entries[3].AddError = ERR_NONE;
 		Memory->Entries[3].VoiceTag = 0;
 		Memory->Entries[3].SMSList[0] = 0;
 		Memory->Entries[4].EntryType = PBK_Number_General;
+		Memory->Entries[4].Location = PBK_Location_Unknown;
 		Memory->Entries[4].AddError = ERR_NONE;
 		Memory->Entries[4].VoiceTag = 0;
 		Memory->Entries[4].SMSList[0] = 0;
 		Memory->Entries[5].EntryType = PBK_Text_Email;
+		Memory->Entries[5].Location = PBK_Location_Unknown;
 		Memory->Entries[5].AddError = ERR_NONE;
 		Memory->Entries[5].VoiceTag = 0;
 		Memory->Entries[5].SMSList[0] = 0;
 		Memory->Entries[6].EntryType = PBK_Text_FirstName;
+		Memory->Entries[6].Location = PBK_Location_Unknown;
 		Memory->Entries[6].AddError = ERR_NONE;
 		Memory->Entries[6].VoiceTag = 0;
 		Memory->Entries[6].SMSList[0] = 0;
 		Memory->Entries[7].EntryType = PBK_Text_LastName;
+		Memory->Entries[7].Location = PBK_Location_Unknown;
 		Memory->Entries[7].AddError = ERR_NONE;
 		Memory->Entries[7].VoiceTag = 0;
 		Memory->Entries[7].SMSList[0] = 0;
 		Memory->Entries[8].EntryType = PBK_Text_Note;
+		Memory->Entries[8].Location = PBK_Location_Unknown;
 		Memory->Entries[8].AddError = ERR_NONE;
 		Memory->Entries[8].VoiceTag = 0;
 		Memory->Entries[8].SMSList[0] = 0;
 
 		/* Get line from reply */
-		str = GetLineString(msg.Buffer, &Priv->Lines, 2);
+		str = GetLineString(msg->Buffer, &Priv->Lines, 2);
 
 		/* Empty entry */
 		if (strcmp(str, "OK") == 0) return ERR_EMPTY;
 
+		/* Philips has it's SPBR as well, but different reply */
+		if ( Priv->Manufacturer == AT_Philips) {
+			error = ATGEN_ParseReply(s,
+						GetLineString(msg->Buffer, &Priv->Lines, 2),
+						"+SPBR: @n, @u, @p",
+						&Memory->Location,
+						Memory->Entries[0].Text, sizeof(Memory->Entries[0].Text),
+						Memory->Entries[1].Text, sizeof(Memory->Entries[1].Text));
+			if (error == ERR_NONE) {
+				/* Set name type */
+				Memory->Entries[0].EntryType = PBK_Text_Name;
+				Memory->Entries[0].Location = PBK_Location_Unknown;
+
+				/* Set number type */
+				Memory->Entries[1].EntryType = PBK_Number_General;
+				Memory->Entries[1].Location = PBK_Location_Unknown;
+				Memory->Entries[1].VoiceTag = 0;
+				Memory->Entries[1].SMSList[0] = 0;
+
+				return ERR_NONE;
+			}
+		}
+
 		/*
 		 * Parse reply string
 		 *
@@ -577,7 +609,7 @@ GSM_Error SAMSUNG_ReplyGetMemory(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		if (error != ERR_NONE) {
 			/*
 			 * Some phones have different string:
-			 * +SPBR: 1,"+919821485060","","","","","apsengupta@gmail.com","6,Aditya","8,Sengupta",1985,2,31,"2,Me","0,"
+			 * +SPBR: 1,"+999999999999","","","","","aaaaaaaaaa@gmail.com","6,Aaaaaa","8,Ssssssss",1999,1,31,"2,Me","0,"
 			 */
 			error = ATGEN_ParseReply(s, str,
 						"+SPBR: @i, @p, @p, @p, @p, @p, @s, @T, @T, @i, @i, @i, @T, @T",
@@ -603,6 +635,7 @@ GSM_Error SAMSUNG_ReplyGetMemory(GSM_Protocol_Message msg, GSM_StateMachine *s)
 				for (j = i + 1; j < Memory->EntriesNum; j++) {
 					CopyUnicodeString(Memory->Entries[j - 1].Text, Memory->Entries[j].Text);
 					Memory->Entries[j - 1].EntryType = Memory->Entries[j].EntryType;
+					Memory->Entries[j - 1].Location = Memory->Entries[j].Location;
 				}
 				Memory->EntriesNum--;
 			}
@@ -610,6 +643,7 @@ GSM_Error SAMSUNG_ReplyGetMemory(GSM_Protocol_Message msg, GSM_StateMachine *s)
 		/* Was there stored birthday? */
 		if (year > 1900) {
 			Memory->Entries[Memory->EntriesNum].EntryType = PBK_Date;
+			Memory->Entries[Memory->EntriesNum].Location = PBK_Location_Unknown;
 			Memory->Entries[Memory->EntriesNum].Date.Year = year;
 			Memory->Entries[Memory->EntriesNum].Date.Month = month;
 			Memory->Entries[Memory->EntriesNum].Date.Day = day;
@@ -656,7 +690,7 @@ GSM_Error SAMSUNG_SetMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 	return ERR_NOTIMPLEMENTED;
 }
 
-GSM_Error SAMSUNG_ORG_ReplyGetCalendarStatus(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_ORG_ReplyGetCalendarStatus(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Phone_ATGENData	*Priv = &s->Phone.Data.Priv.ATGEN;
 	GSM_Error error;
@@ -675,8 +709,12 @@ GSM_Error SAMSUNG_ORG_ReplyGetCalendarStatus(GSM_Protocol_Message msg, GSM_State
 		}
 	}
 
+	if (strcmp(GetLineString(msg->Buffer, &Priv->Lines, 2), "OK") == 0) {
+		return ERR_NOTSUPPORTED;
+	}
+
 	error = ATGEN_ParseReply(s,
-		GetLineString(msg.Buffer, &Priv->Lines, 2),
+		GetLineString(msg->Buffer, &Priv->Lines, 2),
 		"+ORGI: @i, @i, @i, @i, @i",
 		&s->Phone.Data.CalStatus->Used,
 		&s->Phone.Data.CalStatus->Free,
@@ -688,7 +726,7 @@ GSM_Error SAMSUNG_ORG_ReplyGetCalendarStatus(GSM_Protocol_Message msg, GSM_State
 	return ERR_NONE;
 }
 
-GSM_Error SAMSUNG_SSH_ReplyGetCalendarStatus(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_SSH_ReplyGetCalendarStatus(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Phone_ATGENData	*Priv = &s->Phone.Data.Priv.ATGEN;
 	GSM_Error error;
@@ -708,7 +746,7 @@ GSM_Error SAMSUNG_SSH_ReplyGetCalendarStatus(GSM_Protocol_Message msg, GSM_State
 	}
 
 	error = ATGEN_ParseReply(s,
-		GetLineString(msg.Buffer, &Priv->Lines, 2),
+		GetLineString(msg->Buffer, &Priv->Lines, 2),
 		"+SSHI: @i, @i, @i",
 		&s->Phone.Data.CalStatus->Used,
 		&s->Phone.Data.CalStatus->Free,
@@ -943,7 +981,7 @@ par24: Empty
 	return ERR_NONE;
 }
 
-GSM_Error SAMSUNG_ORG_ReplyGetCalendar(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_ORG_ReplyGetCalendar(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Phone_ATGENData	*Priv = &s->Phone.Data.Priv.ATGEN;
 	GSM_Error error;
@@ -963,7 +1001,7 @@ GSM_Error SAMSUNG_ORG_ReplyGetCalendar(GSM_Protocol_Message msg, GSM_StateMachin
 		}
 	}
 
-	line = GetLineString(msg.Buffer, &Priv->Lines, 2);
+	line = GetLineString(msg->Buffer, &Priv->Lines, 2);
 
 	if (strcmp("OK", line) == 0) {
 		return ERR_EMPTY;
@@ -997,7 +1035,7 @@ GSM_Error SAMSUNG_ORG_ReplyGetCalendar(GSM_Protocol_Message msg, GSM_StateMachin
 	}
 }
 
-GSM_Error SAMSUNG_SSH_ReplyGetCalendar(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_SSH_ReplyGetCalendar(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	return ERR_NOTIMPLEMENTED;
 }
@@ -1069,7 +1107,7 @@ GSM_Error SAMSUNG_GetCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 	return error;
 }
 
-GSM_Error SAMSUNG_ORG_ReplySetCalendar(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error SAMSUNG_ORG_ReplySetCalendar(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	return ERR_NOTIMPLEMENTED;
 }
