@@ -30,6 +30,7 @@
 #include "../../misc/misc.h"
 #include "../pfunc.h"
 #include "../at/atfunc.h"
+#include "../at/sonyericsson.h"
 #include "../obex/obexfunc.h"
 #include "atobex.h"
 
@@ -70,6 +71,11 @@ GSM_Error ATOBEX_SetATMode(GSM_StateMachine *s)
 		if (error != ERR_NONE) return error;
 	}
 
+	/* Give Samsung phones some time to recover from protocol switch */
+	if (Priv->HasOBEX == ATOBEX_OBEX_MOBEX || Priv->HasOBEX == ATOBEX_OBEX_TSSPCSW) {
+		sleep(2);
+	}
+
 	/* Initialise AT protocol */
 	error = s->Protocol.Functions->Initialise(s);
 	if (error != ERR_NONE) return error;
@@ -128,6 +134,14 @@ GSM_Error ATOBEX_SetOBEXMode(GSM_StateMachine *s, OBEX_Service service)
 			/* Siemens extension */
 			error = GSM_WaitFor (s, "AT^SQWE=3\r", 10, 0x00, 20, ID_SetOBEX);
 			break;
+		case ATOBEX_OBEX_MOBEX:
+			/* Samsung extension */
+			error = GSM_WaitFor (s, "AT+SYNCML=MOBEXSTART\r", 21, 0x00, 20, ID_SetOBEX);
+			break;
+		case ATOBEX_OBEX_TSSPCSW:
+			/* Samsung extension */
+			error = GSM_WaitFor (s, "AT$TSSPCSW=1\r", 13, 0x00, 20, ID_SetOBEX);
+			break;
 		case ATOBEX_OBEX_None:
 			break;
 	}
@@ -190,6 +204,7 @@ GSM_Error ATOBEX_Initialise(GSM_StateMachine *s)
 
 	/* This can be filled in by AT module init */
 	Priv->HasOBEX = ATOBEX_OBEX_None;
+	Priv->DataService = OBEX_None;
 
 	/* Init AT module */
 	/* This also enables ATOBEX_OBEX_CPROT0 if available */
@@ -199,16 +214,27 @@ GSM_Error ATOBEX_Initialise(GSM_StateMachine *s)
 	/* Does phone have support for AT+MODE=22 switching? */
 	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_MODE22)) {
 		Priv->HasOBEX = ATOBEX_OBEX_MODE22;
+		Priv->DataService = OBEX_IRMC;
 	} else if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_XLNK)) {
 		Priv->HasOBEX = ATOBEX_OBEX_XLNK;
+		Priv->DataService = OBEX_IRMC;
 	} else if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SQWE)) {
 		Priv->HasOBEX = ATOBEX_OBEX_SQWE;
+		Priv->DataService = OBEX_IRMC;
 	} else if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_CPROT)) {
 		Priv->HasOBEX = ATOBEX_OBEX_CPROT0;
+		Priv->DataService = OBEX_IRMC;
+	} else if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_MOBEX)) {
+		Priv->HasOBEX = ATOBEX_OBEX_MOBEX;
+		Priv->DataService = OBEX_m_OBEX;
+	} else if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_TSSPCSW)) {
+		Priv->HasOBEX = ATOBEX_OBEX_TSSPCSW;
+		Priv->DataService = OBEX_m_OBEX;
 	} else {
 		if (PrivAT->Mode) {
 			smprintf(s, "Guessed mode style switching\n");
 			Priv->HasOBEX = ATOBEX_OBEX_MODE22;
+			Priv->DataService = OBEX_IRMC;
 		}
 	}
 
@@ -217,6 +243,7 @@ GSM_Error ATOBEX_Initialise(GSM_StateMachine *s)
 		error = GSM_WaitFor (s, "AT*EOBEX=?\r", 11, 0x00, 4, ID_SetOBEX);
 		if (error == ERR_NONE) {
 			Priv->HasOBEX = ATOBEX_OBEX_EOBEX;
+			Priv->DataService = OBEX_IRMC;
 		}
 	}
 
@@ -469,7 +496,7 @@ GSM_Error ATOBEX_GetSMSC(GSM_StateMachine *s, GSM_SMSC *smsc)
 	return ATGEN_GetSMSC(s, smsc);
 }
 
-GSM_Error ATOBEX_EnterSecurityCode(GSM_StateMachine *s, GSM_SecurityCode Code)
+GSM_Error ATOBEX_EnterSecurityCode(GSM_StateMachine *s, GSM_SecurityCode *Code)
 {
 	GSM_Error error;
 
@@ -616,7 +643,10 @@ GSM_Error ATOBEX_AddFilePart(GSM_StateMachine *s, GSM_File *File, int *Pos, int
 {
 	GSM_Error error;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_AddFilePart(s, File, Pos, Handle);
 }
 
@@ -624,7 +654,10 @@ GSM_Error ATOBEX_SendFilePart(GSM_StateMachine *s, GSM_File *File, int *Pos, int
 {
 	GSM_Error error;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_None))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, OBEX_None);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_SendFilePart(s, File, Pos, Handle);
 }
 
@@ -632,7 +665,10 @@ GSM_Error ATOBEX_GetFilePart(GSM_StateMachine *s, GSM_File *File, int *Handle, i
 {
 	GSM_Error error;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetFilePart(s, File, Handle, Size);
 }
 
@@ -640,7 +676,10 @@ GSM_Error ATOBEX_GetNextFileFolder(GSM_StateMachine *s, GSM_File *File, gboolean
 {
 	GSM_Error error;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetNextFileFolder(s, File, start);
 }
 
@@ -648,7 +687,10 @@ GSM_Error ATOBEX_DeleteFile(GSM_StateMachine *s, unsigned char *ID)
 {
 	GSM_Error error;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_DeleteFile(s, ID);
 }
 
@@ -656,11 +698,26 @@ GSM_Error ATOBEX_AddFolder(GSM_StateMachine *s, GSM_File *File)
 {
 	GSM_Error error;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, OBEX_BrowsingFolders);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_AddFolder(s, File);
 }
 
 /*@}*/
+
+static inline int ATOBEX_UseObex (GSM_StateMachine *s, GSM_MemoryType type)
+{
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
+	if (type == MEM_ME)
+		return 1;
+	if (type != MEM_SM)
+		return 0;
+	if (Priv->HasOBEX == ATOBEX_OBEX_MOBEX || Priv->HasOBEX == ATOBEX_OBEX_TSSPCSW)
+		return 1;
+	return 0;
+}
 /**
  * \defgroup SEATIrMC Mixed AT mode/IrMC functions
  * \ingroup SEPhone
@@ -670,105 +727,147 @@ GSM_Error ATOBEX_AddFolder(GSM_StateMachine *s, GSM_File *File)
 GSM_Error ATOBEX_GetMemoryStatus(GSM_StateMachine *s, GSM_MemoryStatus *Status)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if (Status->MemoryType == MEM_ME) {
-		if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE)
+	if (ATOBEX_UseObex (s, Status->MemoryType)) {
+		error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+		if (error != ERR_NONE) {
 			goto atgen;
+		}
 		return OBEXGEN_GetMemoryStatus(s, Status);
 	}
 
 atgen:
-	if ((error = ATOBEX_SetATMode(s))!= ERR_NONE) return error;
+	error = ATOBEX_SetATMode(s);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return ATGEN_GetMemoryStatus(s, Status);
 }
 
 GSM_Error ATOBEX_GetMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if (entry->MemoryType == MEM_ME) {
-		if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE)
+	if (ATOBEX_UseObex (s, entry->MemoryType)) {
+		error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+		if (error != ERR_NONE) {
 			goto atgen;
+		}
 		return OBEXGEN_GetMemory(s, entry);
 	}
 
 atgen:
-	if ((error = ATOBEX_SetATMode(s))!= ERR_NONE) return error;
+	error = ATOBEX_SetATMode(s);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return ATGEN_GetMemory(s, entry);
 }
 
 GSM_Error ATOBEX_GetNextMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry, gboolean start)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if (entry->MemoryType == MEM_ME) {
-		if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE)
+	if (ATOBEX_UseObex (s, entry->MemoryType)) {
+		error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+		if (error != ERR_NONE) {
 			goto atgen;
+		}
 		return OBEXGEN_GetNextMemory(s, entry, start);
 	}
 
 atgen:
-	if ((error = ATOBEX_SetATMode(s))!= ERR_NONE) return error;
+	error = ATOBEX_SetATMode(s);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return ATGEN_GetNextMemory(s, entry, start);
 }
 
 GSM_Error ATOBEX_SetMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if (entry->MemoryType == MEM_ME) {
-		if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE)
+	if (ATOBEX_UseObex (s, entry->MemoryType)) {
+		error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+		if (error != ERR_NONE) {
 			goto atgen;
+		}
 		return OBEXGEN_SetMemory(s, entry);
 	}
 
 atgen:
-	if ((error = ATOBEX_SetATMode(s))!= ERR_NONE) return error;
+	error = ATOBEX_SetATMode(s);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return ATGEN_SetMemory(s, entry);
 }
 
 GSM_Error ATOBEX_AddMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if (entry->MemoryType == MEM_ME) {
-		if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE)
+	if (ATOBEX_UseObex (s, entry->MemoryType)) {
+		error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+		if (error != ERR_NONE) {
 			goto atgen;
+		}
 		return OBEXGEN_AddMemory(s, entry);
 	}
 
 atgen:
-	if ((error = ATOBEX_SetATMode(s))!= ERR_NONE) return error;
+	error = ATOBEX_SetATMode(s);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return ATGEN_AddMemory(s, entry);
 }
 
 GSM_Error ATOBEX_DeleteMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if (entry->MemoryType == MEM_ME) {
-		if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE)
+	if (ATOBEX_UseObex (s, entry->MemoryType)) {
+		error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+		if (error != ERR_NONE) {
 			goto atgen;
+		}
 		return OBEXGEN_DeleteMemory(s, entry);
 	}
 
 atgen:
-	if ((error = ATOBEX_SetATMode(s))!= ERR_NONE) return error;
+	error = ATOBEX_SetATMode(s);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return ATGEN_DeleteMemory(s, entry);
 }
 
 GSM_Error ATOBEX_DeleteAllMemory(GSM_StateMachine *s, GSM_MemoryType type)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if (type == MEM_ME) {
-		if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE)
+	if (ATOBEX_UseObex (s, type)) {
+		error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+		if (error != ERR_NONE) {
 			goto atgen;
+		}
 		return OBEXGEN_DeleteAllMemory(s, type);
 	}
 
 atgen:
-	if ((error = ATOBEX_SetATMode(s))!= ERR_NONE) return error;
+	error = ATOBEX_SetATMode(s);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return ATGEN_DeleteAllMemory(s, type);
 }
 
@@ -782,168 +881,252 @@ atgen:
 GSM_Error ATOBEX_GetToDoStatus(GSM_StateMachine *s, GSM_ToDoStatus *status)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetTodoStatus(s, status);
 }
 
 GSM_Error ATOBEX_GetToDo (GSM_StateMachine *s, GSM_ToDoEntry *ToDo)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetTodo(s, ToDo);
 }
 
 GSM_Error ATOBEX_GetNextToDo(GSM_StateMachine *s, GSM_ToDoEntry *ToDo, gboolean start)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetNextTodo(s, ToDo, start);
 }
 
 GSM_Error ATOBEX_DeleteAllToDo (GSM_StateMachine *s)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_DeleteAllTodo(s);
 }
 
 GSM_Error ATOBEX_AddToDo (GSM_StateMachine *s, GSM_ToDoEntry *ToDo)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_AddTodo(s, ToDo);
 }
 
 GSM_Error ATOBEX_SetToDo (GSM_StateMachine *s, GSM_ToDoEntry *ToDo)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_SetTodo(s, ToDo);
 }
 
 GSM_Error ATOBEX_DeleteToDo (GSM_StateMachine *s, GSM_ToDoEntry *ToDo)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_DeleteTodo(s, ToDo);
 }
 
 GSM_Error ATOBEX_GetCalendarStatus(GSM_StateMachine *s, GSM_CalendarStatus *status)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetCalendarStatus(s, status);
 }
 
 GSM_Error ATOBEX_GetCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetCalendar(s, Note);
 }
 
 GSM_Error ATOBEX_GetNextCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note, gboolean start)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetNextCalendar(s, Note, start);
 }
 
 GSM_Error ATOBEX_DeleteCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_DeleteCalendar(s, Note);
 }
 
 GSM_Error ATOBEX_AddCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_AddCalendar(s, Note);
 }
 
 GSM_Error ATOBEX_SetCalendar(GSM_StateMachine *s, GSM_CalendarEntry *Note)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_SetCalendar(s, Note);
 }
 
 GSM_Error ATOBEX_DeleteAllCalendar (GSM_StateMachine *s)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_DeleteAllCalendar(s);
 }
 
 GSM_Error ATOBEX_GetNoteStatus(GSM_StateMachine *s, GSM_ToDoStatus *status)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetNoteStatus(s, status);
 }
 
 GSM_Error ATOBEX_GetNote (GSM_StateMachine *s, GSM_NoteEntry *Note)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetNote(s, Note);
 }
 
 GSM_Error ATOBEX_GetNextNote(GSM_StateMachine *s, GSM_NoteEntry *Note, gboolean start)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_GetNextNote(s, Note, start);
 }
 
 GSM_Error ATOBEX_DeleteAllNotes(GSM_StateMachine *s)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_DeleteAllNotes(s);
 }
 
 GSM_Error ATOBEX_AddNote (GSM_StateMachine *s, GSM_NoteEntry *Note)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_AddNote(s, Note);
 }
 
 GSM_Error ATOBEX_SetNote (GSM_StateMachine *s, GSM_NoteEntry *Note)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_SetNote(s, Note);
 }
 
 GSM_Error ATOBEX_DeleteNote (GSM_StateMachine *s, GSM_NoteEntry *Note)
 {
 	GSM_Error 		error;
+	GSM_Phone_ATOBEXData	*Priv = &s->Phone.Data.Priv.ATOBEX;
 
-	if ((error = ATOBEX_SetOBEXMode(s, OBEX_IRMC))!= ERR_NONE) return error;
+	error = ATOBEX_SetOBEXMode(s, Priv->DataService);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	return OBEXGEN_DeleteNote(s, Note);
 }
 
@@ -962,7 +1145,7 @@ GSM_Error ATOBEX_DeleteNote (GSM_StateMachine *s, GSM_NoteEntry *Note)
  * is an easy way to obtain the source under GPL, otherwise the author's parts
  * of this function are GPL 2.0.
  */
-GSM_Error ATOBEX_ReplyGetDateLocale(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATOBEX_ReplyGetDateLocale(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Locale	*locale = s->Phone.Data.Locale;
 	int		format;
@@ -971,7 +1154,7 @@ GSM_Error ATOBEX_ReplyGetDateLocale(GSM_Protocol_Message msg, GSM_StateMachine *
 	switch (s->Phone.Data.Priv.ATGEN.ReplyState) {
 	case AT_Reply_OK:
 		smprintf(s, "Date settings received\n");
-		pos = strstr(msg.Buffer, "*ESDF:");
+		pos = strstr(msg->Buffer, "*ESDF:");
 		if (pos == NULL) return ERR_UNKNOWNRESPONSE;
 		format = atoi(pos + 7);
 		switch (format) {
@@ -1013,7 +1196,7 @@ GSM_Error ATOBEX_ReplyGetDateLocale(GSM_Protocol_Message msg, GSM_StateMachine *
  * is an easy way to obtain the source under GPL, otherwise the author's parts
  * of this function are GPL 2.0.
  */
-GSM_Error ATOBEX_ReplyGetTimeLocale(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATOBEX_ReplyGetTimeLocale(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	int		format;
 	char		*pos;
@@ -1021,7 +1204,7 @@ GSM_Error ATOBEX_ReplyGetTimeLocale(GSM_Protocol_Message msg, GSM_StateMachine *
 	switch (s->Phone.Data.Priv.ATGEN.ReplyState) {
 		case AT_Reply_OK:
 			smprintf(s, "Time settings received\n");
-			pos = strstr(msg.Buffer, "*ESTF:");
+			pos = strstr(msg->Buffer, "*ESTF:");
 			if (pos == NULL) return ERR_UNKNOWNRESPONSE;
 			format = atoi(pos + 7);
 			switch (format) {
@@ -1095,14 +1278,14 @@ GSM_Error ATOBEX_SetLocale(GSM_StateMachine *s, GSM_Locale *locale)
 	return GSM_WaitFor (s, req, strlen(req), 0x00, 3, ID_SetLocale);
 }
 
-GSM_Error ATOBEX_ReplyGetFileSystemStatus(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATOBEX_ReplyGetFileSystemStatus(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Error error;
 
 	switch (s->Phone.Data.Priv.ATGEN.ReplyState) {
 		case AT_Reply_OK:
 			error = ATGEN_ParseReply(s,
-					GetLineString(msg.Buffer, &s->Phone.Data.Priv.ATGEN.Lines, 2),
+					GetLineString(msg->Buffer, &s->Phone.Data.Priv.ATGEN.Lines, 2),
 					"*EMEM: @i, @i, @i, @i, @i",
 					&s->Phone.Data.FileSystemStatus->Free,
 					&s->Phone.Data.FileSystemStatus->Used,
@@ -1134,7 +1317,7 @@ GSM_Error ATOBEX_GetFileSystemStatus(GSM_StateMachine *s, GSM_FileSystemStatus *
 	return GSM_WaitFor (s, "AT*EMEM\r", 8, 0x00, 3, ID_FileSystemStatus);
 }
 
-GSM_Error ATOBEX_ReplyGetBatteryCharge(GSM_Protocol_Message msg, GSM_StateMachine *s)
+GSM_Error ATOBEX_ReplyGetBatteryCharge(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	int tmp, ncapacity, method, state;
 	int vbat1, vbat2, vbat3, vbat4;
@@ -1149,7 +1332,7 @@ GSM_Error ATOBEX_ReplyGetBatteryCharge(GSM_Protocol_Message msg, GSM_StateMachin
 
 	/* Parse version 4 reply */
 	error = ATGEN_ParseReply(s,
-				msg.Buffer,
+				msg->Buffer,
 				"*EBCA: @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i\xd\xa",
 				&bat->BatteryVoltage, /* vbat */
 				&bat->ChargeVoltage, /* dcio */
@@ -1209,7 +1392,7 @@ GSM_Error ATOBEX_ReplyGetBatteryCharge(GSM_Protocol_Message msg, GSM_StateMachin
 
 	/* Parse version 2 reply */
 	error = ATGEN_ParseReply(s,
-				msg.Buffer,
+				msg->Buffer,
 				"*EBCA: @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i, @i\xd\xa",
 				&vbat1, /* vbat1 */
 				&vbat2, /* vbat2 */
@@ -1346,6 +1529,7 @@ GSM_Phone_Functions ATOBEXPhone = {
 	/* There is much more SE phones which support this! */
 	"sonyericsson|ericsson|atobex",
 	ATGENReplyFunctions,
+	NOTSUPPORTED,			/* 	Install			*/
 	ATOBEX_Initialise,
 	ATOBEX_Terminate,
 	ATOBEX_DispatchMessage,
@@ -1480,7 +1664,8 @@ GSM_Phone_Functions ATOBEXPhone = {
 	ATOBEX_AddFolder,
 	ATOBEX_DeleteFile,	/* 	DeleteFolder		*/
 	NOTSUPPORTED,			/* 	GetGPRSAccessPoint	*/
-	NOTSUPPORTED			/* 	SetGPRSAccessPoint	*/
+	NOTSUPPORTED,			/* 	SetGPRSAccessPoint	*/
+	SONYERICSSON_GetScreenshot
 };
 
 #endif
