@@ -59,14 +59,14 @@
 
 GSM_Error DUMMY_Error(GSM_StateMachine *s, const char *message)
 {
-	int i;
-	i = errno;
+	int tmp_errno;
+	tmp_errno = errno;
 	GSM_OSErrorInfo(s, message);
-	if (i == ENOENT) {
+	if (tmp_errno == ENOENT) {
 		return ERR_EMPTY;
-	} else if (i == EACCES) {
+	} else if (tmp_errno == EACCES) {
 		return ERR_PERMISSION;
-	} else if (i == EEXIST) {
+	} else if (tmp_errno == EEXIST) {
 		return ERR_FILEALREADYEXIST;
 	} else {
 		return ERR_UNKNOWN;
@@ -346,7 +346,7 @@ GSM_Error DUMMY_Initialise(GSM_StateMachine *s)
 	strcpy(s->Phone.Data.IMEI, "999999999999999");
 	strcpy(s->Phone.Data.Manufacturer, "Gammu");
 	strcpy(s->Phone.Data.Model, "Dummy");
-	strcpy(s->Phone.Data.Version, VERSION);
+	strcpy(s->Phone.Data.Version, GAMMU_VERSION);
 	strcpy(s->Phone.Data.VerDate, __DATE__);
 
 	EncodeUnicode(Priv->SMSC.Number, "123456", 6);
@@ -358,7 +358,11 @@ GSM_Error DUMMY_Initialise(GSM_StateMachine *s)
 	Priv->SMSC.Format = SMS_FORMAT_Text;
 	Priv->SMSC.Location = 1;
 
-	s->Phone.Data.VerNum = VERSION_NUM;
+	s->Phone.Data.VerNum = GAMMU_VERSION_NUM;
+
+	Priv->locale.AMPMTime = FALSE;
+	Priv->locale.DateFormat = GSM_Date_DDMMYYYY;
+	Priv->locale.DateSeparator = '.';
 
 	return ERR_NONE;
 }
@@ -430,8 +434,9 @@ GSM_Error DUMMY_GetPPM (GSM_StateMachine *s, char *value)
 
 GSM_Error DUMMY_GetFirmware(GSM_StateMachine *s)
 {
-	strcpy(s->Phone.Data.Version, VERSION);
+	strcpy(s->Phone.Data.Version, GAMMU_VERSION);
 	strcpy(s->Phone.Data.VerDate, __DATE__);
+	GSM_CreateFirmwareNumber(s);
 	return ERR_NONE;
 }
 
@@ -449,7 +454,7 @@ GSM_Error DUMMY_GetDateTime(GSM_StateMachine *s, GSM_DateTime *date_time)
 
 GSM_Error DUMMY_GetSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sms)
 {
-	GSM_SMS_Backup		Backup;
+	GSM_SMS_Backup *Backup;
 	char *filename;
 	GSM_Error error;
 	GSM_SMSMessage *SMS;
@@ -458,24 +463,29 @@ GSM_Error DUMMY_GetSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sms)
 
 	location = sms->SMS[0].Location;
 	folder = sms->SMS[0].Folder;
+	Backup = malloc(sizeof(GSM_SMS_Backup));
+	if (Backup == NULL) {
+		return ERR_MOREMEMORY;
+	}
 
 	filename = DUMMY_GetSMSPath(s, &(sms->SMS[0]));
 
-	error = GSM_ReadSMSBackupFile(filename, &Backup);
+	error = GSM_ReadSMSBackupFile(filename, Backup);
 
 	free(filename);
 	filename=NULL;
 
 	if (error != ERR_NONE) {
+		free(Backup);
 		if (error == ERR_CANTOPENFILE) return ERR_EMPTY;
 		return error;
 	}
 
 	sms->Number = 0;
 
-	for (SMS = Backup.SMS[i]; SMS != NULL; SMS = Backup.SMS[++i]) {
+	for (SMS = Backup->SMS[i]; SMS != NULL; SMS = Backup->SMS[++i]) {
 		sms->Number++;
-		sms->SMS[i] = *Backup.SMS[i];
+		sms->SMS[i] = *(Backup->SMS[i]);
 		sms->SMS[i].Location = location + (folder * DUMMY_MAX_SMS);
 		sms->SMS[i].Folder = folder;
 		switch (folder) {
@@ -501,7 +511,8 @@ GSM_Error DUMMY_GetSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sms)
 				break;
 		}
 	}
-	GSM_FreeSMSBackup(&Backup);
+	GSM_FreeSMSBackup(Backup);
+	free(Backup);
 
 	return ERR_NONE;
 }
@@ -527,19 +538,28 @@ GSM_Error DUMMY_SetSMS(GSM_StateMachine *s, GSM_SMSMessage *sms)
 {
 	char *filename=NULL;
 	GSM_Error error;
-	GSM_SMS_Backup backup;
+	GSM_SMS_Backup *Backup;
+
+	Backup = malloc(sizeof(GSM_SMS_Backup));
+	if (Backup == NULL) {
+		return ERR_MOREMEMORY;
+	}
 
 	error = DUMMY_DeleteSMS(s, sms);
 
-	if (error != ERR_EMPTY && error != ERR_NONE) return error;
+	if (error != ERR_EMPTY && error != ERR_NONE) {
+		free(Backup);
+		return error;
+	}
 
 	filename = DUMMY_GetSMSPath(s, sms);
 
-	backup.SMS[0] = sms;
-	backup.SMS[1] = NULL;
+	Backup->SMS[0] = sms;
+	Backup->SMS[1] = NULL;
 
-	error = GSM_AddSMSBackupFile(filename, &backup);
+	error = GSM_AddSMSBackupFile(filename, Backup);
 	free(filename);
+	free(Backup);
 	filename=NULL;
 	return error;
 }
@@ -591,6 +611,10 @@ GSM_Error DUMMY_GetNextSMS(GSM_StateMachine *s, GSM_MultiSMSMessage *sms, gboole
 {
 	char dirname[20]={0};
 
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_DISABLE_GETNEXTSMS)) {
+		return ERR_NOTSUPPORTED;
+	}
+
 	if (start) {
 		sms->SMS[0].Folder = 1;
 		sms->SMS[0].Location = 0;
@@ -661,7 +685,7 @@ GSM_Error DUMMY_DialService(GSM_StateMachine *s, char *number)
 		EncodeUnicode(ussd.Text, ussd_text, len);
 
 		/* Notify application */
-		s->User.IncomingUSSD(s, ussd, s->User.IncomingUSSDUserData);
+		s->User.IncomingUSSD(s, &ussd, s->User.IncomingUSSDUserData);
 	}
 
 	return ERR_NONE;
@@ -789,11 +813,11 @@ GSM_Error DUMMY_GetSMSC(GSM_StateMachine *s, GSM_SMSC *smsc)
 	return ERR_NONE;
 }
 
-GSM_Error DUMMY_EnterSecurityCode(GSM_StateMachine *s, GSM_SecurityCode Code)
+GSM_Error DUMMY_EnterSecurityCode(GSM_StateMachine *s, GSM_SecurityCode *Code)
 {
 	GSM_Phone_DUMMYData	*Priv = &s->Phone.Data.Priv.DUMMY;
 
-	fprintf(Priv->log_file, "Security code %d: %s\n", Code.Type, Code.Code);
+	fprintf(Priv->log_file, "Security code %d: %s\n", Code->Type, Code->Code);
 
 	return ERR_NONE;
 }
@@ -826,6 +850,38 @@ GSM_Error DUMMY_GetSIMIMSI(GSM_StateMachine *s, char *IMSI)
 	return ERR_NONE;
 }
 
+GSM_Error DUMMY_GetCallDivert(GSM_StateMachine *s, GSM_MultiCallDivert *divert)
+{
+	divert->Response.EntriesNum = 5;
+
+	divert->Response.Entries[0].DivertType = GSM_DIVERT_VoiceCalls;
+	divert->Response.Entries[0].CallType = GSM_DIVERT_Busy;
+	divert->Response.Entries[0].Timeout = 10;
+	EncodeUnicode(divert->Response.Entries[0].Number, "", 0);
+
+	divert->Response.Entries[1].DivertType = GSM_DIVERT_FaxCalls;
+	divert->Response.Entries[1].CallType = GSM_DIVERT_NoAnswer;
+	divert->Response.Entries[1].Timeout = 20;
+	EncodeUnicode(divert->Response.Entries[1].Number, "123", 3);
+
+	divert->Response.Entries[2].DivertType = GSM_DIVERT_DataCalls;
+	divert->Response.Entries[2].CallType = GSM_DIVERT_OutOfReach;
+	divert->Response.Entries[2].Timeout = 30;
+	EncodeUnicode(divert->Response.Entries[2].Number, "123", 3);
+
+	divert->Response.Entries[3].DivertType = GSM_DIVERT_VoiceCalls;
+	divert->Response.Entries[3].CallType = GSM_DIVERT_NoAnswer;
+	divert->Response.Entries[3].Timeout = 40;
+	EncodeUnicode(divert->Response.Entries[3].Number, "123", 3);
+
+	divert->Response.Entries[4].DivertType = GSM_DIVERT_AllCalls;
+	divert->Response.Entries[4].CallType = GSM_DIVERT_AllTypes;
+	divert->Response.Entries[4].Timeout = 0;
+	EncodeUnicode(divert->Response.Entries[4].Number, "", 0);
+
+	return ERR_NONE;
+}
+
 GSM_Error DUMMY_SetIncomingCall (GSM_StateMachine *s, gboolean enable)
 {
 	s->Phone.Data.EnableIncomingCall = enable;
@@ -964,7 +1020,7 @@ GSM_Error DUMMY_SetAlarm(GSM_StateMachine *s, GSM_Alarm *entry)
 	error = GSM_SaveBackupFile(filename, &backup, GSM_Backup_VCalendar);
 	free(filename);
 	filename=NULL;
-	return ERR_NONE;
+	return error;
 }
 
 GSM_Error DUMMY_SetIncomingUSSD(GSM_StateMachine *s, gboolean enable)
@@ -983,6 +1039,23 @@ GSM_Error DUMMY_SetRingtone(GSM_StateMachine *s, GSM_Ringtone *Ringtone, int *ma
 	return ERR_NOTIMPLEMENTED;
 }
 
+GSM_Error DUMMY_GetWAPBookmark(GSM_StateMachine *s, GSM_WAPBookmark *bookmark)
+{
+	switch (bookmark->Location) {
+		case 1:
+			EncodeUnicode(bookmark->Address, "http://blog.cihar.com/", 22);
+			EncodeUnicode(bookmark->Title, "Michals weblog", 14);
+			return ERR_NONE;
+		case 2:
+			EncodeUnicode(bookmark->Address, "http://wammu.eu/", 16);
+			EncodeUnicode(bookmark->Title, "Wammu website", 13);
+			return ERR_NONE;
+		default:
+			return ERR_EMPTY;
+
+	}
+}
+
 GSM_Error DUMMY_GetBitmap(GSM_StateMachine *s, GSM_Bitmap *Bitmap)
 {
 	return ERR_NOTIMPLEMENTED;
@@ -1366,6 +1439,10 @@ GSM_Error DUMMY_GetNextMemory(GSM_StateMachine *s, GSM_MemoryEntry *entry, gbool
 {
 	char dirname[20]={0};
 
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_DISABLE_GETNEXT)) {
+		return ERR_NOTSUPPORTED;
+	}
+
 	if (start) {
 		entry->Location = 0;
 	}
@@ -1475,6 +1552,10 @@ GSM_Error DUMMY_GetToDo(GSM_StateMachine *s, GSM_ToDoEntry *entry)
 
 GSM_Error DUMMY_GetNextToDo(GSM_StateMachine *s, GSM_ToDoEntry *entry, gboolean start)
 {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_DISABLE_GETNEXT)) {
+		return ERR_NOTSUPPORTED;
+	}
+
 	if (start) {
 		entry->Location = 0;
 	}
@@ -1575,6 +1656,10 @@ GSM_Error DUMMY_GetCalendar(GSM_StateMachine *s, GSM_CalendarEntry *entry)
 
 GSM_Error DUMMY_GetNextCalendar(GSM_StateMachine *s, GSM_CalendarEntry *entry, gboolean start)
 {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_DISABLE_GETNEXT)) {
+		return ERR_NOTSUPPORTED;
+	}
+
 	if (start) {
 		entry->Location = 0;
 	}
@@ -1687,6 +1772,10 @@ GSM_Error DUMMY_GetNote(GSM_StateMachine *s, GSM_NoteEntry *entry)
 
 GSM_Error DUMMY_GetNextNote(GSM_StateMachine *s, GSM_NoteEntry *entry, gboolean start)
 {
+	if (GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_DISABLE_GETNEXT)) {
+		return ERR_NOTSUPPORTED;
+	}
+
 	if (start) {
 		entry->Location = 0;
 	}
@@ -1752,12 +1841,16 @@ GSM_Error DUMMY_DeleteAllNote(GSM_StateMachine *s)
 
 GSM_Error DUMMY_GetLocale(GSM_StateMachine *s, GSM_Locale *locale)
 {
-	return ERR_NOTIMPLEMENTED;
+	GSM_Phone_DUMMYData *Priv = &s->Phone.Data.Priv.DUMMY;
+	*locale = Priv->locale;
+	return ERR_NONE;
 }
 
 GSM_Error DUMMY_SetLocale(GSM_StateMachine *s, GSM_Locale *locale)
 {
-	return ERR_NOTIMPLEMENTED;
+	GSM_Phone_DUMMYData *Priv = &s->Phone.Data.Priv.DUMMY;
+	Priv->locale = *locale;
+	return ERR_NONE;
 }
 
 GSM_Error DUMMY_GetFileSystemStatus(GSM_StateMachine *s, GSM_FileSystemStatus *Status)
@@ -1794,6 +1887,7 @@ GSM_Reply_Function DUMMYReplyFunctions[] = {
 GSM_Phone_Functions DUMMYPhone = {
 	"dummy",
 	DUMMYReplyFunctions,
+	NOTSUPPORTED,			/* 	Install			*/
 	DUMMY_Initialise,
 	DUMMY_Terminate,
 	DUMMY_DispatchMessage,
@@ -1862,7 +1956,7 @@ GSM_Phone_Functions DUMMYPhone = {
  	NOTSUPPORTED,			/* 	SplitCall		*/
  	NOTSUPPORTED,			/* 	TransferCall		*/
  	NOTSUPPORTED,			/* 	SwitchCall		*/
- 	NOTSUPPORTED,			/* 	GetCallDivert		*/
+	DUMMY_GetCallDivert,
  	NOTSUPPORTED,			/* 	SetCallDivert		*/
  	NOTSUPPORTED,			/* 	CancelAllDiverts	*/
 	DUMMY_SetIncomingCall,
@@ -1873,7 +1967,7 @@ GSM_Phone_Functions DUMMYPhone = {
 	NOTSUPPORTED,			/* 	GetRingtonesInfo	*/
 	NOTSUPPORTED,			/* 	DeleteUserRingtones	*/
 	NOTSUPPORTED,			/* 	PlayTone		*/
-	NOTSUPPORTED,			/* 	GetWAPBookmark		*/
+	DUMMY_GetWAPBookmark,
 	NOTSUPPORTED,			/* 	SetWAPBookmark		*/
 	NOTSUPPORTED,			/* 	DeleteWAPBookmark	*/
 	NOTSUPPORTED,			/* 	GetWAPSettings		*/
@@ -1928,7 +2022,8 @@ GSM_Phone_Functions DUMMYPhone = {
 	DUMMY_AddFolder,
 	DUMMY_DeleteFolder,
 	NOTSUPPORTED,			/* 	GetGPRSAccessPoint	*/
-	NOTSUPPORTED			/* 	SetGPRSAccessPoint	*/
+	NOTSUPPORTED,			/* 	SetGPRSAccessPoint	*/
+	NOTSUPPORTED			/* 	GetScreenshot		*/
 };
 
 /*@}*/
