@@ -12,7 +12,7 @@
 #include "at.h"
 
 static GSM_Error AT_WriteMessage (GSM_StateMachine *s, unsigned const char *buffer,
-				     int length, unsigned char type)
+				     int length, int type)
 {
 	int sent=0, write_data=0, i=0;
 
@@ -120,6 +120,7 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 		{"^DSFLOWRPT:"	,1}, /* ^DSFLOWRPT:00000124,00000082,00000EA6,0000000000012325,000000000022771D,0000BB80,0001F400 */
 		{"^BOOT:"	,1}, /* ^BOOT:27710117,0,0,0,75 */
 		{"^MODE:"	,1}, /* ^MODE:3,3 */
+		{"^CSNR:"	,1}, /* ^CSNR:-93,-23 */
 
 		/* ONDA */
 		{"+ZUSIMR:"	,1}, /* +ZUSIMR:2 */
@@ -186,7 +187,7 @@ static GSM_Error AT_StateMachine(GSM_StateMachine *s, unsigned char rx_char)
 						i++;
 						continue;
 					}
-				  	if (s->Phone.Data.RequestID == ID_SetOBEX &&
+					if ((s->Phone.Data.RequestID == ID_SetOBEX || s->Phone.Data.RequestID == ID_DialVoice)&&
 							strcmp(SpecialAnswers[i].text, "NO CARRIER") == 0) {
 						i++;
 						continue;
@@ -286,7 +287,8 @@ static GSM_Error AT_Initialise(GSM_StateMachine *s)
 	d->LineEnd		= -1;
 	d->wascrlf 		= FALSE;
 	d->EditMode		= FALSE;
-	d->FastWrite		= FALSE;
+	/* Slow write makes sense only on cable for some phones */
+	d->FastWrite		= (s->ConnectionType != GCT_AT);
 	d->CPINNoOK		= FALSE;
 
 	error = s->Device.Functions->DeviceSetParity(s, FALSE);
