@@ -367,71 +367,71 @@ GSM_DateTime	VCALTimeDiff ( GSM_DateTime *Alarm,  GSM_DateTime *Time)
 }
 
 
-GSM_Error GSM_Translate_Category (GSM_CatTranslation direction, char *string, GSM_CalendarNoteType *Type)
+GSM_Error GSM_Translate_Category_From_VCal (const char *string, GSM_CalendarNoteType *Type)
 {
 	/* Mozilla has user defined categories. These must be converted to GSM_CAL_xxx types.
 	   TODO: For now we use hardcoded conversions. Should be user configurable. */
 
-	switch (direction) {
-	case TRANSL_TO_GSM:
-		if (strstr(string,"MEETING")) 			*Type = GSM_CAL_MEETING;
-		else if (strstr(string,"REMINDER")) 		*Type = GSM_CAL_REMINDER;
-		else if (strstr(string,"DATE"))	 		*Type = GSM_CAL_REMINDER; /* SE */
-		else if (strstr(string,"TRAVEL"))	 	*Type = GSM_CAL_TRAVEL;   /* SE */
-		else if (strstr(string,"VACATION"))	 	*Type = GSM_CAL_VACATION; /* SE */
-		else if (strstr(string,"MISCELLANEOUS"))	*Type = GSM_CAL_MEMO;
-		else if (strstr(string,"PHONE CALL")) 		*Type = GSM_CAL_CALL;
-		else if (strstr(string,"SPECIAL OCCASION")) 	*Type = GSM_CAL_BIRTHDAY;
-		else if (strstr(string,"ANNIVERSARY")) 		*Type = GSM_CAL_BIRTHDAY;
-		else if (strstr(string,"APPOINTMENT")) 		*Type = GSM_CAL_MEETING;
-		else if (strstr(string,"SHOPPING LIST")) 	*Type = GSM_CAL_SHOPPING;
-		/* These are the Nokia 6230i categories in local language. */
-		else if (strstr(string,"Erinnerung"))	 	*Type = GSM_CAL_REMINDER;
-		else if (strstr(string,"Besprechung"))	 	*Type = GSM_CAL_MEETING;
-		else if (strstr(string,"Anrufen"))	 	*Type = GSM_CAL_CALL;
-		else if (strstr(string,"Geburtstag"))	 	*Type = GSM_CAL_BIRTHDAY;
-		else if (strstr(string,"Notiz"))	 	*Type = GSM_CAL_MEMO;
+	if (strstr(string,"MEETING")) 			*Type = GSM_CAL_MEETING;
+	else if (strstr(string,"REMINDER")) 		*Type = GSM_CAL_REMINDER;
+	else if (strstr(string,"DATE"))	 		*Type = GSM_CAL_REMINDER; /* SE */
+	else if (strstr(string,"TRAVEL"))	 	*Type = GSM_CAL_TRAVEL;   /* SE */
+	else if (strstr(string,"VACATION"))	 	*Type = GSM_CAL_VACATION; /* SE */
+	else if (strstr(string,"MISCELLANEOUS"))	*Type = GSM_CAL_MEMO;
+	else if (strstr(string,"PHONE CALL")) 		*Type = GSM_CAL_CALL;
+	else if (strstr(string,"SPECIAL OCCASION")) 	*Type = GSM_CAL_BIRTHDAY;
+	else if (strstr(string,"ANNIVERSARY")) 		*Type = GSM_CAL_BIRTHDAY;
+	else if (strstr(string,"APPOINTMENT")) 		*Type = GSM_CAL_MEETING;
+	else if (strstr(string,"SHOPPING LIST")) 	*Type = GSM_CAL_SHOPPING;
+	/* These are the Nokia 6230i categories in local language. */
+	else if (strstr(string,"Erinnerung"))	 	*Type = GSM_CAL_REMINDER;
+	else if (strstr(string,"Besprechung"))	 	*Type = GSM_CAL_MEETING;
+	else if (strstr(string,"Anrufen"))	 	*Type = GSM_CAL_CALL;
+	else if (strstr(string,"Geburtstag"))	 	*Type = GSM_CAL_BIRTHDAY;
+	else if (strstr(string,"Notiz"))	 	*Type = GSM_CAL_MEMO;
 
-		else if (strstr(string,"Reminder"))	 	*Type = GSM_CAL_REMINDER;
-		else if (strstr(string,"Meeting"))	 	*Type = GSM_CAL_MEETING;
-		else if (strstr(string,"Call"))		 	*Type = GSM_CAL_CALL;
-		else if (strstr(string,"Birthday"))	 	*Type = GSM_CAL_BIRTHDAY;
-		else if (strstr(string,"Memo"))		 	*Type = GSM_CAL_MEMO;
-		/* default */
-		else *Type = GSM_CAL_MEETING;
-		break;
+	else if (strstr(string,"Reminder"))	 	*Type = GSM_CAL_REMINDER;
+	else if (strstr(string,"Meeting"))	 	*Type = GSM_CAL_MEETING;
+	else if (strstr(string,"Call"))		 	*Type = GSM_CAL_CALL;
+	else if (strstr(string,"Birthday"))	 	*Type = GSM_CAL_BIRTHDAY;
+	else if (strstr(string,"Memo"))		 	*Type = GSM_CAL_MEMO;
+	/* default */
+	else *Type = GSM_CAL_MEETING;
 
-	case TRANSL_TO_VCAL:
-		switch (*Type) {
-			case GSM_CAL_CALL:
-				strcpy(string, "PHONE CALL");
-				break;
-			case GSM_CAL_MEETING:
-				strcpy(string, "MEETING");
-				break;
-			case GSM_CAL_REMINDER:
-				strcpy(string, "DATE");
-				break;
-			case GSM_CAL_TRAVEL:
-				strcpy(string, "TRAVEL");
-				break;
-			case GSM_CAL_VACATION:
-				strcpy(string, "VACATION");
-				break;
-			case GSM_CAL_BIRTHDAY:
-				strcpy(string, "ANNIVERSARY");
-				break;
-			case GSM_CAL_SHOPPING:
-				strcpy(string, "SHOPPING LIST");
-				break;
-			case GSM_CAL_MEMO:
-			default:
-				strcpy(string, "MISCELLANEOUS");
-				break;
-		}
-		break;
+	return ERR_NONE;
+}
+
+GSM_Error GSM_Translate_Category_To_VCal (char *string, const GSM_CalendarNoteType Type)
+{
+	switch (Type) {
+		case GSM_CAL_CALL:
+			strcpy(string, "PHONE CALL");
+			break;
+		case GSM_CAL_MEETING:
+			strcpy(string, "MEETING");
+			break;
+		case GSM_CAL_REMINDER:
+			strcpy(string, "DATE");
+			break;
+		case GSM_CAL_TRAVEL:
+			strcpy(string, "TRAVEL");
+			break;
+		case GSM_CAL_VACATION:
+			strcpy(string, "VACATION");
+			break;
+		case GSM_CAL_BIRTHDAY:
+			strcpy(string, "ANNIVERSARY");
+			break;
+		case GSM_CAL_SHOPPING:
+			strcpy(string, "SHOPPING LIST");
+			break;
+		case GSM_CAL_MEMO:
+		default:
+			strcpy(string, "MISCELLANEOUS");
+			break;
 	}
-	return 0;
+
+	return ERR_NONE;
 }
 
 /**
@@ -832,7 +832,7 @@ void GSM_ToDo_AdjustDate(GSM_ToDoEntry *note, GSM_DeltaTime *delta)
 	}
 }
 
-GSM_Error GSM_EncodeVCALENDAR(char *Buffer, const size_t buff_len, size_t *Length, GSM_CalendarEntry *note, gboolean header, GSM_VCalendarVersion Version)
+GSM_Error GSM_EncodeVCALENDAR(char *Buffer, const size_t buff_len, size_t *Length, GSM_CalendarEntry *note, const gboolean header, const GSM_VCalendarVersion Version)
 {
 	GSM_DateTime 	deltatime;
 	char 		dtstr[20];
@@ -859,7 +859,7 @@ GSM_Error GSM_EncodeVCALENDAR(char *Buffer, const size_t buff_len, size_t *Lengt
 	}
 
 	/* Store category */
-	GSM_Translate_Category(TRANSL_TO_VCAL, category, &note->Type);
+	GSM_Translate_Category_To_VCal(category, note->Type);
 	error = VC_StoreLine(Buffer, buff_len, Length,  "CATEGORIES:%s", category);
 	if (error != ERR_NONE) return error;
 
@@ -1056,7 +1056,7 @@ void GSM_ToDoFindDefaultTextTimeAlarmCompleted(GSM_ToDoEntry *entry, int *Text,
 	}
 }
 
-GSM_Error GSM_EncodeVTODO(char *Buffer, const size_t buff_len, size_t *Length, GSM_ToDoEntry *note, gboolean header, GSM_VToDoVersion Version)
+GSM_Error GSM_EncodeVTODO(char *Buffer, const size_t buff_len, size_t *Length, const GSM_ToDoEntry *note, const gboolean header, const GSM_VToDoVersion Version)
 {
 	char		category[100];
 	int		i;
@@ -1120,7 +1120,7 @@ GSM_Error GSM_EncodeVTODO(char *Buffer, const size_t buff_len, size_t *Length, G
 		}
 	}
 	/* Store category */
-	GSM_Translate_Category(TRANSL_TO_VCAL, category, &note->Type);
+	GSM_Translate_Category_To_VCal(category, note->Type);
 	error = VC_StoreLine(Buffer, buff_len, Length,  "CATEGORIES:%s", category);
 	if (error != ERR_NONE) return error;
 
@@ -1531,7 +1531,6 @@ GSM_Error GSM_DecodeVCAL_RRULE(GSM_Debug_Info *di, const char *Buffer, GSM_Calen
 						NEXT_NOSPACE(0);
 
 						while (isalpha((int)*pos)) {
-							have_info = TRUE;
 							GET_DOW(CAL_REPEAT_DAYOFWEEK, 0);
 							NEXT_NOSPACE(0);
 						}
@@ -1713,7 +1712,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 	gboolean		is_date_only;
 	gboolean		date_only = FALSE;
 	int		lBuffer;
- 	int 		Time=-1, Alarm=-1;
+ 	int 		Time=-1;
 	char		*rrule = NULL;
 
 	if (!Buffer) return ERR_EMPTY;
@@ -1739,14 +1738,14 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 				Calendar->Type = 0;
 				date_only = TRUE;
 				dstflag = 0;
-				Time=-1; Alarm=-1;
+				Time=-1;
 				Level 		= 1;
 			}
 			if (strstr(Line,"BEGIN:VTODO")) {
 				ToDo->Priority 	= GSM_Priority_None;
 				ToDo->Type = GSM_CAL_MEMO;
 				dstflag = 0;
-				Time=-1; Alarm=-1;
+				Time=-1;
 				Level 		= 2;
 			}
 			break;
@@ -1773,9 +1772,8 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 				if (Calendar->EntriesNum == 0) return ERR_EMPTY;
 
 				if (trigger.Timezone != -999 * 3600) {
-					Alarm = Calendar->EntriesNum;
-					Calendar->Entries[Alarm].Date = GSM_AddTime (Calendar->Entries[Time].Date, trigger);
-					Calendar->Entries[Alarm].EntryType = CAL_TONE_ALARM_DATETIME;
+					Calendar->Entries[Calendar->EntriesNum].Date = GSM_AddTime (Calendar->Entries[Time].Date, trigger);
+					Calendar->Entries[Calendar->EntriesNum].EntryType = CAL_TONE_ALARM_DATETIME;
 					Calendar->Entries[Calendar->EntriesNum].AddError = ERR_NONE;
 					Calendar->EntriesNum++;
 				}
@@ -1839,7 +1837,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 			/* Event type. Must be set correctly to let phone calendar work as expected. For example
 			   without GSM_CAL_MEETING the time part of an event date/time will be dropped. */
 			if (strstr(Line,"CATEGORIES:")) {
-				GSM_Translate_Category(TRANSL_TO_GSM, Line + 11, &Calendar->Type);
+				GSM_Translate_Category_From_VCal(Line + 11, &Calendar->Type);
 				break;
 			}
 
@@ -1854,30 +1852,25 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 
 			if ((ReadVCALText(Line, "SUMMARY", Buff, CalVer == Mozilla_iCalendar))) {
 				Calendar->Entries[Calendar->EntriesNum].EntryType = CAL_TEXT;
-				CopyUnicodeString(Calendar->Entries[Calendar->EntriesNum].Text,
-					DecodeUnicodeSpecialChars(Buff));
+				CopyUnicodeString(Calendar->Entries[Calendar->EntriesNum].Text, Buff);
 				Calendar->Entries[Calendar->EntriesNum].AddError = ERR_NONE;
 				Calendar->EntriesNum++;
 			}
 			if ((ReadVCALText(Line, "DESCRIPTION", Buff, CalVer == Mozilla_iCalendar))) {
-				CopyUnicodeString(Buff,DecodeUnicodeSpecialChars(Buff));
 				Calendar->Entries[Calendar->EntriesNum].EntryType = CAL_DESCRIPTION;
-				CopyUnicodeString(Calendar->Entries[Calendar->EntriesNum].Text,
-					DecodeUnicodeSpecialChars(Buff));
+				CopyUnicodeString(Calendar->Entries[Calendar->EntriesNum].Text, Buff);
 				Calendar->Entries[Calendar->EntriesNum].AddError = ERR_NONE;
 				Calendar->EntriesNum++;
 			}
 			if (ReadVCALText(Line, "LOCATION", Buff, CalVer == Mozilla_iCalendar)) {
 				Calendar->Entries[Calendar->EntriesNum].EntryType = CAL_LOCATION;
-				CopyUnicodeString(Calendar->Entries[Calendar->EntriesNum].Text,
-					DecodeUnicodeSpecialChars(Buff));
+				CopyUnicodeString(Calendar->Entries[Calendar->EntriesNum].Text, Buff);
 				Calendar->Entries[Calendar->EntriesNum].AddError = ERR_NONE;
 				Calendar->EntriesNum++;
 			}
 			if ((ReadVCALText(Line, "X-IRMC-LUID", Buff, CalVer == Mozilla_iCalendar))) {
 				Calendar->Entries[Calendar->EntriesNum].EntryType = CAL_LUID;
-				CopyUnicodeString(Calendar->Entries[Calendar->EntriesNum].Text,
-					DecodeUnicodeSpecialChars(Buff));
+				CopyUnicodeString(Calendar->Entries[Calendar->EntriesNum].Text, Buff);
 				Calendar->Entries[Calendar->EntriesNum].AddError = ERR_NONE;
 				Calendar->EntriesNum++;
 			}
@@ -1913,14 +1906,12 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 				} else {
 					Calendar->Entries[Calendar->EntriesNum].EntryType = CAL_SILENT_ALARM_DATETIME;
 				}
-				Alarm = Calendar->EntriesNum;
 				Calendar->Entries[Calendar->EntriesNum].AddError = ERR_NONE;
 				Calendar->EntriesNum++;
 			}
 			if (ReadVCALDate(Line, "AALARM", &Date, &is_date_only)) {
 				Calendar->Entries[Calendar->EntriesNum].Date = Date;
 				Calendar->Entries[Calendar->EntriesNum].EntryType = CAL_TONE_ALARM_DATETIME;
-				Alarm = Calendar->EntriesNum;
 				Calendar->Entries[Calendar->EntriesNum].AddError = ERR_NONE;
 				Calendar->EntriesNum++;
 			}
@@ -1958,7 +1949,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 			}
 
 			if (strstr(Line,"CATEGORIES:")) {
-				GSM_Translate_Category(TRANSL_TO_GSM, Line+11, &ToDo->Type);
+				GSM_Translate_Category_From_VCal(Line+11, &ToDo->Type);
 			}
 
 			if (strncmp(Line, "UID:", 4) == 0) {
@@ -2008,7 +1999,6 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 			if (ReadVCALDate(Line, "DALARM", &Date, &is_date_only)) {
 				ToDo->Entries[ToDo->EntriesNum].Date = Date;
 				ToDo->Entries[ToDo->EntriesNum].EntryType = TODO_SILENT_ALARM_DATETIME;
-				Alarm = Calendar->EntriesNum;
 				ToDo->EntriesNum++;
 			}
 			if (ReadVCALDate(Line, "LAST-MODIFIED", &Date, &is_date_only)) {
@@ -2019,26 +2009,22 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 			if (ReadVCALDate(Line, "AALARM", &Date, &is_date_only)) {
 				ToDo->Entries[ToDo->EntriesNum].Date = Date;
 				ToDo->Entries[ToDo->EntriesNum].EntryType = TODO_ALARM_DATETIME;
-				Alarm = Calendar->EntriesNum;
 				ToDo->EntriesNum++;
 			}
 
 			if ((ReadVCALText(Line, "SUMMARY", Buff, ToDoVer == Mozilla_VToDo))) {
 				ToDo->Entries[ToDo->EntriesNum].EntryType = TODO_TEXT;
-				CopyUnicodeString(ToDo->Entries[ToDo->EntriesNum].Text,
-					DecodeUnicodeSpecialChars(Buff));
+				CopyUnicodeString(ToDo->Entries[ToDo->EntriesNum].Text, Buff);
 				ToDo->EntriesNum++;
 			}
 			if ((ReadVCALText(Line, "DESCRIPTION", Buff, ToDoVer == Mozilla_VToDo))) {
 				ToDo->Entries[ToDo->EntriesNum].EntryType = TODO_DESCRIPTION;
-				CopyUnicodeString(ToDo->Entries[ToDo->EntriesNum].Text,
-					DecodeUnicodeSpecialChars(Buff));
+				CopyUnicodeString(ToDo->Entries[ToDo->EntriesNum].Text, Buff);
 				ToDo->EntriesNum++;
 			}
 			if ((ReadVCALText(Line, "LOCATION", Buff, ToDoVer == Mozilla_VToDo))) {
 				ToDo->Entries[ToDo->EntriesNum].EntryType = TODO_LOCATION;
-				CopyUnicodeString(ToDo->Entries[ToDo->EntriesNum].Text,
-					DecodeUnicodeSpecialChars(Buff));
+				CopyUnicodeString(ToDo->Entries[ToDo->EntriesNum].Text, Buff);
 				ToDo->EntriesNum++;
 			}
 			if (ReadVCALText(Line, "PRIORITY", Buff, ToDoVer == Mozilla_VToDo)) {
@@ -2059,8 +2045,7 @@ GSM_Error GSM_DecodeVCALENDAR_VTODO(GSM_Debug_Info *di, char *Buffer, size_t *Po
 			}
 			if ((ReadVCALText(Line, "X-IRMC-LUID", Buff, ToDoVer == Mozilla_VToDo))) {
 				ToDo->Entries[ToDo->EntriesNum].EntryType = TODO_LUID;
-				CopyUnicodeString(ToDo->Entries[ToDo->EntriesNum].Text,
-					DecodeUnicodeSpecialChars(Buff));
+				CopyUnicodeString(ToDo->Entries[ToDo->EntriesNum].Text, Buff);
 				ToDo->EntriesNum++;
 			}
 			if ((ReadVCALText(Line, "CLASS", Buff, ToDoVer == Mozilla_VToDo))) {
