@@ -49,7 +49,7 @@ static struct keys_table_position Keys[] = {
 GSM_Error MakeKeySequence(char *text, GSM_KeyCode *KeyCode, size_t *Length)
 {
 	int 		i,j;
-	unsigned char 	key;
+	char key;
 
 	for (i=0;i<(int)(strlen(text));i++) {
 		key 	   = text[i];
@@ -114,7 +114,7 @@ GSM_Error GSM_ReadFile(const char *FileName, GSM_File *File)
 	return ERR_NONE;
 }
 
-static void GSM_JADFindLine(GSM_File File, const char *Name, char *Value)
+static void GSM_JADFindLine(GSM_File *File, const char *Name, char *Value)
 {
 	unsigned char 	Line[2000];
 	size_t		Pos = 0;
@@ -122,7 +122,7 @@ static void GSM_JADFindLine(GSM_File File, const char *Name, char *Value)
 	Value[0] = 0;
 
 	while (1) {
-		if (MyGetLine(File.Buffer, &Pos, Line, File.Used, sizeof(Line), FALSE) != ERR_NONE) break;
+		if (MyGetLine(File->Buffer, &Pos, Line, File->Used, sizeof(Line), FALSE) != ERR_NONE) break;
 		if (strlen(Line) == 0) break;
 		if (!strncmp(Line,Name,strlen(Name))) {
 			Pos = strlen(Name);
@@ -133,7 +133,7 @@ static void GSM_JADFindLine(GSM_File File, const char *Name, char *Value)
 	}
 }
 
-GSM_Error GSM_JADFindData(GSM_File File, char *Vendor, char *Name, char *JAR, char *Version, int *Size)
+GSM_Error GSM_JADFindData(GSM_File *File, char *Vendor, char *Name, char *JAR, char *Version, int *Size)
 {
 	char Size2[200];
 
@@ -500,7 +500,7 @@ gboolean ReadVCALText(char *Buffer, const char *Start, unsigned char *Value, con
 		}
 	}
 	/* Store last token */
-	tokens[token++] = begin;
+	tokens[token] = begin;
 
 	/* Compare first token, it must be in place */
 	pos = Buffer;
@@ -662,6 +662,11 @@ gboolean ReadVCALText(char *Buffer, const char *Start, unsigned char *Value, con
 						Value[(2 * j) + 1] == 'N')
 						) {
 				Value[(2 * i) + 1] = '\n';
+			} else if (Value[(2 * j)] == 0 && (
+						Value[(2 * j) + 1] == 'r' ||
+						Value[(2 * j) + 1] == 'R')
+						) {
+				Value[(2 * i) + 1] = '\r';
 			} else if (Value[(2 * j)] == 0 && Value[(2 * j) + 1] == '\\') {
 				Value[(2 * i) + 1] = '\\';
 			} else if (Value[(2 * j)] == 0 && Value[(2 * j) + 1] == ';') {
