@@ -94,9 +94,8 @@ void GSM_PhonebookFindDefaultNameNumberGroup(const GSM_MemoryEntry *entry, int *
 		for (i = 0; i < entry->EntriesNum; i++) {
 			switch (entry->Entries[i].EntryType) {
 				case PBK_Number_Mobile:
-				case PBK_Number_Work:
+				case PBK_Number_General:
 				case PBK_Number_Fax:
-				case PBK_Number_Home:
 				case PBK_Number_Pager:
 				case PBK_Number_Other:
 					*Number = i;
@@ -127,7 +126,7 @@ void GSM_PhonebookFindDefaultNameNumberGroup(const GSM_MemoryEntry *entry, int *
 GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_len, size_t *Length, GSM_MemoryEntry *pbk, const gboolean header, const GSM_VCardVersion Version)
 {
 	int Name, Number, Group, i;
-	int firstname = -1, lastname = -1;
+	int firstname = -1, secondname = -1, lastname = -1;
 	int address = -1, city = -1, state = -1, zip = -1, country = -1;
 	int workaddress = -1, workcity = -1, workstate = -1, workzip = -1, workcountry = -1;
 	unsigned char buffer[1024];
@@ -183,52 +182,64 @@ GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_le
 					error = VC_Store(Buffer, buff_len, Length, "FN");
 					if (error != ERR_NONE) return error;
 					break;
+				case PBK_Text_NamePrefix:
+					error = VC_Store(Buffer, buff_len, Length, "X-NAME-PREFIX");
+					if (error != ERR_NONE) return error;
+					break;
+				case PBK_Text_NameSuffix:
+					error = VC_Store(Buffer, buff_len, Length, "X-NAME-SUFFIX");
+					if (error != ERR_NONE) return error;
+					break;
 				case PBK_Text_FirstName:
 					firstname = i;
 					ignore = TRUE;
 					break;
+				case PBK_Text_SecondName:
+					secondname = i;
+					ignore = TRUE;
+					break;
 				case PBK_Text_LastName:
 					lastname = i;
 					ignore = TRUE;
 					break;
 				case PBK_Text_StreetAddress:
-					address = i;
+					if (pbk->Entries[i].Location == PBK_Location_Work) {
+						workaddress = i;
+					} else {
+						address = i;
+					}
 					ignore = TRUE;
 					break;
 				case PBK_Text_City:
-					city = i;
+					if (pbk->Entries[i].Location == PBK_Location_Work) {
+						workcity = i;
+					} else {
+						city = i;
+					}
 					ignore = TRUE;
 					break;
 				case PBK_Text_State:
-					state = i;
+					if (pbk->Entries[i].Location == PBK_Location_Work) {
+						workstate = i;
+					} else {
+						state = i;
+					}
 					ignore = TRUE;
 					break;
 				case PBK_Text_Zip:
-					zip = i;
+					if (pbk->Entries[i].Location == PBK_Location_Work) {
+						workzip = i;
+					} else {
+						zip = i;
+					}
 					ignore = TRUE;
 					break;
 				case PBK_Text_Country:
-					country = i;
-					ignore = TRUE;
-					break;
-				case PBK_Text_WorkStreetAddress:
-					workaddress = i;
-					ignore = TRUE;
-					break;
-				case PBK_Text_WorkCity:
-					workcity = i;
-					ignore = TRUE;
-					break;
-				case PBK_Text_WorkState:
-					workstate = i;
-					ignore = TRUE;
-					break;
-				case PBK_Text_WorkZip:
-					workzip = i;
-					ignore = TRUE;
-					break;
-				case PBK_Text_WorkCountry:
-					workcountry = i;
+					if (pbk->Entries[i].Location == PBK_Location_Work) {
+						workcountry = i;
+					} else {
+						country = i;
+					}
 					ignore = TRUE;
 					break;
 				case PBK_Date:
@@ -242,15 +253,12 @@ GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_le
 					ignore = TRUE;
 					break;
 				case PBK_Number_General:
+				case PBK_Number_Video:
 				case PBK_Number_Other:
 				case PBK_Number_Pager:
 				case PBK_Number_Mobile  :
-				case PBK_Number_Work    :
 				case PBK_Number_Fax     :
-				case PBK_Number_Home    :
 				case PBK_Number_Messaging    :
-				case PBK_Number_Mobile_Work:
-				case PBK_Number_Mobile_Home:
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
 						ignore = TRUE;
 						break;
@@ -262,6 +270,18 @@ GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_le
 						error = VC_Store(Buffer, buff_len, Length, ";PREF");
 						if (error != ERR_NONE) return error;
 					}
+					switch (pbk->Entries[i].Location) {
+						case PBK_Location_Home:
+							error = VC_Store(Buffer, buff_len, Length, ";HOME");
+							if (error != ERR_NONE) return error;
+							break;
+						case PBK_Location_Work:
+							error = VC_Store(Buffer, buff_len, Length, ";WORK");
+							if (error != ERR_NONE) return error;
+							break;
+						case PBK_Location_Unknown:
+							break;
+					}
 					switch (pbk->Entries[i].EntryType) {
 						case PBK_Number_Other:
 							error = VC_Store(Buffer, buff_len, Length, ";OTHER");
@@ -275,30 +295,18 @@ GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_le
 							error = VC_Store(Buffer, buff_len, Length, ";CELL");
 							if (error != ERR_NONE) return error;
 							break;
-						case PBK_Number_Mobile_Home:
-							error = VC_Store(Buffer, buff_len, Length, ";HOME;CELL");
-							if (error != ERR_NONE) return error;
-							break;
-						case PBK_Number_Mobile_Work:
-							error = VC_Store(Buffer, buff_len, Length, ";WORK;CELL");
-							if (error != ERR_NONE) return error;
-							break;
-						case PBK_Number_Work:
-							error = VC_Store(Buffer, buff_len, Length, ";WORK");
-							if (error != ERR_NONE) return error;
-							break;
 						case PBK_Number_Fax:
 							error = VC_Store(Buffer, buff_len, Length, ";FAX");
 							if (error != ERR_NONE) return error;
 							break;
-						case PBK_Number_Home:
-							error = VC_Store(Buffer, buff_len, Length, ";HOME");
-							if (error != ERR_NONE) return error;
-							break;
 						case PBK_Number_Messaging:
 							error = VC_Store(Buffer, buff_len, Length, ";MSG");
 							if (error != ERR_NONE) return error;
 							break;
+						case PBK_Number_Video:
+							error = VC_Store(Buffer, buff_len, Length, ";VIDEO");
+							if (error != ERR_NONE) return error;
+							break;
 						default:
 							break;
 					}
@@ -321,15 +329,13 @@ GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_le
 					 */
 					error = VC_StoreText(Buffer, buff_len, Length, pbk->Entries[i].Text, "LABEL", FALSE);
 					if (error != ERR_NONE) return error;
-					error = VC_Store(Buffer, buff_len, Length, "ADR;HOME");
-					if (error != ERR_NONE) return error;
-					break;
-				case PBK_Text_WorkPostal    :
-					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
-						ignore = TRUE;
-						break;
+					if (pbk->Entries[i].Location == PBK_Location_Work) {
+						error = VC_Store(Buffer, buff_len, Length, "ADR;WORK");
+					} else if (pbk->Entries[i].Location == PBK_Location_Home) {
+						error = VC_Store(Buffer, buff_len, Length, "ADR;HOME");
+					} else {
+						error = VC_Store(Buffer, buff_len, Length, "ADR");
 					}
-					error = VC_Store(Buffer, buff_len, Length, "ADR;WORK");
 					if (error != ERR_NONE) return error;
 					break;
 				case PBK_Text_Email     :
@@ -357,6 +363,54 @@ GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_le
 					error = VC_Store(Buffer, buff_len, Length, "X-IRMC-LUID");
 					if (error != ERR_NONE) return error;
 					break;
+				case PBK_Text_VOIP      :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = TRUE;
+						break;
+					}
+					error = VC_Store(Buffer, buff_len, Length, "X-SIP;VOIP");
+					if (error != ERR_NONE) return error;
+					break;
+				case PBK_Text_SWIS      :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = TRUE;
+						break;
+					}
+					error = VC_Store(Buffer, buff_len, Length, "X-SIP;SWIS");
+					if (error != ERR_NONE) return error;
+					break;
+				case PBK_Text_WVID      :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = TRUE;
+						break;
+					}
+					error = VC_Store(Buffer, buff_len, Length, "X-WV-ID");
+					if (error != ERR_NONE) return error;
+					break;
+				case PBK_Text_SIP      :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = TRUE;
+						break;
+					}
+					error = VC_Store(Buffer, buff_len, Length, "X-SIP");
+					if (error != ERR_NONE) return error;
+					break;
+				case PBK_Text_DTMF      :
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = TRUE;
+						break;
+					}
+					error = VC_Store(Buffer, buff_len, Length, "X-DTMF");
+					if (error != ERR_NONE) return error;
+					break;
+				case PBK_PushToTalkID:
+					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
+						ignore = TRUE;
+						break;
+					}
+					error = VC_Store(Buffer, buff_len, Length, "X-SIP;POC");
+					if (error != ERR_NONE) return error;
+					break;
 				case PBK_Text_JobTitle:
 					if (UnicodeLength(pbk->Entries[i].Text) == 0) {
 						ignore = TRUE;
@@ -426,7 +480,6 @@ GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_le
 				case PBK_Text_Custom3:
 				case PBK_Text_Custom4:
 				case PBK_Text_PictureName:
-				case PBK_PushToTalkID:
 					pbk->Entries[i].AddError = ERR_NOTSUPPORTED;
 					ignore = TRUE;
 					break;
@@ -437,7 +490,7 @@ GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_le
 			}
 		}
 		/* Save name if it is composed from parts */
-		if (firstname != -1 || lastname != -1) {
+		if (firstname != -1 || secondname != -1 || lastname != -1) {
 			pos = 0;
 			if (lastname != -1) {
 				CopyUnicodeString(buffer + 2*pos, pbk->Entries[lastname].Text);
@@ -450,6 +503,13 @@ GSM_Error GSM_EncodeVCARD(GSM_Debug_Info *di, char *Buffer, const size_t buff_le
 				CopyUnicodeString(buffer + 2*pos, pbk->Entries[firstname].Text);
 				pos += UnicodeLength(pbk->Entries[firstname].Text);
 			}
+			if (secondname != -1) {
+				buffer[2*pos] = 0;
+				buffer[2*pos + 1] = ' ';
+				pos++;
+				CopyUnicodeString(buffer + 2*pos, pbk->Entries[secondname].Text);
+				pos += UnicodeLength(pbk->Entries[secondname].Text);
+			}
 			buffer[2*pos] = 0;
 			buffer[2*pos + 1] = 0;
 			error = VC_StoreText(Buffer, buff_len, Length, buffer, "N", FALSE);
@@ -656,6 +716,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 				if (s == NULL) {
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Name;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					continue;
 				} else {
@@ -663,6 +724,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					/* Skip empty name */
 					if (UnicodeLength(Pbk->Entries[Pbk->EntriesNum].Text) > 0) {
 						Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_LastName;
+						Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 						Pbk->EntriesNum++;
 					}
 
@@ -671,6 +733,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					CHECK_NUM_ENTRIES;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_FirstName;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					continue;
 				}
@@ -690,6 +753,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 				while (isspace((int)*s) && *s) s++;
 
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Photo;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->Entries[Pbk->EntriesNum].Picture.Type = PICTURE_JPG;
 
 				/* We allocate here more memory than is actually required */
@@ -705,12 +769,27 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 
 			if (ReadVCALText(Line, "TEL",		   Buff,  (version >= 3)) ||
 			    ReadVCALText(Line, "TEL;VOICE",	     Buff,  (version >= 3)) ||
+			    ReadVCALText(Line, "TEL;TYPE=MAIN",	     Buff,  (version >= 3)) ||
 			    ReadVCALText(Line, "TEL;TYPE=VOICE",	     Buff,  (version >= 3))) {
 				if (Buff[1] == '+') {
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_General;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
+				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
+				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
+				Pbk->EntriesNum++;
+				continue;
+			}
+			if (ReadVCALText(Line, "TEL;VIDEO",	     Buff,  (version >= 3)) ||
+			    ReadVCALText(Line, "TEL;TYPE=VIDEO",	     Buff,  (version >= 3))) {
+				if (Buff[1] == '+') {
+					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
+				}
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Video;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -727,6 +806,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Mobile;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -741,7 +821,8 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
-				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Mobile_Work;
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Mobile;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Work;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -756,7 +837,8 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
-				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Mobile_Home;
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Mobile;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Home;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -772,7 +854,8 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
-				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Work;
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_General;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Work;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -789,6 +872,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Other;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -805,6 +889,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Other;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -821,6 +906,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Other;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -838,6 +924,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Fax;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -853,7 +940,8 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					GSM_TweakInternationalNumber(Buff, NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN);
 				}
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
-				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_Home;
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Number_General;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Home;
 				Pbk->Entries[Pbk->EntriesNum].SMSList[0] = 0;
 				Pbk->Entries[Pbk->EntriesNum].VoiceTag = 0;
 				Pbk->EntriesNum++;
@@ -862,23 +950,25 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 			if (ReadVCALText(Line, "TITLE", Buff,  (version >= 3))) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_JobTitle;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
 			if (ReadVCALText(Line, "NOTE", Buff,  (version >= 3))) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Note;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
 			if (ReadVCALText(Line, "LABEL", Buff,  (version >= 3)) ||
-			    ReadVCALText(Line, "ADR", Buff,  (version >= 3)) ||
-			    ReadVCALText(Line, "ADR;HOME", Buff,  (version >= 3))) {
+			    ReadVCALText(Line, "ADR", Buff,  (version >= 3))) {
 				pos = 0;
 				s = VCALGetTextPart(Buff, &pos); /* PO box, ignore for now */
 				if (s == NULL) {
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Postal;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					continue;
 				} else {
@@ -888,6 +978,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_StreetAddress;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					CHECK_NUM_ENTRIES;
 
@@ -895,6 +986,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_City;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					CHECK_NUM_ENTRIES;
 
@@ -902,6 +994,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_State;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					CHECK_NUM_ENTRIES;
 
@@ -909,6 +1002,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Zip;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					CHECK_NUM_ENTRIES;
 
@@ -916,16 +1010,19 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Country;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					continue;
 				}
 			}
-			if (ReadVCALText(Line, "ADR;WORK", Buff,  (version >= 3))) {
+			if (ReadVCALText(Line, "ADR;HOME", Buff,  (version >= 3)) ||
+			    ReadVCALText(Line, "ADR;TYPE=HOME", Buff,  (version >= 3))) {
 				pos = 0;
 				s = VCALGetTextPart(Buff, &pos); /* PO box, ignore for now */
 				if (s == NULL) {
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
-					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkPostal;
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Postal;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Home;
 					Pbk->EntriesNum++;
 					continue;
 				} else {
@@ -934,74 +1031,196 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
-					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkStreetAddress;
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_StreetAddress;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Home;
 					Pbk->EntriesNum++;
 					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
-					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkCity;
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_City;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Home;
 					Pbk->EntriesNum++;
 					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
-					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkState;
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_State;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Home;
 					Pbk->EntriesNum++;
 					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
-					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkZip;
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Zip;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Home;
 					Pbk->EntriesNum++;
 					CHECK_NUM_ENTRIES;
 
 					s = VCALGetTextPart(Buff, &pos);
 					if (s == NULL) continue;
 					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
-					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WorkCountry;
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Country;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Home;
+					Pbk->EntriesNum++;
+					continue;
+				}
+			}
+			if (ReadVCALText(Line, "ADR;WORK", Buff,  (version >= 3)) ||
+			    ReadVCALText(Line, "ADR;TYPE=WORK", Buff,  (version >= 3))) {
+				pos = 0;
+				s = VCALGetTextPart(Buff, &pos); /* PO box, ignore for now */
+				if (s == NULL) {
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Postal;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Work;
+					Pbk->EntriesNum++;
+					continue;
+				} else {
+					s = VCALGetTextPart(Buff, &pos); /* Don't know ... */
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_StreetAddress;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Work;
+					Pbk->EntriesNum++;
+					CHECK_NUM_ENTRIES;
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_City;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Work;
+					Pbk->EntriesNum++;
+					CHECK_NUM_ENTRIES;
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_State;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Work;
+					Pbk->EntriesNum++;
+					CHECK_NUM_ENTRIES;
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Zip;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Work;
+					Pbk->EntriesNum++;
+					CHECK_NUM_ENTRIES;
+
+					s = VCALGetTextPart(Buff, &pos);
+					if (s == NULL) continue;
+					CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text, s);
+					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Country;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Work;
 					Pbk->EntriesNum++;
 					continue;
 				}
 			}
 			if (ReadVCALText(Line, "EMAIL", Buff,  (version >= 3)) ||
 			    ReadVCALText(Line, "EMAIL;TYPE=OTHER", Buff,  (version >= 3)) ||
+			    ReadVCALText(Line, "EMAIL;TYPE=INTERNET", Buff,  (version >= 3)) ||
 			    ReadVCALText(Line, "EMAIL;INTERNET", Buff,  (version >= 3))) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Email;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
 			if (ReadVCALText(Line, "X-IRMC-LUID", Buff,  (version >= 3))) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_LUID;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
+				Pbk->EntriesNum++;
+				continue;
+			}
+			if (ReadVCALText(Line, "X-DTMF", Buff,  (version >= 3))) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_DTMF;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
+				Pbk->EntriesNum++;
+				continue;
+			}
+			if (ReadVCALText(Line, "X-SIP", Buff,  (version >= 3))) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_SIP;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
+				Pbk->EntriesNum++;
+				continue;
+			}
+			if (ReadVCALText(Line, "X-SIP;VOIP", Buff,  (version >= 3))) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_VOIP;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
+				Pbk->EntriesNum++;
+				continue;
+			}
+			if (ReadVCALText(Line, "X-WV-ID", Buff,  (version >= 3))) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_WVID;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
+				Pbk->EntriesNum++;
+				continue;
+			}
+			if (ReadVCALText(Line, "X-SIP;SWIS", Buff,  (version >= 3))) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_SWIS;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
+				Pbk->EntriesNum++;
+				continue;
+			}
+			if (ReadVCALText(Line, "X-SIP;POC", Buff,  (version >= 3))) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_PushToTalkID;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
 			if (ReadVCALText(Line, "URL", Buff,  (version >= 3))) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_URL;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
 			if (ReadVCALText(Line, "ORG", Buff,  (version >= 3))) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_Company;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
 			if (ReadVCALText(Line, "NICKNAME", Buff,  (version >= 3))) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_NickName;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
 			if (ReadVCALText(Line, "FN", Buff,  (version >= 3))) {
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_FormalName;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
+				Pbk->EntriesNum++;
+				continue;
+			}
+			if (ReadVCALText(Line, "X-NAME-PREFIX", Buff,  (version >= 3))) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_NamePrefix;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
+				Pbk->EntriesNum++;
+				continue;
+			}
+			if (ReadVCALText(Line, "X-NAME-SUFFIX", Buff,  (version >= 3))) {
+				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
+				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Text_NameSuffix;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
@@ -1009,12 +1228,14 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 				CopyUnicodeString(Pbk->Entries[Pbk->EntriesNum].Text,Buff);
 				Pbk->Entries[Pbk->EntriesNum].Number = -1;
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Category;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
 			if (ReadVCALText(Line, "BDAY", Buff,  (version >= 3))) {
 				if (ReadVCALDateTime(DecodeUnicodeString(Buff), &Pbk->Entries[Pbk->EntriesNum].Date)) {
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Date;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					continue;
 				}
@@ -1022,6 +1243,7 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 			if (ReadVCALText(Line, "LAST-MODIFIED", Buff,  (version >= 3))) {
 				if (ReadVCALDateTime(DecodeUnicodeString(Buff), &Pbk->Entries[Pbk->EntriesNum].Date)) {
 					Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_LastModified;
+					Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 					Pbk->EntriesNum++;
 					continue;
 				}
@@ -1029,12 +1251,14 @@ GSM_Error GSM_DecodeVCARD(GSM_Debug_Info *di, char *Buffer, size_t *Pos, GSM_Mem
 			if (ReadVCALText(Line, "X-PRIVATE", Buff,  (version >= 3))) {
 				Pbk->Entries[Pbk->EntriesNum].Number = atoi(DecodeUnicodeString(Buff));
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Private;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
 			if (ReadVCALText(Line, "X-CALLER-GROUP", Buff,  (version >= 3))) {
 				Pbk->Entries[Pbk->EntriesNum].Number = atoi(DecodeUnicodeString(Buff));
 				Pbk->Entries[Pbk->EntriesNum].EntryType = PBK_Caller_Group;
+				Pbk->Entries[Pbk->EntriesNum].Location = PBK_Location_Unknown;
 				Pbk->EntriesNum++;
 				continue;
 			}
