@@ -36,7 +36,7 @@ unsigned char GSM_MakeSMSIDFromTime(void)
 
 void GSM_Find_Free_Used_SMS2(GSM_Debug_Info *di, GSM_Coding_Type Coding,GSM_SMSMessage SMS, size_t *UsedText, size_t *FreeText, size_t *FreeBytes)
 {
-	size_t UsedBytes;
+	size_t UsedBytes = 0;
 
 	switch (Coding) {
 	case SMS_Coding_Default_No_Compression:
@@ -156,7 +156,7 @@ void GSM_MakeMultiPartSMS(GSM_Debug_Info *di, GSM_MultiSMSMessage	*SMS,
 			  int			Class,
 			  unsigned char		ReplaceMessage)
 {
-	size_t 		Len,UsedText,CopiedText,CopiedSMSText;
+	size_t 		Len,UsedText = 0,CopiedText = 0,CopiedSMSText = 0;
 	int		j;
 	unsigned char 	UDHID;
 	GSM_DateTime 	Date;
@@ -198,7 +198,7 @@ void GSM_MakeMultiPartSMS(GSM_Debug_Info *di, GSM_MultiSMSMessage	*SMS,
 }
 
 /* Calculates number of SMS and number of left chars in SMS */
-void GSM_SMSCounter(GSM_Debug_Info *di, size_t 		MessageLength,
+void GSM_SMSCounter(GSM_Debug_Info *di,
 		    unsigned char 	*MessageBuffer,
 		    GSM_UDH	 	UDHType,
 		    GSM_Coding_Type 	Coding,
@@ -209,7 +209,7 @@ void GSM_SMSCounter(GSM_Debug_Info *di, size_t 		MessageLength,
 	GSM_MultiSMSMessage 	MultiSMS;
 
 	MultiSMS.Number = 0;
-	GSM_MakeMultiPartSMS(di, &MultiSMS,MessageBuffer,MessageLength,UDHType,Coding,-1,FALSE);
+	GSM_MakeMultiPartSMS(di, &MultiSMS,MessageBuffer,UnicodeLength(MessageBuffer),UDHType,Coding,-1,FALSE);
 	GSM_Find_Free_Used_SMS2(di, Coding,MultiSMS.SMS[MultiSMS.Number-1], &UsedText, CharsLeft, &FreeBytes);
 	*SMSNum = MultiSMS.Number;
 }
@@ -242,7 +242,7 @@ static void GSM_EncodeSMS30MultiPartSMS(GSM_MultiPartSMSInfo *Info,
 			 * but on 3310 4.02 it was possible to save about 196 chars
 			 * (without cutting) */
 			len = 196;
-			Info->Entries[0].RingtoneNotes=GSM_EncodeNokiaRTTLRingtone(*Info->Entries[0].Ringtone,Buffer+(*Length),&len);
+			Info->Entries[0].RingtoneNotes=GSM_EncodeNokiaRTTLRingtone(Info->Entries[0].Ringtone,Buffer+(*Length),&len);
 			Buffer[(*Length)-2] = len / 256;
 			Buffer[(*Length)-1] = len % 256;
 			*Length = *Length + len;
@@ -515,12 +515,12 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_Debug_Info *di,
 		Class	= 1;
 		/* 7 = length of UDH_NokiaRingtone UDH header */
 		Length  = GSM_MAX_8BIT_SMS_LENGTH-7;
-		Info->Entries[0].RingtoneNotes = GSM_EncodeNokiaRTTLRingtone(*Info->Entries[0].Ringtone,Buffer,&Length);
+		Info->Entries[0].RingtoneNotes = GSM_EncodeNokiaRTTLRingtone(Info->Entries[0].Ringtone,Buffer,&Length);
 		if (Info->Entries[0].ID == SMS_NokiaRingtone) break;
 		if (Info->Entries[0].RingtoneNotes != Info->Entries[0].Ringtone->NoteTone.NrCommands) {
 			UDH    = UDH_NokiaRingtoneLong;
 			Length = (GSM_MAX_8BIT_SMS_LENGTH-12)*3;
-			Info->Entries[0].RingtoneNotes = GSM_EncodeNokiaRTTLRingtone(*Info->Entries[0].Ringtone,Buffer,&Length);
+			Info->Entries[0].RingtoneNotes = GSM_EncodeNokiaRTTLRingtone(Info->Entries[0].Ringtone,Buffer,&Length);
 		}
 		break;
 	case SMS_NokiaOperatorLogoLong:
@@ -1235,6 +1235,20 @@ GSM_Error GSM_LinkSMS(GSM_Debug_Info *di, GSM_MultiSMSMessage **InputMessages, G
 						InputMessages[i]->SMS[0].UDH.AllParts	= InputMessages[i]->SMS[0].UDH.Text[w+3];
 						InputMessages[i]->SMS[0].UDH.PartNumber	= InputMessages[i]->SMS[0].UDH.Text[w+4];
 						break;
+					case 0x05:
+						smfprintf(di, "Adding ID to user UDH - OTA/MMS with 8/16 bit ID\n");
+						if (InputMessages[i]->SMS[0].UDH.Text[w-1] == 6) {
+							InputMessages[i]->SMS[0].UDH.ID8bit = -1;
+							InputMessages[i]->SMS[0].UDH.ID16bit = InputMessages[i]->SMS[0].UDH.Text[w+2]*256+InputMessages[i]->SMS[0].UDH.Text[w+3];
+							InputMessages[i]->SMS[0].UDH.AllParts = 1;
+							InputMessages[i]->SMS[0].UDH.PartNumber = 1;
+						} else {
+							InputMessages[i]->SMS[0].UDH.ID16bit = -1;
+							InputMessages[i]->SMS[0].UDH.ID8bit = InputMessages[i]->SMS[0].UDH.Text[InputMessages[i]->SMS[0].UDH.Text[w-1] - 2];
+							InputMessages[i]->SMS[0].UDH.AllParts = InputMessages[i]->SMS[0].UDH.Text[InputMessages[i]->SMS[0].UDH.Text[w-1] - 1];
+							InputMessages[i]->SMS[0].UDH.PartNumber = InputMessages[i]->SMS[0].UDH.Text[InputMessages[i]->SMS[0].UDH.Text[w-1]];
+						}
+						break;
 					case 0x08:
 						smfprintf(di, "Adding ID to user UDH - linked SMS with 16 bit ID\n");
 						InputMessages[i]->SMS[0].UDH.ID8bit	= -1;
