@@ -101,6 +101,12 @@ static GSM_Error GSM_DecodeSMSDateTime(GSM_Debug_Info *di, GSM_DateTime *DT, con
 
 	if (req[6]&0x08) DT->Timezone = -DT->Timezone;
 
+	if (!CheckDate(DT) || !CheckTime(DT)) {
+		smfprintf(di, "Invalid date & time!\n");
+		DT->Year = 0;
+		return ERR_NONE;
+	}
+
 	smfprintf(di, "Decoding date & time: %s\n", OSDateTime(*DT, TRUE));
 
 	return ERR_NONE;
@@ -238,11 +244,16 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_Debug_Info *di, GSM_SMSMessage *SMS, unsign
 	if (buffer[Layout.firstbyte] & 64) {
 		/* Length of UDH header */
 		off = (buffer[Layout.Text] + 1);
+		if (off > buffer[Layout.TPUDL]) {
+			return ERR_CORRUPTED;
+		}
 		SMS->UDH.Length = off;
 		smfprintf(di, "UDH header available (length %i)\n",off);
 
 		/* Copy UDH header into SMS->UDH */
-		for (i = 0; i < off; i++) SMS->UDH.Text[i] = buffer[Layout.Text + i];
+		for (i = 0; i < off; i++) {
+			SMS->UDH.Text[i] = buffer[Layout.Text + i];
+		}
 
 		GSM_DecodeUDHHeader(di, &SMS->UDH);
 	}
@@ -365,19 +376,24 @@ GSM_Error GSM_DecodeSMSFrameStatusReportData(GSM_Debug_Info *di, GSM_SMSMessage
 GSM_Error GSM_DecodePDUFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, const unsigned char *buffer, size_t length, size_t *final_pos, gboolean SMSC)
 {
 	size_t pos = 0;
-	int type;
+	int type, tp_pi;
 	int vpf = 0;
 	int udh = 0;
 	int i,w;
 	unsigned char	output[161];
 	int			datalength;
+	gboolean have_data = FALSE;
+	GSM_Error error;
 
 	/* Set some sane data */
 	GSM_SetDefaultReceivedSMSData(SMS);
 
 	/* Parse SMSC if it is included */
 	if (SMSC) {
-		pos += GSM_UnpackSemiOctetNumber(di, SMS->SMSC.Number, buffer + pos, FALSE);
+		error = GSM_UnpackSemiOctetNumber(di, SMS->SMSC.Number, buffer, &pos, length, FALSE);
+		if (error != ERR_NONE) {
+			return error;
+		}
 		smfprintf(di, "SMS center number : \"%s\"\n",DecodeUnicodeString(SMS->SMSC.Number));
 	}
 
@@ -410,6 +426,9 @@ GSM_Error GSM_DecodePDUFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, const unsi
 			smfprintf(di, ", UDH included");
 			udh = 1;
 		}
+		if (type & (1 << 5)) {
+			smfprintf(di, ", Delivery report requested");
+		}
 	}
 	if (SMS->PDU == SMS_Submit) {
 		vpf = (type & (0x3 << 3)) >> 3;
@@ -442,7 +461,10 @@ GSM_Error GSM_DecodePDUFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, const unsi
 	}
 
 	/* Remote number */
-	pos += GSM_UnpackSemiOctetNumber(di, SMS->Number, buffer + pos, TRUE);
+	error = GSM_UnpackSemiOctetNumber(di, SMS->Number, buffer, &pos, length, TRUE);
+	if (error != ERR_NONE) {
+		return error;
+	}
 	smfprintf(di, "Remote number : \"%s\"\n",DecodeUnicodeString(SMS->Number));
 	if (pos >= length) {
 		smfprintf(di, "Ran out of buffer when parsing PDU!\n");
@@ -454,7 +476,7 @@ GSM_Error GSM_DecodePDUFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, const unsi
 		/* Protocol identifier */
 		smfprintf(di, "SMS PID: 0x%02X\n", buffer[pos]);
 		if (buffer[pos] > 0x40 && buffer[pos] < 0x48) {
-			SMS->ReplaceMessage = buffer[pos];
+			SMS->ReplaceMessage = buffer[pos] - 0x40;
 		}
 		pos++;
 		if (pos >= length) {
@@ -503,6 +525,37 @@ GSM_Error GSM_DecodePDUFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, const unsi
 		/* Status */
 		GSM_DecodeSMSStatusReportData(di, SMS, buffer[pos]);
 		pos++;
+
+		/* Optional part coming from ETSI 123 040, section 9.2.2.3 */
+		/* We ignore 0xff, because some buggy phones pad SMS with it */
+		if (pos < length && buffer[pos] != 0xff) {
+			/* TP-Parameter-Indicator */
+			tp_pi = buffer[pos];
+			pos++;
+
+			/* We have TP-PID */
+			if (tp_pi & 1) {
+				smfprintf(di, "SMS PID: 0x%02X\n", buffer[pos]);
+				if (buffer[pos] > 0x40 && buffer[pos] < 0x48) {
+					SMS->ReplaceMessage = buffer[pos] - 0x40;
+				}
+				pos++;
+			}
+
+			/* We have TP-DCS */
+			if (tp_pi & 2) {
+				smfprintf(di, "SMS DCS: 0x%02X\n", buffer[pos]);
+				SMS->Coding = GSM_GetMessageCoding(di, buffer[pos]);
+				pos++;
+			} else {
+				SMS->Coding = GSM_GetMessageCoding(di, 0);
+			}
+
+			/* We have TP-UDL */
+			if (tp_pi & 4) {
+				have_data = TRUE;
+			}
+		}
 	}
 
 	/* Validity period */
@@ -528,14 +581,21 @@ GSM_Error GSM_DecodePDUFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, const unsi
 	}
 
 	/* Data */
-	if (SMS->PDU == SMS_Submit || SMS->PDU == SMS_Deliver) {
+	if (SMS->PDU == SMS_Submit || SMS->PDU == SMS_Deliver || have_data) {
 		datalength = buffer[pos];
 		if (SMS->Coding == SMS_Coding_Default_No_Compression) {
 			datalength = (datalength * 7) / 8;
 			if ((buffer[pos] * 7) % 8 != 0) {
 				datalength++;
 			}
+		} else if (SMS->Coding == SMS_Coding_Unicode_Compression) {
+			smfprintf(di, "Compressed unicode messages not yet supported!\n");
+			return ERR_NOTSUPPORTED;
+		} else if (SMS->Coding == SMS_Coding_Default_Compression) {
+			smfprintf(di, "Compressed 7-bit messages not yet supported!\n");
+			return ERR_NOTSUPPORTED;
 		}
+
 		if (pos + datalength >= length) {
 			smfprintf(di, "Ran out of buffer when parsing PDU!\n");
 			return ERR_CORRUPTED;
@@ -607,6 +667,9 @@ GSM_Error GSM_DecodePDUFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, const unsi
 GSM_Error GSM_DecodeSMSFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, unsigned char *buffer, GSM_SMSMessageLayout Layout)
 {
 	GSM_DateTime	zerodt = {0,0,0,0,0,0,0};
+	size_t pos;
+	GSM_Error error;
+
 #ifdef DEBUG
 	if (Layout.firstbyte == 255) {
 		smfprintf(di, "ERROR: firstbyte in SMS layout not set\n");
@@ -625,7 +688,11 @@ GSM_Error GSM_DecodeSMSFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, unsigned c
 	GSM_SetDefaultReceivedSMSData(SMS);
 
 	if (Layout.SMSCNumber!=255) {
-		GSM_UnpackSemiOctetNumber(di, SMS->SMSC.Number,buffer+Layout.SMSCNumber,FALSE);
+		pos = Layout.SMSCNumber;
+		error = GSM_UnpackSemiOctetNumber(di, SMS->SMSC.Number, buffer, &pos, 1000, FALSE);
+		if (error != ERR_NONE) {
+			return error;
+		}
 		smfprintf(di, "SMS center number : \"%s\"\n",DecodeUnicodeString(SMS->SMSC.Number));
 	}
 	if ((buffer[Layout.firstbyte] & 0x80)!=0) SMS->ReplyViaSameSMSC=TRUE;
@@ -633,7 +700,11 @@ GSM_Error GSM_DecodeSMSFrame(GSM_Debug_Info *di, GSM_SMSMessage *SMS, unsigned c
 	if (SMS->ReplyViaSameSMSC) smfprintf(di, "SMS centre set for reply\n");
 #endif
 	if (Layout.Number!=255) {
-		GSM_UnpackSemiOctetNumber(di, SMS->Number,buffer+Layout.Number,TRUE);
+		pos = Layout.Number;
+		error = GSM_UnpackSemiOctetNumber(di, SMS->Number, buffer, &pos, 1000, TRUE);
+		if (error != ERR_NONE) {
+			return error;
+		}
 		smfprintf(di, "Remote number : \"%s\"\n",DecodeUnicodeString(SMS->Number));
 	}
 	if (Layout.TPDCS != 255) {
@@ -731,10 +802,8 @@ static int GSM_EncodeSMSFrameText(GSM_Debug_Info *di, GSM_SMSMessage *SMS, unsig
 		buffer[Layout.firstbyte] |= 0x40;			/* GSM 03.40 section 9.2.3.23 (TP-User-Data-Header-Indicator) */
 		off = 1 + SMS->UDH.Text[0];				/* off - length of the User Data Header */
 		memcpy(buffer+Layout.Text, SMS->UDH.Text, off);		/* we copy the udh */
-#ifdef DEBUG
 		smfprintf(di, "UDH, length %i\n",off);
 		DumpMessageText(di, SMS->UDH.Text, off);
-#endif
 	}
 	switch (SMS->Coding) {
 		case SMS_Coding_8bit:
@@ -742,12 +811,10 @@ static int GSM_EncodeSMSFrameText(GSM_Debug_Info *di, GSM_SMSMessage *SMS, unsig
 			/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme)
 			 * and GSM 03.38 section 4 */
 			buffer[Layout.TPDCS] |= (1 << 2);
-			memcpy(buffer+(Layout.Text+off), SMS->Text, SMS->Length);
+			memcpy(buffer+(Layout.Text+off), SMS->Text, MIN(SMS->Length, 140));
 			size2 = size = SMS->Length+off;
-#ifdef DEBUG
 			smfprintf(di, "8 bit SMS, length %i\n",SMS->Length);
 			DumpMessageText(di, SMS->Text, SMS->Length);
-#endif
 			break;
 		case SMS_Coding_Default_No_Compression:
 			p = 0;
@@ -755,7 +822,7 @@ static int GSM_EncodeSMSFrameText(GSM_Debug_Info *di, GSM_SMSMessage *SMS, unsig
 				p+=7;
 				w=(p-off)%p;
 			} while (w<0);
-			p = UnicodeLength(SMS->Text);
+			p = MIN(UnicodeLength(SMS->Text), 160);
 			EncodeDefault(buff, SMS->Text, &p, TRUE, NULL);
 			size = GSM_PackSevenBitsToEight(w, buff, buffer+(Layout.Text+off), p);
 			size += off;
@@ -770,13 +837,11 @@ static int GSM_EncodeSMSFrameText(GSM_Debug_Info *di, GSM_SMSMessage *SMS, unsig
 			/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme)
 			 * and GSM 03.38 section 4 */
 			buffer[Layout.TPDCS] |= (1 << 3);
-			EncodeUnicodeSpecialNOKIAChars(buffer+(Layout.Text+off), SMS->Text, UnicodeLength(SMS->Text));
+			EncodeUnicodeSpecialNOKIAChars(buffer+(Layout.Text+off), SMS->Text, MIN(UnicodeLength(SMS->Text), 70));
 			size=size2=UnicodeLength(buffer+(Layout.Text+off))*2+off;
-#ifdef DEBUG
 			smfprintf(di, "Unicode SMS, length %i\n",(size2-off)/2);
 			DumpMessageText(di, buffer+(Layout.Text+off), size2-off);
 			smfprintf(di, "%s\n",DecodeUnicodeString(buffer+(Layout.Text+off)));
-#endif
 			break;
 		default:
 			break;
@@ -1027,6 +1092,47 @@ gboolean GSM_DecodeSiemensOTASMS(GSM_Debug_Info *di, GSM_SiemensOTASMSInfo	*Info
 	return TRUE;
 }
 
+GSM_Coding_Type GSM_StringToSMSCoding(const char *s)
+{
+	/* Maintain those without compression for backward compatibility */
+	if (s == NULL)
+		return SMS_Coding_Default_No_Compression;
+	else if (strcmp("Unicode", s) == 0)
+		return SMS_Coding_Unicode_No_Compression;
+	else if (strcmp("Unicode_No_Compression", s) == 0)
+		return SMS_Coding_Unicode_No_Compression;
+	else if (strcmp("Unicode_Compression", s) == 0)
+		return SMS_Coding_Unicode_Compression;
+	else if (strcmp("Default", s) == 0)
+		return SMS_Coding_Default_No_Compression;
+	else if (strcmp("Default_No_Compression", s) == 0)
+		return SMS_Coding_Default_No_Compression;
+	else if (strcmp("Default_Compression", s) == 0)
+		return SMS_Coding_Default_Compression;
+	else if (strcmp("8bit", s) == 0)
+		return SMS_Coding_8bit;
+
+	return 0;
+}
+
+const char *GSM_SMSCodingToString(GSM_Coding_Type type)
+{
+	switch (type) {
+		case SMS_Coding_Unicode_No_Compression:
+			return "Unicode_No_Compression";
+		case SMS_Coding_Unicode_Compression:
+			return "Unicode_Compression";
+		case SMS_Coding_Default_No_Compression:
+			return "Default_No_Compression";
+		case SMS_Coding_Default_Compression:
+			return "Default_Compression";
+		case SMS_Coding_8bit:
+			return "8bit";
+	}
+
+	return NULL;
+}
+
 /* How should editor hadle tabs in this file? Add editor commands here.
  * vim: noexpandtab sw=8 ts=8 sts=8:
  */
