@@ -24,13 +24,26 @@
 #include "convertors.h"
 #include "misc.h"
 
+const char *LocationToString(GSM_EntryLocation location)
+{
+    switch (location) {
+        case PBK_Location_Home:
+            return "Home";
+        case PBK_Location_Work:
+            return "Work";
+        case PBK_Location_Unknown:
+            return "Unknown";
+    }
+    return "Unknown";
+}
+
 #define convert_text(name) \
                 s = strGammuToPython(entry->Entries[i].Text); \
                 if (s == NULL) { \
                     Py_DECREF(v); \
                     return NULL; \
                 } \
-                f = Py_BuildValue("{s:s,s:u,s:i}", "Type", name, "Value", s, "AddError", (int)entry->Entries[i].AddError); \
+                f = Py_BuildValue("{s:s,s:u,s:i,s:s}", "Type", name, "Value", s, "AddError", (int)entry->Entries[i].AddError, "Location", LocationToString(entry->Entries[i].Location)); \
                 free(s);
 
 #define convert_number(name) \
@@ -74,21 +87,9 @@ PyObject *MemoryEntryToPython(const GSM_MemoryEntry * entry)
 			case PBK_Number_Mobile:
 				convert_number("Mobile");
 				break;
-			case PBK_Number_Mobile_Work:
-				convert_number("Mobile_Work");
-				break;
-			case PBK_Number_Mobile_Home:
-				convert_number("Mobile_Home");
-				break;
-			case PBK_Number_Work:
-				convert_number("Work");
-				break;
 			case PBK_Number_Fax:
 				convert_number("Fax");
 				break;
-			case PBK_Number_Home:
-				convert_number("Home");
-				break;
 			case PBK_Number_Pager:
 				convert_number("Pager");
 				break;
@@ -101,9 +102,6 @@ PyObject *MemoryEntryToPython(const GSM_MemoryEntry * entry)
 			case PBK_Text_Postal:
 				convert_text("Text_Postal");
 				break;
-			case PBK_Text_WorkPostal:
-				convert_text("Text_WorkPostal");
-				break;
 			case PBK_Text_Email:
 				convert_text("Text_Email");
 				break;
@@ -151,12 +149,21 @@ PyObject *MemoryEntryToPython(const GSM_MemoryEntry * entry)
 			case PBK_Text_FirstName:
 				convert_text("Text_FirstName");
 				break;
+			case PBK_Text_SecondName:
+				convert_text("Text_SecondName");
+				break;
 			case PBK_Text_NickName:
 				convert_text("Text_NickName");
 				break;
 			case PBK_Text_FormalName:
 				convert_text("Text_FormalName");
 				break;
+			case PBK_Text_NamePrefix:
+				convert_text("Text_NamePrefix");
+				break;
+			case PBK_Text_NameSuffix:
+				convert_text("Text_NameSuffix");
+				break;
 			case PBK_Text_Company:
 				convert_text("Text_Company");
 				break;
@@ -193,21 +200,6 @@ PyObject *MemoryEntryToPython(const GSM_MemoryEntry * entry)
 			case PBK_Text_Country:
 				convert_text("Text_Country");
 				break;
-			case PBK_Text_WorkStreetAddress:
-				convert_text("Text_WorkStreetAddress");
-				break;
-			case PBK_Text_WorkCity:
-				convert_text("Text_WorkCity");
-				break;
-			case PBK_Text_WorkState:
-				convert_text("Text_WorkState");
-				break;
-			case PBK_Text_WorkZip:
-				convert_text("Text_WorkZip");
-				break;
-			case PBK_Text_WorkCountry:
-				convert_text("Text_WorkCountry");
-				break;
 			case PBK_Text_Custom1:
 				convert_text("Text_Custom1");
 				break;
@@ -223,6 +215,21 @@ PyObject *MemoryEntryToPython(const GSM_MemoryEntry * entry)
 			case PBK_Text_LUID:
 				convert_text("Text_LUID");
 				break;
+			case PBK_Text_VOIP:
+				convert_text("Text_VOIP");
+				break;
+			case PBK_Text_SWIS:
+				convert_text("Text_SWIS");
+				break;
+			case PBK_Text_WVID:
+				convert_text("Text_WVID");
+				break;
+			case PBK_Text_SIP:
+				convert_text("Text_SIP");
+				break;
+			case PBK_Text_DTMF:
+				convert_text("Text_DTMF");
+				break;
 			case PBK_Text_UserID:
 				convert_text("Text_UserID");
 				break;
@@ -249,6 +256,9 @@ PyObject *MemoryEntryToPython(const GSM_MemoryEntry * entry)
 			case PBK_Number_Messaging:
 				convert_number("Messaging");
 				break;
+			case PBK_Number_Video:
+				convert_number("Video");
+				break;
 			case PBK_PushToTalkID:
 				convert_text("PushToTalkID");
 				break;
@@ -329,7 +339,7 @@ int MemoryEntryFromPython(PyObject * dict, GSM_MemoryEntry * entry,
 	PyObject *item;
 	Py_ssize_t len, data_len;
 	Py_ssize_t i;
-	char *type;
+	char *type, *location;
 	char valuetype;
 	const char *bmptype;
 
@@ -392,6 +402,19 @@ int MemoryEntryFromPython(PyObject * dict, GSM_MemoryEntry * entry,
 		if (type == NULL)
 			return 0;
 
+		location = GetCharFromDict(item, "Location");
+		if (location == NULL) {
+            entry->Entries[i].Location = PBK_Location_Unknown;
+        } else {
+            if (strcmp(location, "Home") == 0) {
+                entry->Entries[i].Location = PBK_Location_Home;
+            } else if (strcmp(location, "Work") == 0) {
+                entry->Entries[i].Location = PBK_Location_Work;
+            } else {
+                entry->Entries[i].Location = PBK_Location_Unknown;
+            }
+        }
+
 		/* Zero everything, just to be sure */
 		entry->Entries[i].Text[0] = 0;
 		entry->Entries[i].Text[1] = 0;
@@ -417,22 +440,26 @@ int MemoryEntryFromPython(PyObject * dict, GSM_MemoryEntry * entry,
 			entry->Entries[i].EntryType = PBK_Number_General;
 		} else if (strcmp("Number_Mobile_Work", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType = PBK_Number_Mobile_Work;
+			entry->Entries[i].EntryType = PBK_Number_Mobile;
+            entry->Entries[i].Location = PBK_Location_Work;
 		} else if (strcmp("Number_Mobile_Home", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType = PBK_Number_Mobile_Home;
+			entry->Entries[i].EntryType = PBK_Number_Mobile;
+            entry->Entries[i].Location = PBK_Location_Home;
 		} else if (strcmp("Number_Mobile", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Number_Mobile;
 		} else if (strcmp("Number_Work", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType = PBK_Number_Work;
+			entry->Entries[i].EntryType = PBK_Number_General;
+            entry->Entries[i].Location = PBK_Location_Work;
 		} else if (strcmp("Number_Fax", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Number_Fax;
 		} else if (strcmp("Number_Home", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType = PBK_Number_Home;
+			entry->Entries[i].EntryType = PBK_Number_General;
+            entry->Entries[i].Location = PBK_Location_Home;
 		} else if (strcmp("Number_Pager", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Number_Pager;
@@ -442,6 +469,9 @@ int MemoryEntryFromPython(PyObject * dict, GSM_MemoryEntry * entry,
 		} else if (strcmp("Number_Messaging", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Number_Messaging;
+		} else if (strcmp("Number_Video", type) == 0) {
+			valuetype = 't';
+			entry->Entries[i].EntryType = PBK_Number_Video;
 		} else if (strcmp("Text_Note", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Text_Note;
@@ -450,7 +480,8 @@ int MemoryEntryFromPython(PyObject * dict, GSM_MemoryEntry * entry,
 			entry->Entries[i].EntryType = PBK_Text_Postal;
 		} else if (strcmp("Text_WorkPostal", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType = PBK_Text_WorkPostal;
+			entry->Entries[i].EntryType = PBK_Text_Postal;
+            entry->Entries[i].Location = PBK_Location_Work;
 		} else if (strcmp("Text_Email", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Text_Email;
@@ -478,12 +509,21 @@ int MemoryEntryFromPython(PyObject * dict, GSM_MemoryEntry * entry,
 		} else if (strcmp("Text_FirstName", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Text_FirstName;
+		} else if (strcmp("Text_SecondName", type) == 0) {
+			valuetype = 't';
+			entry->Entries[i].EntryType = PBK_Text_SecondName;
 		} else if (strcmp("Text_NickName", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Text_NickName;
 		} else if (strcmp("Text_FormalName", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Text_FormalName;
+		} else if (strcmp("Text_NameSuffix", type) == 0) {
+			valuetype = 't';
+			entry->Entries[i].EntryType = PBK_Text_NameSuffix;
+		} else if (strcmp("Text_NamePrefix", type) == 0) {
+			valuetype = 't';
+			entry->Entries[i].EntryType = PBK_Text_NamePrefix;
 		} else if (strcmp("Text_Company", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Text_Company;
@@ -513,20 +553,24 @@ int MemoryEntryFromPython(PyObject * dict, GSM_MemoryEntry * entry,
 			entry->Entries[i].EntryType = PBK_Text_Country;
 		} else if (strcmp("Text_WorkStreetAddress", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType =
-			    PBK_Text_WorkStreetAddress;
+			entry->Entries[i].EntryType = PBK_Text_StreetAddress;
+            entry->Entries[i].Location = PBK_Location_Work;
 		} else if (strcmp("Text_WorkCity", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType = PBK_Text_WorkCity;
+			entry->Entries[i].EntryType = PBK_Text_City;
+            entry->Entries[i].Location = PBK_Location_Work;
 		} else if (strcmp("Text_WorkState", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType = PBK_Text_WorkState;
+			entry->Entries[i].EntryType = PBK_Text_State;
+            entry->Entries[i].Location = PBK_Location_Work;
 		} else if (strcmp("Text_WorkZip", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType = PBK_Text_WorkZip;
+			entry->Entries[i].EntryType = PBK_Text_Zip;
+            entry->Entries[i].Location = PBK_Location_Work;
 		} else if (strcmp("Text_WorkCountry", type) == 0) {
 			valuetype = 't';
-			entry->Entries[i].EntryType = PBK_Text_WorkCountry;
+			entry->Entries[i].EntryType = PBK_Text_Country;
+            entry->Entries[i].Location = PBK_Location_Work;
 		} else if (strcmp("Text_Custom1", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Text_Custom1;
@@ -542,6 +586,21 @@ int MemoryEntryFromPython(PyObject * dict, GSM_MemoryEntry * entry,
 		} else if (strcmp("Text_LUID", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Text_LUID;
+		} else if (strcmp("Text_VOIP", type) == 0) {
+			valuetype = 't';
+			entry->Entries[i].EntryType = PBK_Text_VOIP;
+		} else if (strcmp("Text_SWIS", type) == 0) {
+			valuetype = 't';
+			entry->Entries[i].EntryType = PBK_Text_SWIS;
+		} else if (strcmp("Text_WVID", type) == 0) {
+			valuetype = 't';
+			entry->Entries[i].EntryType = PBK_Text_WVID;
+		} else if (strcmp("Text_SIP", type) == 0) {
+			valuetype = 't';
+			entry->Entries[i].EntryType = PBK_Text_SIP;
+		} else if (strcmp("Text_DTMF", type) == 0) {
+			valuetype = 't';
+			entry->Entries[i].EntryType = PBK_Text_DTMF;
 		} else if (strcmp("Text_UserID", type) == 0) {
 			valuetype = 't';
 			entry->Entries[i].EntryType = PBK_Text_UserID;
