@@ -272,6 +272,7 @@ int SMSCFromPython(PyObject * dict, GSM_SMSC * smsc, gboolean complete)
 	if (!complete) {
 		smsc->Location = GetIntFromDict(dict, "Location");
 		if (smsc->Location == INT_INVALID) {
+			smsc->Location = 0;
 			PyErr_Clear();
 
 			if (!CopyStringFromDict
@@ -522,56 +523,29 @@ char *SMSTypeToString(GSM_SMSMessageType type)
 
 GSM_Coding_Type StringToSMSCoding(const char *s)
 {
-	/* Maintain those without compression for backward compatibility */
-	if (strcmp("Unicode", s) == 0)
-		return SMS_Coding_Unicode_No_Compression;
-	else if (strcmp("Unicode_No_Compression", s) == 0)
-		return SMS_Coding_Unicode_No_Compression;
-	else if (strcmp("Unicode_Compression", s) == 0)
-		return SMS_Coding_Unicode_Compression;
-	else if (strcmp("Default", s) == 0)
-		return SMS_Coding_Default_No_Compression;
-	else if (strcmp("Default_No_Compression", s) == 0)
-		return SMS_Coding_Default_No_Compression;
-	else if (strcmp("Default_Compression", s) == 0)
-		return SMS_Coding_Default_Compression;
-	else if (strcmp("8bit", s) == 0)
-		return SMS_Coding_8bit;
+	GSM_Coding_Type t;
 
-	PyErr_Format(PyExc_ValueError, "Bad value for SMSCoding: '%s'", s);
-	return 0;
+	t = GSM_StringToSMSCoding(s);
+
+	if (t == 0) {
+		PyErr_Format(PyExc_ValueError, "Bad value for SMSCoding: '%s'", s);
+	}
+
+	return t;
 }
 
 char *SMSCodingToString(GSM_Coding_Type type)
 {
-	char *s = NULL;
+	const char *s;
 
-	switch (type) {
-		case SMS_Coding_Unicode_No_Compression:
-			s = strdup("Unicode_No_Compression");
-			break;
-		case SMS_Coding_Unicode_Compression:
-			s = strdup("Unicode_Compression");
-			break;
-		case SMS_Coding_Default_No_Compression:
-			s = strdup("Default_No_Compression");
-			break;
-		case SMS_Coding_Default_Compression:
-			s = strdup("Default_Compression");
-			break;
-		case SMS_Coding_8bit:
-			s = strdup("8bit");
-			break;
-	}
+	s = GSM_SMSCodingToString(type);
 
 	if (s == NULL) {
-		PyErr_Format(PyExc_ValueError,
-			     "Bad value for Coding_Type from Gammu: '%d'",
-			     type);
+		PyErr_Format(PyExc_ValueError, "Bad value for Coding_Type from Gammu: '%d'", type);
 		return NULL;
 	}
 
-	return s;
+	return strdup(s);
 }
 
 GSM_SMS_State StringToSMSState(const char *s)
@@ -765,9 +739,17 @@ int SMSFromPython(PyObject * dict, GSM_SMSMessage * sms, int needslocation,
 		}
 	}
 
-	if (sms->UDH.Type == UDH_NoUDH || sms->UDH.Type == UDH_UserUDH ||
-	    sms->UDH.Type == UDH_ConcatenatedMessages
-	    || sms->UDH.Type == UDH_ConcatenatedMessages16bit) {
+	s = GetCharFromDict(dict, "Coding");
+	if (s == NULL) {
+		sms->Coding = SMS_Coding_Default_No_Compression;
+		PyErr_Clear();
+	} else {
+		sms->Coding = StringToSMSCoding(s);
+		if (sms->Coding == 0)
+			return 0;
+	}
+
+	if (sms->Coding != SMS_Coding_8bit) {
 		/* No UDH/UserUDH => copy as text */
 		if (!CopyStringFromDict
 		    (dict, "Text", GSM_MAX_SMS_LENGTH, sms->Text))
@@ -866,16 +848,6 @@ int SMSFromPython(PyObject * dict, GSM_SMSMessage * sms, int needslocation,
 			return 0;
 	}
 
-	s = GetCharFromDict(dict, "Coding");
-	if (s == NULL) {
-		sms->Coding = SMS_Coding_Default_No_Compression;
-		PyErr_Clear();
-	} else {
-		sms->Coding = StringToSMSCoding(s);
-		if (sms->Coding == 0)
-			return 0;
-	}
-
 	sms->DateTime = GetDateTimeFromDict(dict, "DateTime");
 	if (sms->DateTime.Year == -1) {
 		sms->DateTime = nulldt;
@@ -955,9 +927,7 @@ PyObject *SMSToPython(GSM_SMSMessage * sms)
 		return NULL;
 	}
 
-	if (sms->UDH.Type == UDH_NoUDH || sms->UDH.Type == UDH_UserUDH ||
-	    sms->UDH.Type == UDH_ConcatenatedMessages
-	    || sms->UDH.Type == UDH_ConcatenatedMessages16bit) {
+	if (sms->Coding != SMS_Coding_8bit) {
 		/* No UDH/User UDH => copy as text */
 		text = UnicodeStringToPythonL(sms->Text, sms->Length);
 	} else {
@@ -986,7 +956,7 @@ PyObject *SMSToPython(GSM_SMSMessage * sms)
 	}
 
 	coding = SMSCodingToString(sms->Coding);
-	if (type == NULL) {
+	if (coding == NULL) {
 		Py_DECREF(smsc);
 		Py_DECREF(udh);
 		free(mt);
