@@ -1,7 +1,7 @@
 /**
  * Main SMSD program
  */
-/* Copyright (c) 2009 Michal Cihar <michal@cihar.com> */
+/* Copyright (c) 2009 - 2011 Michal Cihar <michal@cihar.com> */
 /* Licensend under GNU GPL 2 */
 
 #include <gammu-smsd.h>
@@ -51,7 +51,7 @@ void smsd_reconfigure(int signum)
 
 NORETURN void version(void)
 {
-	printf("Gammu-smsd version %s\n", VERSION);
+	printf("Gammu-smsd version %s\n", GAMMU_VERSION);
 	printf("Built %s on %s using %s\n", __TIME__, __DATE__, GetCompiler());
 	printf("Compiled in features:\n");
 	printf("OS support:\n");
@@ -95,9 +95,12 @@ NORETURN void version(void)
 #endif
 #ifdef LIBDBI_FOUND
 	printf("  - %s\n", "DBI");
+#endif
+#ifdef ODBC_FOUND
+	printf("  - %s\n", "ODBC");
 #endif
 	printf("\n");
-	printf("Copyright (C) 2003 - 2010 Michal Cihar <michal@cihar.com> and other authors.\n");
+	printf("Copyright (C) 2003 - 2011 Michal Cihar <michal@cihar.com> and other authors.\n");
 	printf("\n");
 	printf("License GPLv2: GNU GPL version 2 <http://creativecommons.org/licenses/GPL/2.0/>.\n");
 	printf("This is free software: you are free to change and redistribute it.\n");
