@@ -12,7 +12,7 @@
 
 #include "common.h"
 
-extern GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message msg, GSM_StateMachine * s);
+extern GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine * s);
 
 #define BUFFER_SIZE 16384
 
@@ -89,13 +89,14 @@ int main(int argc, char **argv)
 	s->Phone.Data.GetSMSMessage = &sms;
 
 	/* Parse it */
-	error = ATGEN_ReplyGetSMSMessage(msg, s);
+	error = ATGEN_ReplyGetSMSMessage(&msg, s);
 	sms.SMS[0].Memory = MEM_SM;
 
 	/* Display message */
 	if (error == ERR_NONE) {
 		DisplayMultiSMSInfo(&sms, FALSE, TRUE, NULL, NULL);
 		DisplayMultiSMSInfo(&sms, TRUE, TRUE, NULL, NULL);
+		printf("Parts: %d, count: %d, ID16: %d, ID8: %d\n", sms.SMS[0].UDH.AllParts, sms.Number, sms.SMS[0].UDH.ID16bit, sms.SMS[0].UDH.ID8bit);
 	}
 
 	/* This is normally done by ATGEN_Terminate */
